searchData={"items":[{"type":"module","title":"LangChain.NativeTool","doc":"Represents built-in tools available from AI/LLM services that can be used within the LangChain framework.\n\nNative tools are functionalities provided directly by language model services (like Google AI, OpenAI)\nthat can be invoked by LLMs to perform specific actions or retrieve information. They are\n\"native\" because they're built into the AI service itself, rather than implemented externally.\n\nEach native tool has:\n- A unique name that identifies it (e.g., \"google_search\", \"code_interpreter\")\n- A configuration map that contains tool-specific settings","ref":"LangChain.NativeTool.html"},{"type":"module","title":"Google Search Grounding - LangChain.NativeTool","doc":"Starting with Gemini 2.0, Google Search is available as a native tool. This enables the model\nto decide when to use Google Search to enhance the factuality and recency of responses.\n\nGoogle Search as a tool enables:\n- More accurate and up-to-date answers with grounding sources\n- Retrieving information from the web for further analysis\n- Finding relevant images, videos, or other media for multimodal reasoning\n- Supporting specialized tasks like coding, technical troubleshooting, etc.","ref":"LangChain.NativeTool.html#module-google-search-grounding"},{"type":"module","title":"Example with Google Search - LangChain.NativeTool","doc":"```elixir\nalias LangChain.Chains.LLMChain\nalias LangChain.Message\nalias LangChain.NativeTool\n\nmodel = ChatGoogleAI.new!(%{temperature: 0, stream: false, model: \"gemini-2.0-flash\"})\n\n{:ok, updated_chain} =\n   %{llm: model, verbose: false, stream: false}\n   |> LLMChain.new!()\n   |> LLMChain.add_message(\n     Message.new_user!(\"What is the current Google stock price?\")\n   )\n   |> LLMChain.add_tools(NativeTool.new!(%{name: \"google_search\", configuration: %{}}))\n   |> LLMChain.run()\n```","ref":"LangChain.NativeTool.html#module-example-with-google-search"},{"type":"function","title":"LangChain.NativeTool.new/1","doc":"Build a new native tool.","ref":"LangChain.NativeTool.html#new/1"},{"type":"function","title":"LangChain.NativeTool.new!/1","doc":"Build a new native tool and return it or raise an error if invalid.","ref":"LangChain.NativeTool.html#new!/1"},{"type":"type","title":"LangChain.NativeTool.configuration/0","doc":"","ref":"LangChain.NativeTool.html#t:configuration/0"},{"type":"type","title":"LangChain.NativeTool.t/0","doc":"","ref":"LangChain.NativeTool.html#t:t/0"},{"type":"module","title":"LangChain.Telemetry","doc":"Telemetry events for LangChain.\n\nThis module defines telemetry events that other applications can attach to.\nIt provides a standardized way to emit events for various operations in the\nLangChain library without implementing tracing functionality.","ref":"LangChain.Telemetry.html"},{"type":"module","title":"Event Naming - LangChain.Telemetry","doc":"Events follow the convention: `[:langchain, component, operation, stage]`","ref":"LangChain.Telemetry.html#module-event-naming"},{"type":"module","title":"Core Events - LangChain.Telemetry","doc":"* `[:langchain, :llm, :call, :start]` - Emitted when an LLM call starts\n* `[:langchain, :llm, :call, :stop]` - Emitted when an LLM call completes\n* `[:langchain, :llm, :call, :exception]` - Emitted when an LLM call raises an exception\n* `[:langchain, :llm, :prompt]` - Emitted when a prompt is sent to an LLM\n* `[:langchain, :llm, :response]` - Emitted when a response is received from an LLM\n* `[:langchain, :llm, :response, :non_streaming]` - Emitted when a non-streaming response is received from an LLM\n* `[:langchain, :llm, :response, :streaming]` - Emitted when a streaming response is received from an LLM\n* `[:langchain, :chain, :execute, :start]` - Emitted when a chain execution starts\n* `[:langchain, :chain, :execute, :stop]` - Emitted when a chain execution completes\n* `[:langchain, :chain, :execute, :exception]` - Emitted when a chain execution raises an exception\n* `[:langchain, :message, :process, :start]` - Emitted when message processing starts\n* `[:langchain, :message, :process, :stop]` - Emitted when message processing completes\n* `[:langchain, :message, :process, :exception]` - Emitted when message processing raises an exception\n* `[:langchain, :tool, :call, :start]` - Emitted when a tool call starts\n* `[:langchain, :tool, :call, :stop]` - Emitted when a tool call completes\n* `[:langchain, :tool, :call, :exception]` - Emitted when a tool call raises an exception","ref":"LangChain.Telemetry.html#module-core-events"},{"type":"module","title":"Usage - LangChain.Telemetry","doc":"To attach to these events in your application:\n\n```elixir\n:telemetry.attach(\n  \"my-handler-id\",\n  [:langchain, :llm, :call, :stop],\n  &MyApp.handle_llm_call/4,\n  nil\n)\n\ndef handle_llm_call(_event_name, measurements, metadata, _config) do\n  # Process the event\n  IO.inspect(measurements)\n  IO.inspect(metadata)\nend\n```","ref":"LangChain.Telemetry.html#module-usage"},{"type":"function","title":"LangChain.Telemetry.chain_execute_start/1","doc":"Emits a chain execution start event.","ref":"LangChain.Telemetry.html#chain_execute_start/1"},{"type":"function","title":"LangChain.Telemetry.emit_event/3","doc":"Emits a telemetry event with the given name, measurements, and metadata.","ref":"LangChain.Telemetry.html#emit_event/3"},{"type":"function","title":"Parameters - LangChain.Telemetry.emit_event/3","doc":"* `event_name` - The name of the event as a list of atoms\n  * `measurements` - A map of measurements for the event\n  * `metadata` - A map of metadata for the event","ref":"LangChain.Telemetry.html#emit_event/3-parameters"},{"type":"function","title":"Examples - LangChain.Telemetry.emit_event/3","doc":"iex> LangChain.Telemetry.emit_event([:langchain, :llm, :call, :start], %{system_time: System.system_time()}, %{model: \"gpt-4\"})","ref":"LangChain.Telemetry.html#emit_event/3-examples"},{"type":"function","title":"LangChain.Telemetry.llm_call_start/1","doc":"Emits an LLM call start event.","ref":"LangChain.Telemetry.html#llm_call_start/1"},{"type":"function","title":"LangChain.Telemetry.llm_prompt/2","doc":"Emits an LLM prompt event.","ref":"LangChain.Telemetry.html#llm_prompt/2"},{"type":"function","title":"LangChain.Telemetry.llm_response/2","doc":"Emits an LLM response event.","ref":"LangChain.Telemetry.html#llm_response/2"},{"type":"function","title":"LangChain.Telemetry.memory_read_start/1","doc":"Emits a memory read start event.","ref":"LangChain.Telemetry.html#memory_read_start/1"},{"type":"function","title":"LangChain.Telemetry.memory_write_start/1","doc":"Emits a memory write start event.","ref":"LangChain.Telemetry.html#memory_write_start/1"},{"type":"function","title":"LangChain.Telemetry.message_process_start/1","doc":"Emits a message processing start event.","ref":"LangChain.Telemetry.html#message_process_start/1"},{"type":"function","title":"LangChain.Telemetry.retriever_get_relevant_documents_start/1","doc":"Emits a retriever get relevant documents start event.","ref":"LangChain.Telemetry.html#retriever_get_relevant_documents_start/1"},{"type":"function","title":"LangChain.Telemetry.span/3","doc":"Wraps a function call with start and stop telemetry events.","ref":"LangChain.Telemetry.html#span/3"},{"type":"function","title":"Parameters - LangChain.Telemetry.span/3","doc":"* `event_prefix` - The prefix for the event name as a list of atoms\n  * `metadata` - A map of metadata for the event\n  * `fun` - The function to execute","ref":"LangChain.Telemetry.html#span/3-parameters"},{"type":"function","title":"Returns - LangChain.Telemetry.span/3","doc":"The result of the function call.","ref":"LangChain.Telemetry.html#span/3-returns"},{"type":"function","title":"Examples - LangChain.Telemetry.span/3","doc":"iex> LangChain.Telemetry.span([:langchain, :llm, :call], %{model: \"gpt-4\"}, fn ->\n    ...>   # Call the LLM\n    ...>   {:ok, \"response\"}\n    ...> end)","ref":"LangChain.Telemetry.html#span/3-examples"},{"type":"function","title":"LangChain.Telemetry.start_event/2","doc":"Emits a start event and returns a function to emit the corresponding stop event.\n\nThis is useful for span-like events where you want to measure the duration of an operation.","ref":"LangChain.Telemetry.html#start_event/2"},{"type":"function","title":"Parameters - LangChain.Telemetry.start_event/2","doc":"* `event_prefix` - The prefix for the event name as a list of atoms\n  * `metadata` - A map of metadata for the event","ref":"LangChain.Telemetry.html#start_event/2-parameters"},{"type":"function","title":"Returns - LangChain.Telemetry.start_event/2","doc":"A function that accepts additional metadata to be merged with the original metadata\n  and emits the stop event with the duration measurement.","ref":"LangChain.Telemetry.html#start_event/2-returns"},{"type":"function","title":"Examples - LangChain.Telemetry.start_event/2","doc":"iex> stop_fun = LangChain.Telemetry.start_event([:langchain, :llm, :call], %{model: \"gpt-4\"})\n    iex> # Do some work\n    iex> stop_fun.(%{result: \"success\"})","ref":"LangChain.Telemetry.html#start_event/2-examples"},{"type":"function","title":"LangChain.Telemetry.tool_call/2","doc":"Emits a tool call event.","ref":"LangChain.Telemetry.html#tool_call/2"},{"type":"function","title":"LangChain.Telemetry.tool_call_start/1","doc":"Emits a tool call start event.","ref":"LangChain.Telemetry.html#tool_call_start/1"},{"type":"module","title":"LangChain.Utils.AwsEventstreamDecoder","doc":"Decodes AWS messages in the application/vnd.amazon.eventstream content-type.\nIgnores the headers because on Bedrock it's the same content type, event type & message type headers in every message.","ref":"LangChain.Utils.AwsEventstreamDecoder.html"},{"type":"function","title":"LangChain.Utils.AwsEventstreamDecoder.decode/1","doc":"","ref":"LangChain.Utils.AwsEventstreamDecoder.html#decode/1"},{"type":"module","title":"LangChain.Utils.BedrockStreamDecoder","doc":"","ref":"LangChain.Utils.BedrockStreamDecoder.html"},{"type":"function","title":"LangChain.Utils.BedrockStreamDecoder.decode_stream/2","doc":"","ref":"LangChain.Utils.BedrockStreamDecoder.html#decode_stream/2"},{"type":"module","title":"LangChain.Utils.Parser.LLAMA_3_1_CustomToolParser","doc":"","ref":"LangChain.Utils.Parser.LLAMA_3_1_CustomToolParser.html"},{"type":"function","title":"LangChain.Utils.Parser.LLAMA_3_1_CustomToolParser.parse/1","doc":"","ref":"LangChain.Utils.Parser.LLAMA_3_1_CustomToolParser.html#parse/1"},{"type":"function","title":"LangChain.Utils.Parser.LLAMA_3_1_CustomToolParser.parse_function_call/2","doc":"Parses the given `binary` as parse_function_call.\n\nReturns `{:ok, [token], rest, context, position, byte_offset}` or\n`{:error, reason, rest, context, line, byte_offset}` where `position`\ndescribes the location of the parse_function_call (start position) as `{line, offset_to_start_of_line}`.\n\nTo column where the error occurred can be inferred from `byte_offset - offset_to_start_of_line`.","ref":"LangChain.Utils.Parser.LLAMA_3_1_CustomToolParser.html#parse_function_call/2"},{"type":"function","title":"Options - LangChain.Utils.Parser.LLAMA_3_1_CustomToolParser.parse_function_call/2","doc":"* `:byte_offset` - the byte offset for the whole binary, defaults to 0\n  * `:line` - the line and the byte offset into that line, defaults to `{1, byte_offset}`\n  * `:context` - the initial context value. It will be converted to a map","ref":"LangChain.Utils.Parser.LLAMA_3_1_CustomToolParser.html#parse_function_call/2-options"},{"type":"module","title":"LangChain.Utils.Parser.LLAMA_3_2_CustomToolParser","doc":"","ref":"LangChain.Utils.Parser.LLAMA_3_2_CustomToolParser.html"},{"type":"function","title":"LangChain.Utils.Parser.LLAMA_3_2_CustomToolParser.parse/1","doc":"","ref":"LangChain.Utils.Parser.LLAMA_3_2_CustomToolParser.html#parse/1"},{"type":"function","title":"LangChain.Utils.Parser.LLAMA_3_2_CustomToolParser.parse_function_calls/2","doc":"Parses the given `binary` as parse_function_calls.\n\nReturns `{:ok, [token], rest, context, position, byte_offset}` or\n`{:error, reason, rest, context, line, byte_offset}` where `position`\ndescribes the location of the parse_function_calls (start position) as `{line, offset_to_start_of_line}`.\n\nTo column where the error occurred can be inferred from `byte_offset - offset_to_start_of_line`.","ref":"LangChain.Utils.Parser.LLAMA_3_2_CustomToolParser.html#parse_function_calls/2"},{"type":"function","title":"Options - LangChain.Utils.Parser.LLAMA_3_2_CustomToolParser.parse_function_calls/2","doc":"* `:byte_offset` - the byte offset for the whole binary, defaults to 0\n  * `:line` - the line and the byte offset into that line, defaults to `{1, byte_offset}`\n  * `:context` - the initial context value. It will be converted to a map","ref":"LangChain.Utils.Parser.LLAMA_3_2_CustomToolParser.html#parse_function_calls/2-options"},{"type":"module","title":"LangChain.ChatModels.ChatAnthropic","doc":"Module for interacting with [Anthropic models](https://docs.anthropic.com/claude/docs/models-overview#claude-3-a-new-generation-of-ai).\n\nParses and validates inputs for making requests to [Anthropic's messages API](https://docs.anthropic.com/claude/reference/messages_post).\n\nConverts responses into more specialized `LangChain` data structures.","ref":"LangChain.ChatModels.ChatAnthropic.html"},{"type":"module","title":"Callbacks - LangChain.ChatModels.ChatAnthropic","doc":"See the set of available callbacks: `LangChain.Chains.ChainCallbacks`","ref":"LangChain.ChatModels.ChatAnthropic.html#module-callbacks"},{"type":"module","title":"Rate Limit API Response Headers - LangChain.ChatModels.ChatAnthropic","doc":"Anthropic returns rate limit information in the response headers. Those can be\naccessed using an LLM callback like this:\n\n    handler = %{\n      on_llm_ratelimit_info: fn _chain, headers ->\n        IO.inspect(headers)\n      end\n    }\n\n    %{llm: ChatAnthropic.new!(%{model: \"...\"})}\n    |> LLMChain.new!()\n    # ... add messages ...\n    |> LLMChain.add_callback(handler)\n    |> LLMChain.run()\n\nWhen a request is received, something similar to the following will be output\nto the console.\n\n    %{\n      \"anthropic-ratelimit-requests-limit\" => [\"50\"],\n      \"anthropic-ratelimit-requests-remaining\" => [\"49\"],\n      \"anthropic-ratelimit-requests-reset\" => [\"2024-06-08T04:28:30Z\"],\n      \"anthropic-ratelimit-tokens-limit\" => [\"50000\"],\n      \"anthropic-ratelimit-tokens-remaining\" => [\"50000\"],\n      \"anthropic-ratelimit-tokens-reset\" => [\"2024-06-08T04:28:30Z\"],\n      \"request-id\" => [\"req_1234\"]\n    }","ref":"LangChain.ChatModels.ChatAnthropic.html#module-rate-limit-api-response-headers"},{"type":"module","title":"Token Usage - LangChain.ChatModels.ChatAnthropic","doc":"Anthropic returns token usage information as part of the response body. The\n`LangChain.TokenUsage` is added to the `metadata` of the `LangChain.Message`\nand `LangChain.MessageDelta` structs that are processed under the `:usage`\nkey.\n\n```elixir\n%LangChain.MessageDelta{\n  content: [],\n  status: :incomplete,\n  index: nil,\n  role: :assistant,\n  tool_calls: nil,\n  metadata: %{\n          usage: %LangChain.TokenUsage{\n            input: 55,\n            output: 4,\n            raw: %{\n              \"cache_creation_input_tokens\" => 0,\n              \"cache_read_input_tokens\" => 0,\n              \"input_tokens\" => 55,\n              \"output_tokens\" => 4\n            }\n          }\n  }\n}\n```\n\nThe `TokenUsage` data is accumulated for `MessageDelta` structs and the final usage information will be on the `LangChain.Message`.","ref":"LangChain.ChatModels.ChatAnthropic.html#module-token-usage"},{"type":"module","title":"Tool Choice - LangChain.ChatModels.ChatAnthropic","doc":"Anthropic supports forcing a tool to be used.\n- https://docs.anthropic.com/en/docs/build-with-claude/tool-use#forcing-tool-use\n\nThis is supported through the `tool_choice` options. It takes a plain Elixir map to provide the configuration.\n\nBy default, the LLM will choose a tool call if a tool is available and it determines it is needed. That's the \"auto\" mode.","ref":"LangChain.ChatModels.ChatAnthropic.html#module-tool-choice"},{"type":"module","title":"Example - LangChain.ChatModels.ChatAnthropic","doc":"Force the LLM's response to make a tool call of the \"get_weather\" function.\n\n    ChatAnthropic.new(%{\n      model: \"...\",\n      tool_choice: %{\"type\" => \"tool\", \"name\" => \"get_weather\"}\n    })","ref":"LangChain.ChatModels.ChatAnthropic.html#module-example"},{"type":"module","title":"AWS Bedrock Support - LangChain.ChatModels.ChatAnthropic","doc":"Anthropic Claude is supported in [AWS Bedrock](https://docs.aws.amazon.com/bedrock/latest/userguide/what-is-bedrock.html).\n\nTo configure `ChatAnthropic` for use on AWS Bedrock:\n\n1. Request [Model Access](https://console.aws.amazon.com/bedrock/home?#/modelaccess) to get access to the Anthropic models you intend to use.\n2. Using your AWS Console, create an Access Key for your application.\n3. Set the key values in your `AWS_ACCESS_KEY_ID` and `AWS_SECRET_ACCESS_KEY` ENVs.\n4. Get the Model ID for the model you intend to use. [Base Models](https://console.aws.amazon.com/bedrock/home?#/models)\n5. Refer to `LangChain.Utils.BedrockConfig` for setting up the Bedrock authentication credentials for your environment.\n6. Setup your ChatAnthropic similar to the following:\n\n    alias LangChain.ChatModels.ChatAnthropic\n\n    ChatAnthropic.new!(%{\n      model: \"anthropic.claude-3-5-sonnet-20241022-v2:0\",\n      bedrock: BedrockConfig.from_application_env!()\n    })","ref":"LangChain.ChatModels.ChatAnthropic.html#module-aws-bedrock-support"},{"type":"module","title":"Thinking - LangChain.ChatModels.ChatAnthropic","doc":"Models like Claude 3.7 Sonnet introduced a hybrid approach which allows for \"thinking\" and reasoning.\nSee the [Anthropic thinking documentation](https://docs.anthropic.com/en/docs/build-with-claude/extended-thinking)\nfor up-to-date instructions on the usage.\n\nFor instance, enabling thinking may require the `temperature` to be set to `1` and other settings like `topP` may not be allowed.\n\nThe model supports a `:thinking` attribute where the data is a map that matches the structure in the\n[Anthropic documentation](https://docs.anthropic.com/en/api/messages#body-thinking). It is passed along as-is.\n\n**Example:**\n\n    # Enable thinking and budget 2,000 tokens for the thinking space.\n    model = ChatAnthropic.new!(%{\n      model: \"claude-3-7-sonnet-latest\",\n      thinking: %{type: \"enabled\", budget_tokens: 2000}\n    })\n\n    # Disable thinking\n    model = ChatAnthropic.new!(%{\n      model: \"claude-3-7-sonnet-latest\",\n      thinking: %{type: \"disabled\"}\n    })\n\nAs of the documentation for Claude 3.7 Sonnet, the minimum budget for thinking is 1024 tokens.","ref":"LangChain.ChatModels.ChatAnthropic.html#module-thinking"},{"type":"module","title":"Prompt Caching - LangChain.ChatModels.ChatAnthropic","doc":"Anthropic supports [prompt caching](https://docs.anthropic.com/en/docs/build-with-claude/prompt-caching) to\nreduce costs and latency for frequently repeated content. Prompt caching works by caching large blocks of\ncontent that are likely to be reused across multiple requests.\n\nPrompt caching is configured through the `cache_control` option in `ContentPart` options. It can be applied\nto both system messages, regular user messages, tool results, and tool definitions.\n\nAnthropic limits a conversation to max of 4 cache_control blocks and will refuse to service requests with more.","ref":"LangChain.ChatModels.ChatAnthropic.html#module-prompt-caching"},{"type":"module","title":"Basic Usage - LangChain.ChatModels.ChatAnthropic","doc":"Setting `cache_control: true` is a shortcut for the default ephemeral cache control:\n\n    # System message with caching\n    Message.new_system!([\n      ContentPart.text!(\"You are an AI assistant analyzing literary works.\"),\n      ContentPart.text!(\" \", cache_control: true)\n    ])\n\n    # User message with caching\n    Message.new_user!([\n      ContentPart.text!(\"Please analyze this document:\"),\n      ContentPart.text!(\" \", cache_control: true)\n    ])","ref":"LangChain.ChatModels.ChatAnthropic.html#module-basic-usage"},{"type":"module","title":"Advanced Cache Control - LangChain.ChatModels.ChatAnthropic","doc":"For more explicit control over caching parameters, you can provide a map instead of `true`:\n\n    ContentPart.text!(\"content\", cache_control: %{\"type\" => \"ephemeral\", \"ttl\" => \"1h\"})\n\nWhen `cache_control: true` is used, it automatically expands to `%{\"type\" => \"ephemeral\"}` in the API request.\nIf you need specific cache control settings like TTL, providing them explicitly preserves the exact values\nsent to the API.\n\nThe default is \"5m\" for 5 minutes but supports \"1h\" for 1 hour depending on your account.","ref":"LangChain.ChatModels.ChatAnthropic.html#module-advanced-cache-control"},{"type":"module","title":"Supported Content Types - LangChain.ChatModels.ChatAnthropic","doc":"Prompt caching can be applied to:\n- Text content in system messages\n- Text content in user messages\n- Tool results in the `content` field when returning a list of `ContentPart` structs.\n- Tool definitions in the `options` field when creating a `Function` struct.\n\nFor more information, see the [Anthropic prompt caching documentation](https://docs.anthropic.com/en/docs/build-with-claude/prompt-caching).","ref":"LangChain.ChatModels.ChatAnthropic.html#module-supported-content-types"},{"type":"function","title":"LangChain.ChatModels.ChatAnthropic.call/3","doc":"Calls the Anthropic API passing the ChatAnthropic struct with configuration, plus\neither a simple message or the list of messages to act as the prompt.\n\nOptionally pass in a callback function that can be executed as data is\nreceived from the API.\n\n**NOTE:** This function *can* be used directly, but the primary interface\nshould be through `LangChain.Chains.LLMChain`. The `ChatAnthropic` module is more focused on\ntranslating the `LangChain` data structures to and from the Anthropic API.\n\nAnother benefit of using `LangChain.Chains.LLMChain` is that it combines the\nstorage of messages, adding functions, adding custom context that should be\npassed to functions, and automatically applying `LangChain.MessageDelta`\nstructs as they are are received, then converting those to the full\n`LangChain.Message` once fully complete.","ref":"LangChain.ChatModels.ChatAnthropic.html#call/3"},{"type":"function","title":"LangChain.ChatModels.ChatAnthropic.content_part_for_api/1","doc":"Converts a ContentPart to the format expected by the Anthropic API.\n\nHandles different content types:\n- `:text` - Converts to a text content part, optionally with cache control settings\n- `:thinking` - Converts to a thinking content part with required signature\n- `:unsupported` - Handles custom content types specified in options\n- `:image` - Converts to an image content part with base64 data and media type\n- `:image_url` - Raises an error as Anthropic doesn't support image URLs","ref":"LangChain.ChatModels.ChatAnthropic.html#content_part_for_api/1"},{"type":"function","title":"Options - LangChain.ChatModels.ChatAnthropic.content_part_for_api/1","doc":"For `:text` type:\n- `:cache_control` - When provided, adds cache control settings to the content\n\nFor `:thinking` type:\n- `:signature` - Required signature for thinking content\n\nFor `:unsupported` type:\n- `:type` - Required string specifying the custom content type\n\nFor `:image` type:\n- `:media` - Required media type (`:png`, `:jpg`, `:jpeg`, `:gif`, `:webp`, or a string)\n\nReturns `nil` for unsupported content without required options.","ref":"LangChain.ChatModels.ChatAnthropic.html#content_part_for_api/1-options"},{"type":"function","title":"LangChain.ChatModels.ChatAnthropic.content_parts_for_api/1","doc":"Converts a list of ContentParts to the format expected by the Anthropic API.","ref":"LangChain.ChatModels.ChatAnthropic.html#content_parts_for_api/1"},{"type":"function","title":"LangChain.ChatModels.ChatAnthropic.for_api/1","doc":"Convert a LangChain structure to the expected map of data for the Anthropic API.","ref":"LangChain.ChatModels.ChatAnthropic.html#for_api/1"},{"type":"function","title":"LangChain.ChatModels.ChatAnthropic.for_api/3","doc":"Return the params formatted for an API request.","ref":"LangChain.ChatModels.ChatAnthropic.html#for_api/3"},{"type":"function","title":"LangChain.ChatModels.ChatAnthropic.function_for_api/1","doc":"Convert a Function to the format expected by the Anthropic API.","ref":"LangChain.ChatModels.ChatAnthropic.html#function_for_api/1"},{"type":"function","title":"LangChain.ChatModels.ChatAnthropic.get_system_text/1","doc":"","ref":"LangChain.ChatModels.ChatAnthropic.html#get_system_text/1"},{"type":"function","title":"LangChain.ChatModels.ChatAnthropic.message_for_api/1","doc":"Converts a Message to the format expected by the Anthropic API.","ref":"LangChain.ChatModels.ChatAnthropic.html#message_for_api/1"},{"type":"function","title":"LangChain.ChatModels.ChatAnthropic.new/1","doc":"Setup a ChatAnthropic client configuration.","ref":"LangChain.ChatModels.ChatAnthropic.html#new/1"},{"type":"function","title":"LangChain.ChatModels.ChatAnthropic.new!/1","doc":"Setup a ChatAnthropic client configuration and return it or raise an error if invalid.","ref":"LangChain.ChatModels.ChatAnthropic.html#new!/1"},{"type":"function","title":"LangChain.ChatModels.ChatAnthropic.post_process_and_combine_messages/1","doc":"After all the messages have been converted using `for_api/1`, this combines\nmultiple sequential tool response messages. The Anthropic API is very strict\nabout user, assistant, user, assistant sequenced messages.","ref":"LangChain.ChatModels.ChatAnthropic.html#post_process_and_combine_messages/1"},{"type":"function","title":"LangChain.ChatModels.ChatAnthropic.relevant_event?/1","doc":"","ref":"LangChain.ChatModels.ChatAnthropic.html#relevant_event?/1"},{"type":"function","title":"LangChain.ChatModels.ChatAnthropic.restore_from_map/1","doc":"Restores the model from the config.","ref":"LangChain.ChatModels.ChatAnthropic.html#restore_from_map/1"},{"type":"function","title":"LangChain.ChatModels.ChatAnthropic.serialize_config/1","doc":"Generate a config map that can later restore the model's configuration.","ref":"LangChain.ChatModels.ChatAnthropic.html#serialize_config/1"},{"type":"type","title":"LangChain.ChatModels.ChatAnthropic.t/0","doc":"","ref":"LangChain.ChatModels.ChatAnthropic.html#t:t/0"},{"type":"module","title":"LangChain.ChatModels.ChatBumblebee","doc":"Represents a chat model hosted by Bumblebee and accessed through an\n`Nx.Serving`.\n\nMany types of models can be hosted through Bumblebee, so this attempts to\nrepresent the most common features and provide a single implementation where\npossible.\n\nFor streaming responses, the Bumblebee serving must be configured with\n`stream: true` and should include `stream_done: true` as well.\n\nExample:\n\n    Bumblebee.Text.generation(model_info, tokenizer, generation_config,\n      # ...\n      stream: true,\n      stream_done: true\n    )\n\nThis supports a non streaming response as well, in which case, a completed\n`LangChain.Message` is returned at the completion.\n\nThe `stream_done` option sends a final message to let us know when the stream\nis complete and includes some token information.\n\nThe chat model can be created like this and provided to an LLMChain:\n\n    ChatBumblebee.new!(%{\n      serving: @serving_name,\n      template_format: @template_format,\n      receive_timeout: @receive_timeout,\n      stream: true\n    })\n\nThe `serving` is the module name of the `Nx.Serving` that is hosting the\nmodel.\n\nThe following are the supported values for `template_format`. These are\nprovided by `LangChain.Utils.ChatTemplates`.\n\nChat models are trained against specific content formats for the messages.\nSome models have no special concept of a system message. See the\n`LangChain.Utils.ChatTemplates` documentation for specific format examples.\n\nUsing the wrong format with a model may result in poor performance or\nhallucinations. It will not result in an error.","ref":"LangChain.ChatModels.ChatBumblebee.html"},{"type":"module","title":"Full example of chat through Bumblebee - LangChain.ChatModels.ChatBumblebee","doc":"Here's a full example of having a streaming conversation with Llama 2 through\nBumblebee.\n\n    defmodule MyApp.BumblebeeChat do\n      @doc false\n      alias LangChain.Message\n      alias LangChain.ChatModels.ChatBumblebee\n      alias LangChain.Chains.LLMChain\n\n      def run_chat do\n        # Used when streaming responses. The function fires as data is received.\n        callback_fn = fn\n          %LangChain.MessageDelta{} = delta ->\n            # write to the console as the response is streamed back\n            IO.write(delta.content)\n\n          %LangChain.Message{} = message ->\n            # inspect the fully finished message that was assembled from all the deltas\n            IO.inspect(message, label: \"FULLY ASSEMBLED MESSAGE\")\n        end\n\n        # create and run the chain\n        {:ok, _updated_chain, %Message{} = message} =\n          LLMChain.new!(%{\n            llm:\n              ChatBumblebee.new!(%{\n                serving: Llama2ChatModel,\n                template_format: :llama_2,\n                stream: true\n              }),\n            verbose: true\n          })\n          |> LLMChain.add_message(Message.new_system!(\"You are a helpful assistant.\"))\n          |> LLMChain.add_message(Message.new_user!(\"What is the capital of Taiwan? And share up to 5 interesting facts about the city.\"))\n          |> LLMChain.run(callback_fn: callback_fn)\n\n        # print the LLM's fully assembled answer\n        IO.puts(\"\\n\\n\")\n        IO.puts(message.content)\n        IO.puts(\"\\n\\n\")\n      end\n    end\n\nThen run the code in IEx:\n\n      recompile; MyApp.BumblebeeChat.run_chat","ref":"LangChain.ChatModels.ChatBumblebee.html#module-full-example-of-chat-through-bumblebee"},{"type":"function","title":"LangChain.ChatModels.ChatBumblebee.new/1","doc":"Setup a ChatBumblebee client configuration.","ref":"LangChain.ChatModels.ChatBumblebee.html#new/1"},{"type":"function","title":"LangChain.ChatModels.ChatBumblebee.new!/1","doc":"Setup a ChatBumblebee client configuration and return it or raise an error if invalid.","ref":"LangChain.ChatModels.ChatBumblebee.html#new!/1"},{"type":"function","title":"LangChain.ChatModels.ChatBumblebee.restore_from_map/1","doc":"Restores the model from the config.","ref":"LangChain.ChatModels.ChatBumblebee.html#restore_from_map/1"},{"type":"function","title":"LangChain.ChatModels.ChatBumblebee.serialize_config/1","doc":"Generate a config map that can later restore the model's configuration.","ref":"LangChain.ChatModels.ChatBumblebee.html#serialize_config/1"},{"type":"type","title":"LangChain.ChatModels.ChatBumblebee.callback_fn/0","doc":"","ref":"LangChain.ChatModels.ChatBumblebee.html#t:callback_fn/0"},{"type":"type","title":"LangChain.ChatModels.ChatBumblebee.t/0","doc":"","ref":"LangChain.ChatModels.ChatBumblebee.html#t:t/0"},{"type":"module","title":"LangChain.ChatModels.ChatGoogleAI","doc":"Parses and validates inputs for making a request for the Google AI  Chat API.\n\nConverts response into more specialized `LangChain` data structures.\n\n**NOTE:** The GoogleAI service is unique in how it reports TokenUsage\ninformation. So far, it's the only API that returns TokenUsage for each\nreturned delta, where the generated token count is incremented with one. Other\nservices return the total TokenUsage data at the end. This Chat model fires\nthe callback each time it is received.\n\n**Google Search Integration**\n\nStarting with Gemini 2.0, this module supports Google Search as a native tool,\nallowing the model to automatically search the web for recent information to ground\nits responses and improve factuality. Check out the [Google AI Documentation](https://ai.google.dev/gemini-api/docs/grounding?lang=rest)\nfor more information.\n\nExample Usage:\n\n```elixir\nalias LangChain.Chains.LLMChain\nalias LangChain.Message\nalias LangChain.NativeTool\n\nmodel = ChatGoogleAI.new!(%{temperature: 0, stream: false, model: \"gemini-2.0-flash\"})\n\n{:ok, updated_chain} =\n   %{llm: model, verbose: false, stream: false}\n   |> LLMChain.new!()\n   |> LLMChain.add_message(\n     Message.new_user!(\"What is the current Google stock price?\")\n   )\n   |> LLMChain.add_tools(NativeTool.new!(%{name: \"google_search\", configuration: %{}}))\n   |> LLMChain.run()\n```\n\nThe above call will return the current Google stock price.\n\nWhen `google_search` is used, the model will also return grounding information in the metadata attribute of the assistant message.","ref":"LangChain.ChatModels.ChatGoogleAI.html"},{"type":"function","title":"LangChain.ChatModels.ChatGoogleAI.call/3","doc":"Calls the Google AI API passing the ChatGoogleAI struct with configuration, plus\neither a simple message or the list of messages to act as the prompt.\n\nOptionally pass in a list of tools available to the LLM for requesting\nexecution in response.\n\nOptionally pass in a callback function that can be executed as data is\nreceived from the API.\n\n**NOTE:** This function *can* be used directly, but the primary interface\nshould be through `LangChain.Chains.LLMChain`. The `ChatGoogleAI` module is more focused on\ntranslating the `LangChain` data structures to and from the Google AI API.\n\nAnother benefit of using `LangChain.Chains.LLMChain` is that it combines the\nstorage of messages, adding tools, adding custom context that should be\npassed to tools, and automatically applying `LangChain.MessageDelta`\nstructs as they are are received, then converting those to the full\n`LangChain.Message` once fully complete.","ref":"LangChain.ChatModels.ChatGoogleAI.html#call/3"},{"type":"function","title":"LangChain.ChatModels.ChatGoogleAI.complete_final_delta/1","doc":"","ref":"LangChain.ChatModels.ChatGoogleAI.html#complete_final_delta/1"},{"type":"function","title":"LangChain.ChatModels.ChatGoogleAI.do_process_response/3","doc":"","ref":"LangChain.ChatModels.ChatGoogleAI.html#do_process_response/3"},{"type":"function","title":"LangChain.ChatModels.ChatGoogleAI.for_api/3","doc":"","ref":"LangChain.ChatModels.ChatGoogleAI.html#for_api/3"},{"type":"function","title":"LangChain.ChatModels.ChatGoogleAI.get_message_contents/1","doc":"Return the content parts for the message.","ref":"LangChain.ChatModels.ChatGoogleAI.html#get_message_contents/1"},{"type":"function","title":"LangChain.ChatModels.ChatGoogleAI.new/1","doc":"Setup a ChatGoogleAI client configuration.","ref":"LangChain.ChatModels.ChatGoogleAI.html#new/1"},{"type":"function","title":"LangChain.ChatModels.ChatGoogleAI.new!/1","doc":"Setup a ChatGoogleAI client configuration and return it or raise an error if invalid.","ref":"LangChain.ChatModels.ChatGoogleAI.html#new!/1"},{"type":"function","title":"LangChain.ChatModels.ChatGoogleAI.restore_from_map/1","doc":"Restores the model from the config.","ref":"LangChain.ChatModels.ChatGoogleAI.html#restore_from_map/1"},{"type":"function","title":"LangChain.ChatModels.ChatGoogleAI.serialize_config/1","doc":"Generate a config map that can later restore the model's configuration.","ref":"LangChain.ChatModels.ChatGoogleAI.html#serialize_config/1"},{"type":"type","title":"LangChain.ChatModels.ChatGoogleAI.t/0","doc":"","ref":"LangChain.ChatModels.ChatGoogleAI.html#t:t/0"},{"type":"module","title":"LangChain.ChatModels.ChatMistralAI","doc":"","ref":"LangChain.ChatModels.ChatMistralAI.html"},{"type":"function","title":"LangChain.ChatModels.ChatMistralAI.call/3","doc":"Calls the Mistral API passing the ChatMistralAI struct plus either a simple string\nprompt or a list of messages as the prompt. Optionally pass in a list of tools.","ref":"LangChain.ChatModels.ChatMistralAI.html#call/3"},{"type":"function","title":"LangChain.ChatModels.ChatMistralAI.for_api/2","doc":"Converts a LangChain Message-based structure into the expected map of data for\nMistral. We also include any `tool_calls` stored on the message.","ref":"LangChain.ChatModels.ChatMistralAI.html#for_api/2"},{"type":"function","title":"LangChain.ChatModels.ChatMistralAI.for_api/3","doc":"Formats this struct plus the given messages and tools as a request payload.","ref":"LangChain.ChatModels.ChatMistralAI.html#for_api/3"},{"type":"function","title":"LangChain.ChatModels.ChatMistralAI.new/1","doc":"","ref":"LangChain.ChatModels.ChatMistralAI.html#new/1"},{"type":"function","title":"LangChain.ChatModels.ChatMistralAI.new!/1","doc":"","ref":"LangChain.ChatModels.ChatMistralAI.html#new!/1"},{"type":"function","title":"LangChain.ChatModels.ChatMistralAI.restore_from_map/1","doc":"Restores the model from the config map.","ref":"LangChain.ChatModels.ChatMistralAI.html#restore_from_map/1"},{"type":"function","title":"LangChain.ChatModels.ChatMistralAI.serialize_config/1","doc":"Generate a config map that can later restore the model's configuration.","ref":"LangChain.ChatModels.ChatMistralAI.html#serialize_config/1"},{"type":"type","title":"LangChain.ChatModels.ChatMistralAI.t/0","doc":"","ref":"LangChain.ChatModels.ChatMistralAI.html#t:t/0"},{"type":"behaviour","title":"LangChain.ChatModels.ChatModel","doc":"","ref":"LangChain.ChatModels.ChatModel.html"},{"type":"callback","title":"LangChain.ChatModels.ChatModel.call/3","doc":"","ref":"LangChain.ChatModels.ChatModel.html#c:call/3"},{"type":"function","title":"LangChain.ChatModels.ChatModel.restore_from_map/1","doc":"Restore a ChatModel from a serialized config map.","ref":"LangChain.ChatModels.ChatModel.html#restore_from_map/1"},{"type":"callback","title":"LangChain.ChatModels.ChatModel.restore_from_map/1","doc":"","ref":"LangChain.ChatModels.ChatModel.html#c:restore_from_map/1"},{"type":"function","title":"LangChain.ChatModels.ChatModel.serialize_config/1","doc":"Create a serializable map from a ChatModel's current configuration that can\nlater be restored.","ref":"LangChain.ChatModels.ChatModel.html#serialize_config/1"},{"type":"callback","title":"LangChain.ChatModels.ChatModel.serialize_config/1","doc":"","ref":"LangChain.ChatModels.ChatModel.html#c:serialize_config/1"},{"type":"type","title":"LangChain.ChatModels.ChatModel.call_response/0","doc":"","ref":"LangChain.ChatModels.ChatModel.html#t:call_response/0"},{"type":"type","title":"LangChain.ChatModels.ChatModel.t/0","doc":"","ref":"LangChain.ChatModels.ChatModel.html#t:t/0"},{"type":"type","title":"LangChain.ChatModels.ChatModel.tool/0","doc":"","ref":"LangChain.ChatModels.ChatModel.html#t:tool/0"},{"type":"type","title":"LangChain.ChatModels.ChatModel.tools/0","doc":"","ref":"LangChain.ChatModels.ChatModel.html#t:tools/0"},{"type":"module","title":"LangChain.ChatModels.ChatOllamaAI","doc":"Represents the [Ollama AI Chat model](https://github.com/jmorganca/ollama/blob/main/docs/api.md#generate-a-chat-completion)\n\nParses and validates inputs for making a requests from the Ollama Chat API.\n\nConverts responses into more specialized `LangChain` data structures.\n\nThe module's functionalities include:\n\n- Initializing a new `ChatOllamaAI` struct with defaults or specific attributes.\n- Validating and casting input data to fit the expected schema.\n- Preparing and sending requests to the Ollama AI service API.\n- Managing both streaming and non-streaming API responses.\n- Processing API responses to convert them into suitable message formats.\n\nThe `ChatOllamaAI` struct has fields to configure the AI, including but not limited to:\n\n- `endpoint`: URL of the Ollama AI service.\n- `model`: The AI model used, e.g., \"llama2:latest\".\n- `receive_timeout`: Max wait time for AI service responses.\n- `temperature`: Influences the AI's response creativity.\n\nFor detailed info on on all other parameters see documentation here:\nhttps://github.com/jmorganca/ollama/blob/main/docs/modelfile.md#valid-parameters-and-values\n\nThis module is for use within LangChain and follows the `ChatModel` behavior,\noutlining callbacks AI chat models must implement.\n\nUsage examples and more details are in the LangChain documentation or the\nmodule's function docs.","ref":"LangChain.ChatModels.ChatOllamaAI.html"},{"type":"module","title":"Tool Support - LangChain.ChatModels.ChatOllamaAI","doc":"Currently, `ChatOllamaAI` supports tool calls when not streaming the responses.\nStreaming tool calls is not yet supported.","ref":"LangChain.ChatModels.ChatOllamaAI.html#module-tool-support"},{"type":"function","title":"LangChain.ChatModels.ChatOllamaAI.call/3","doc":"Calls the Ollama Chat Completion API struct with configuration, plus\neither a simple message or the list of messages to act as the prompt.\n\n**NOTE:** This function *can* be used directly, but the primary interface\nshould be through `LangChain.Chains.LLMChain`. The `ChatOllamaAI` module is more focused on\ntranslating the `LangChain` data structures to and from the Ollama API.\n\nAnother benefit of using `LangChain.Chains.LLMChain` is that it combines the\nstorage of messages, adding functions, adding custom context that should be\npassed to functions, and automatically applying `LangChain.MessageDelta`\nstructs as they are are received, then converting those to the full\n`LangChain.Message` once fully complete.","ref":"LangChain.ChatModels.ChatOllamaAI.html#call/3"},{"type":"function","title":"LangChain.ChatModels.ChatOllamaAI.do_process_response/2","doc":"","ref":"LangChain.ChatModels.ChatOllamaAI.html#do_process_response/2"},{"type":"function","title":"LangChain.ChatModels.ChatOllamaAI.for_api/1","doc":"","ref":"LangChain.ChatModels.ChatOllamaAI.html#for_api/1"},{"type":"function","title":"LangChain.ChatModels.ChatOllamaAI.for_api/3","doc":"Return the params formatted for an API request.","ref":"LangChain.ChatModels.ChatOllamaAI.html#for_api/3"},{"type":"function","title":"LangChain.ChatModels.ChatOllamaAI.new/1","doc":"Creates a new `ChatOllamaAI` struct with the given attributes.","ref":"LangChain.ChatModels.ChatOllamaAI.html#new/1"},{"type":"function","title":"LangChain.ChatModels.ChatOllamaAI.new!/1","doc":"Creates a new `ChatOllamaAI` struct with the given attributes. Will raise an error if the changeset is invalid.","ref":"LangChain.ChatModels.ChatOllamaAI.html#new!/1"},{"type":"function","title":"LangChain.ChatModels.ChatOllamaAI.restore_from_map/1","doc":"Restores the model from the config.","ref":"LangChain.ChatModels.ChatOllamaAI.html#restore_from_map/1"},{"type":"function","title":"LangChain.ChatModels.ChatOllamaAI.serialize_config/1","doc":"Generate a config map that can later restore the model's configuration.","ref":"LangChain.ChatModels.ChatOllamaAI.html#serialize_config/1"},{"type":"type","title":"LangChain.ChatModels.ChatOllamaAI.t/0","doc":"","ref":"LangChain.ChatModels.ChatOllamaAI.html#t:t/0"},{"type":"module","title":"LangChain.ChatModels.ChatOpenAI","doc":"Represents the [OpenAI\nChatModel](https://platform.openai.com/docs/api-reference/chat/create).\n\nParses and validates inputs for making a requests from the OpenAI Chat API.\n\nConverts responses into more specialized `LangChain` data structures.\n\n- https://github.com/openai/openai-cookbook/blob/main/examples/How_to_call_functions_with_chat_models.ipynb","ref":"LangChain.ChatModels.ChatOpenAI.html"},{"type":"module","title":"ContentPart Types - LangChain.ChatModels.ChatOpenAI","doc":"OpenAI supports several types of content parts that can be combined in a single message:","ref":"LangChain.ChatModels.ChatOpenAI.html#module-contentpart-types"},{"type":"module","title":"Text Content - LangChain.ChatModels.ChatOpenAI","doc":"Basic text content is the default and most common type:\n\n    Message.new_user!(\"Hello, how are you?\")","ref":"LangChain.ChatModels.ChatOpenAI.html#module-text-content"},{"type":"module","title":"Image Content - LangChain.ChatModels.ChatOpenAI","doc":"OpenAI supports both base64-encoded images and image URLs:\n\n    # Using a base64 encoded image\n    Message.new_user!([\n      ContentPart.text!(\"What's in this image?\"),\n      ContentPart.image!(\"base64_encoded_image_data\", media: :jpg)\n    ])\n\n    # Using an image URL\n    Message.new_user!([\n      ContentPart.text!(\"Describe this image:\"),\n      ContentPart.image_url!(\"https://example.com/image.jpg\")\n    ])\n\nFor images, you can specify the detail level which affects token usage:\n- `detail: \"low\"` - Lower resolution, fewer tokens\n- `detail: \"high\"` - Higher resolution, more tokens\n- `detail: \"auto\"` - Let the model decide","ref":"LangChain.ChatModels.ChatOpenAI.html#module-image-content"},{"type":"module","title":"File Content - LangChain.ChatModels.ChatOpenAI","doc":"OpenAI supports both base64-encoded files and file IDs:\n\n    # Using a base64 encoded file\n    Message.new_user!([\n      ContentPart.text!(\"Process this file:\"),\n      ContentPart.file!(\"base64_encoded_file_data\",\n        type: :base64,\n        filename: \"document.pdf\"\n      )\n    ])\n\n    # Using a file ID (after uploading to OpenAI)\n    Message.new_user!([\n      ContentPart.text!(\"Process this file:\"),\n      ContentPart.file!(\"file-1234\", type: :file_id)\n    ])","ref":"LangChain.ChatModels.ChatOpenAI.html#module-file-content"},{"type":"module","title":"Callbacks - LangChain.ChatModels.ChatOpenAI","doc":"See the set of available callbacks: `LangChain.Chains.ChainCallbacks`","ref":"LangChain.ChatModels.ChatOpenAI.html#module-callbacks"},{"type":"module","title":"Rate Limit API Response Headers - LangChain.ChatModels.ChatOpenAI","doc":"OpenAI returns rate limit information in the response headers. Those can be\naccessed using the LLM callback `on_llm_ratelimit_info` like this:\n\n    handlers = %{\n      on_llm_ratelimit_info: fn _model, headers ->\n        IO.inspect(headers)\n      end\n    }\n\n    {:ok, chat} = ChatOpenAI.new(%{callbacks: [handlers]})\n\nWhen a request is received, something similar to the following will be output\nto the console.\n\n    %{\n      \"x-ratelimit-limit-requests\" => [\"5000\"],\n      \"x-ratelimit-limit-tokens\" => [\"160000\"],\n      \"x-ratelimit-remaining-requests\" => [\"4999\"],\n      \"x-ratelimit-remaining-tokens\" => [\"159973\"],\n      \"x-ratelimit-reset-requests\" => [\"12ms\"],\n      \"x-ratelimit-reset-tokens\" => [\"10ms\"],\n      \"x-request-id\" => [\"req_1234\"]\n    }","ref":"LangChain.ChatModels.ChatOpenAI.html#module-rate-limit-api-response-headers"},{"type":"module","title":"Token Usage - LangChain.ChatModels.ChatOpenAI","doc":"OpenAI returns token usage information as part of the response body. The\n`LangChain.TokenUsage` is added to the `metadata` of the `LangChain.Message`\nand `LangChain.MessageDelta` structs that are processed under the `:usage`\nkey.\n\nThe OpenAI documentation instructs to provide the `stream_options` with the\n`include_usage: true` for the information to be provided.\n\n```elixir\nchat = ChatOpenAI.new!(%{stream: true, stream_options: %{include_usage: true}})\n```\n\nThe `TokenUsage` data is accumulated for `MessageDelta` structs and the final usage information will be on the `LangChain.Message`.\n\nNOTE: Of special note is that the `TokenUsage` information is returned once\nfor all \"choices\" in the response. The `LangChain.TokenUsage` data is added to\neach message, but if your usage requests multiple choices, you will see the\nsame usage information for each choice but it is duplicated and only one\nresponse is meaningful.","ref":"LangChain.ChatModels.ChatOpenAI.html#module-token-usage"},{"type":"module","title":"Tool Choice - LangChain.ChatModels.ChatOpenAI","doc":"OpenAI's ChatGPT API supports forcing a tool to be used.\n- https://platform.openai.com/docs/api-reference/chat/create#chat-create-tool_choice\n\nThis is supported through the `tool_choice` options. It takes a plain Elixir\nmap to provide the configuration.\n\nBy default, the LLM will choose a tool call if a tool is available and it\ndetermines it is needed. That's the \"auto\" mode.","ref":"LangChain.ChatModels.ChatOpenAI.html#module-tool-choice"},{"type":"module","title":"Example - LangChain.ChatModels.ChatOpenAI","doc":"For the LLM's response to make a tool call of the \"get_weather\" function.\n\n    ChatOpenAI.new(%{\n      model: \"...\",\n      tool_choice: %{\"type\" => \"function\", \"function\" => %{\"name\" => \"get_weather\"}}\n    })","ref":"LangChain.ChatModels.ChatOpenAI.html#module-example"},{"type":"module","title":"Azure OpenAI Support - LangChain.ChatModels.ChatOpenAI","doc":"To use `ChatOpenAI` with Microsoft's Azure hosted OpenAI models, the\n`endpoint` must be overridden and the API key needs to be provided in some\nway. The [MS Quickstart guide for REST\naccess](https://learn.microsoft.com/en-us/azure/ai-services/openai/chatgpt-quickstart?tabs=command-line%2Cjavascript-keyless%2Ctypescript-keyless%2Cpython-new&pivots=rest-api)\nmay be helpful.\n\nIn order to use it, you must have an Azure account and from the console, a\nmodel must be deployed for your account. Use the Azure AI Foundry and Azure\nOpenAI Service to deploy the model you want to use. The entire URL is used as\nthe `endpoint` and the provided `key` is used as the `api_key`.\n\nThe following is an example of setting up `ChatOpenAI` for use with an Azure\nhosted model.\n\n    endpoint = System.fetch_env!(\"AZURE_OPENAI_ENDPOINT\")\n    api_key = System.fetch_env!(\"AZURE_OPENAI_KEY\")\n\n    llm =\n      ChatOpenAI.new!(%{\n        endpoint: endpoint,\n        api_key: api_key,\n        seed: 0,\n        temperature: 1,\n        stream: false\n      })\n\nThe URL itself specifies the model to use and the `model` attribute is\ndisregarded.\n\nA fake example URL for the endpoint value:\n\n`https://some-subdomain.cognitiveservices.azure.com/openai/deployments/gpt-4o-mini/chat/completions?api-version=2024-08-01-preview\"`","ref":"LangChain.ChatModels.ChatOpenAI.html#module-azure-openai-support"},{"type":"module","title":"Reasoning Model Support - LangChain.ChatModels.ChatOpenAI","doc":"OpenAI made some significant API changes with the introduction of their\n\"reasoning\" models. This includes the `o1` and `o1-mini` models.\n\nTo enable this mode, set `:reasoning_mode` to `true`:\n\n    model = ChatOpenAI.new!(%{reasoning_mode: true})\n\nSetting `reasoning_mode` to `true` does at least the two following things:\n\n- Set `:developer` as the `role` for system messages. The OpenAI documentation\n  says API calls to `o1` and newer models must use the `role: :developer`\n  instead of `role: :system` and errors if not set correctly.\n- The `:reasoning_effort` option included in LLM requests. This setting is\n  only permitted on a reasoning model. The `:reasoning_effort` values support\n  the \"low\", \"medium\" (default), and \"high\" options specified in the OpenAI\n  documentation. This instructs the LLM on how much time, and tokens, should\n  be spent on thinking through and reasoning about the request and the\n  response.","ref":"LangChain.ChatModels.ChatOpenAI.html#module-reasoning-model-support"},{"type":"function","title":"LangChain.ChatModels.ChatOpenAI.call/3","doc":"Calls the OpenAI API passing the ChatOpenAI struct with configuration, plus\neither a simple message or the list of messages to act as the prompt.\n\nOptionally pass in a list of tools available to the LLM for requesting\nexecution in response.\n\nOptionally pass in a callback function that can be executed as data is\nreceived from the API.\n\n**NOTE:** This function *can* be used directly, but the primary interface\nshould be through `LangChain.Chains.LLMChain`. The `ChatOpenAI` module is more\nfocused on translating the `LangChain` data structures to and from the OpenAI\nAPI.\n\nAnother benefit of using `LangChain.Chains.LLMChain` is that it combines the\nstorage of messages, adding tools, adding custom context that should be\npassed to tools, and automatically applying `LangChain.MessageDelta`\nstructs as they are are received, then converting those to the full\n`LangChain.Message` once fully complete.","ref":"LangChain.ChatModels.ChatOpenAI.html#call/3"},{"type":"function","title":"LangChain.ChatModels.ChatOpenAI.content_part_for_api/2","doc":"Convert a ContentPart to the expected map of data for the OpenAI API.","ref":"LangChain.ChatModels.ChatOpenAI.html#content_part_for_api/2"},{"type":"function","title":"LangChain.ChatModels.ChatOpenAI.content_parts_for_api/2","doc":"Convert a list of ContentParts to the expected map of data for the OpenAI API.","ref":"LangChain.ChatModels.ChatOpenAI.html#content_parts_for_api/2"},{"type":"function","title":"LangChain.ChatModels.ChatOpenAI.decode_stream/2","doc":"Decode a streamed response from an OpenAI-compatible server. Parses a string\nof received content into an Elixir map data structure using string keys.\n\nIf a partial response was received, meaning the JSON text is split across\nmultiple data frames, then the incomplete portion is returned as-is in the\nbuffer. The function will be successively called, receiving the incomplete\nbuffer data from a previous call, and assembling it to parse.","ref":"LangChain.ChatModels.ChatOpenAI.html#decode_stream/2"},{"type":"function","title":"LangChain.ChatModels.ChatOpenAI.for_api/2","doc":"Convert a LangChain Message-based structure to the expected map of data for\nthe OpenAI API. This happens within the context of the model configuration as\nwell. The additional context is needed to correctly convert a role to either\n`:system` or `:developer`.\n\nNOTE: The `ChatOpenAI` model's functions are reused in other modules. For this\nreason, model is more generally defined as a struct.","ref":"LangChain.ChatModels.ChatOpenAI.html#for_api/2"},{"type":"function","title":"LangChain.ChatModels.ChatOpenAI.for_api/3","doc":"Return the params formatted for an API request.","ref":"LangChain.ChatModels.ChatOpenAI.html#for_api/3"},{"type":"function","title":"LangChain.ChatModels.ChatOpenAI.new/1","doc":"Setup a ChatOpenAI client configuration.","ref":"LangChain.ChatModels.ChatOpenAI.html#new/1"},{"type":"function","title":"LangChain.ChatModels.ChatOpenAI.new!/1","doc":"Setup a ChatOpenAI client configuration and return it or raise an error if invalid.","ref":"LangChain.ChatModels.ChatOpenAI.html#new!/1"},{"type":"function","title":"LangChain.ChatModels.ChatOpenAI.restore_from_map/1","doc":"Restores the model from the config.","ref":"LangChain.ChatModels.ChatOpenAI.html#restore_from_map/1"},{"type":"function","title":"LangChain.ChatModels.ChatOpenAI.serialize_config/1","doc":"Generate a config map that can later restore the model's configuration.","ref":"LangChain.ChatModels.ChatOpenAI.html#serialize_config/1"},{"type":"type","title":"LangChain.ChatModels.ChatOpenAI.t/0","doc":"","ref":"LangChain.ChatModels.ChatOpenAI.html#t:t/0"},{"type":"module","title":"LangChain.ChatModels.ChatPerplexity","doc":"Represents the [Perplexity Chat model](https://docs.perplexity.ai/api-reference/chat-completions).\n\nThis module implements a client for the Perplexity Chat API, providing functions to validate input parameters,\nformat API requests, and parse API responses into LangChain's structured data types.\n\nPerplexity does not natively support tool calling in the same manner as some other chat models.\nTo overcome this limitation, this module employs a workaround using structured outputs via a JSON schema.\nWhen tools are provided, the API request is augmented with a JSON schema that defines the expected format\nfor tool calls. The response processing logic then detects and decodes these tool call details, converting them\ninto corresponding ToolCall structs. This approach allows LangChain to seamlessly emulate tool calling functionality\nand integrate it with its standard workflow, similar to how ChatOpenAI handles function calls.\n\nIn addition, this module supports various configuration options such as temperature, top_p, top_k,\nand streaming, as well as callbacks for token usage and new message events.\n\nOverall, this implementation provides a unified interface for interacting with the Perplexity Chat API\nwhile working around its limitations regarding tool calling.","ref":"LangChain.ChatModels.ChatPerplexity.html"},{"type":"module","title":"Tool Calls - LangChain.ChatModels.ChatPerplexity","doc":"In order to use tool calls, you need specifically prompt Perplexity as outlined in their\n[Prompt Guide](https://docs.perplexity.ai/guides/prompt-guide) as well as the\n[Structured Outputs Guide](https://docs.perplexity.ai/guides/structured-outputs).\n\nProvide it additional prompting like:\n\n```\nRules:\n1. Provide only the final answer. It is important that you do not include any explanation on the steps below.\n2. Do not show the intermediate steps information.\n\nOutput a JSON object with the following fields:\n- title: The article title\n- keywords: An array of SEO keywords\n- meta_description: The SEO meta description\n```","ref":"LangChain.ChatModels.ChatPerplexity.html#module-tool-calls"},{"type":"function","title":"LangChain.ChatModels.ChatPerplexity.decode_stream/2","doc":"Decode a streamed response from the Perplexity API.","ref":"LangChain.ChatModels.ChatPerplexity.html#decode_stream/2"},{"type":"function","title":"LangChain.ChatModels.ChatPerplexity.do_process_response/2","doc":"","ref":"LangChain.ChatModels.ChatPerplexity.html#do_process_response/2"},{"type":"function","title":"LangChain.ChatModels.ChatPerplexity.for_api/2","doc":"Convert a LangChain Message-based structure to the expected map of data for\nthe Perplexity API.","ref":"LangChain.ChatModels.ChatPerplexity.html#for_api/2"},{"type":"function","title":"LangChain.ChatModels.ChatPerplexity.for_api/3","doc":"Return the params formatted for an API request.","ref":"LangChain.ChatModels.ChatPerplexity.html#for_api/3"},{"type":"function","title":"LangChain.ChatModels.ChatPerplexity.new/1","doc":"Setup a ChatPerplexity client configuration.","ref":"LangChain.ChatModels.ChatPerplexity.html#new/1"},{"type":"function","title":"LangChain.ChatModels.ChatPerplexity.new!/1","doc":"Setup a ChatPerplexity client configuration and return it or raise an error if invalid.","ref":"LangChain.ChatModels.ChatPerplexity.html#new!/1"},{"type":"function","title":"LangChain.ChatModels.ChatPerplexity.restore_from_map/1","doc":"Restores the model from the config.","ref":"LangChain.ChatModels.ChatPerplexity.html#restore_from_map/1"},{"type":"function","title":"LangChain.ChatModels.ChatPerplexity.serialize_config/1","doc":"Generate a config map that can later restore the model's configuration.","ref":"LangChain.ChatModels.ChatPerplexity.html#serialize_config/1"},{"type":"type","title":"LangChain.ChatModels.ChatPerplexity.t/0","doc":"","ref":"LangChain.ChatModels.ChatPerplexity.html#t:t/0"},{"type":"module","title":"LangChain.ChatModels.ChatVertexAI","doc":"Parses and validates inputs for making a request for the Google AI  Chat API.\n\nConverts response into more specialized `LangChain` data structures.\n\nExample Usage:\n\n```elixir\nalias LangChain.Chains.LLMChain\nalias LangChain.Message\nalias LangChain.Message.ContentPart\nalias LangChain.ChatModels.ChatVertexAI\n\n\nconfig = %{\n      model: \"gemini-2.0-flash\",\n      api_key: ..., # vertex requires gcloud auth token https://cloud.google.com/vertex-ai/generative-ai/docs/start/quickstarts/quickstart-multimodal#rest\n      temperature: 1.0,\n      top_p: 0.8,\n      receive_timeout: ...\n    }\n model = ChatVertexAI.new!(config)\n\n    %{llm: model, verbose: false, stream: false}\n    |> LLMChain.new!()\n    |> LLMChain.add_message(\n      Message.new_user!([\n        ContentPart.new!(%{type: :text, content: \"Analyse the provided file and share a summary\"}),\n        ContentPart.new!(%{\n          type: :file_url,\n          content: ...,\n          options: [media: ...]\n        })\n      ])\n    )\n    |> LLMChain.run()\nThe above call will return summary of the media content.\n```","ref":"LangChain.ChatModels.ChatVertexAI.html"},{"type":"function","title":"LangChain.ChatModels.ChatVertexAI.call/3","doc":"Calls the Google AI API passing the ChatVertexAI struct with configuration,\nplus either a simple message or the list of messages to act as the prompt.\n\nOptionally pass in a list of tools available to the LLM for requesting\nexecution in response.\n\n**NOTE:** This function *can* be used directly, but the primary interface\nshould be through `LangChain.Chains.LLMChain`. The `ChatVertexAI` module is\nmore focused on translating the `LangChain` data structures to and from the\nOpenAI API.\n\nAnother benefit of using `LangChain.Chains.LLMChain` is that it combines the\nstorage of messages, adding tools, adding custom context that should be passed\nto tools, and automatically applying `LangChain.MessageDelta` structs as they\nare are received, then converting those to the full `LangChain.Message` once\nfully complete.","ref":"LangChain.ChatModels.ChatVertexAI.html#call/3"},{"type":"function","title":"LangChain.ChatModels.ChatVertexAI.complete_final_delta/1","doc":"","ref":"LangChain.ChatModels.ChatVertexAI.html#complete_final_delta/1"},{"type":"function","title":"LangChain.ChatModels.ChatVertexAI.do_process_response/2","doc":"","ref":"LangChain.ChatModels.ChatVertexAI.html#do_process_response/2"},{"type":"function","title":"LangChain.ChatModels.ChatVertexAI.for_api/3","doc":"","ref":"LangChain.ChatModels.ChatVertexAI.html#for_api/3"},{"type":"function","title":"LangChain.ChatModels.ChatVertexAI.get_message_contents/1","doc":"Return the content parts for the message.","ref":"LangChain.ChatModels.ChatVertexAI.html#get_message_contents/1"},{"type":"function","title":"LangChain.ChatModels.ChatVertexAI.new/1","doc":"Setup a ChatVertexAI client configuration.","ref":"LangChain.ChatModels.ChatVertexAI.html#new/1"},{"type":"function","title":"LangChain.ChatModels.ChatVertexAI.new!/1","doc":"Setup a ChatVertexAI client configuration and return it or raise an error if invalid.","ref":"LangChain.ChatModels.ChatVertexAI.html#new!/1"},{"type":"function","title":"LangChain.ChatModels.ChatVertexAI.restore_from_map/1","doc":"Restores the model from the config.","ref":"LangChain.ChatModels.ChatVertexAI.html#restore_from_map/1"},{"type":"function","title":"LangChain.ChatModels.ChatVertexAI.serialize_config/1","doc":"Generate a config map that can later restore the model's configuration.","ref":"LangChain.ChatModels.ChatVertexAI.html#serialize_config/1"},{"type":"type","title":"LangChain.ChatModels.ChatVertexAI.t/0","doc":"","ref":"LangChain.ChatModels.ChatVertexAI.html#t:t/0"},{"type":"module","title":"LangChain.Chains.DataExtractionChain","doc":"Defines an LLMChain for performing data extraction from a body of text.\n\nProvide the schema for desired information to be parsed into. It is treated as\nthough there are 0 to many instances of the data structure being described so\ninformation is returned as an array.\n\nOriginally based on:\n- https://github.com/langchain-ai/langchainjs/blob/main/langchain/src/chains/openai_functions/extraction.ts#L43","ref":"LangChain.Chains.DataExtractionChain.html"},{"type":"module","title":"Example - LangChain.Chains.DataExtractionChain","doc":"# JSONSchema definition of data we want to capture or extract.\n    schema_parameters = %{\n      type: \"object\",\n      properties: %{\n        person_name: %{type: \"string\"},\n        person_age: %{type: \"number\"},\n        person_hair_color: %{type: \"string\"},\n        dog_name: %{type: \"string\"},\n        dog_breed: %{type: \"string\"}\n      },\n      required: []\n    }\n\n    # Model setup\n    {:ok, chat} = ChatOpenAI.new(%{temperature: 0})\n\n    # run the chain on the text information\n    data_prompt =\n      \"Alex is 5 feet tall. Claudia is 4 feet taller than Alex and jumps higher than him.\n      Claudia is a brunette and Alex is blonde. Alex's dog Frosty is a labrador and likes to play hide and seek.\"\n\n    {:ok, result} = LangChain.Chains.DataExtractionChain.run(chat, schema_parameters, data_prompt)\n\n    # Example result\n    [\n      %{\n        \"dog_breed\" => \"labrador\",\n        \"dog_name\" => \"Frosty\",\n        \"person_age\" => nil,\n        \"person_hair_color\" => \"blonde\",\n        \"person_name\" => \"Alex\"\n      },\n      %{\n        \"dog_breed\" => nil,\n        \"dog_name\" => nil,\n        \"person_age\" => nil,\n        \"person_hair_color\" => \"brunette\",\n        \"person_name\" => \"Claudia\"\n      }\n    ]\n\nThe `schema_parameters` in the previous example can also be expressed using a\nlist of `LangChain.FunctionParam` structs. An equivalent version looks like\nthis:\n\n    alias LangChain.FunctionParam\n\n    schema_parameters = [\n      FunctionParam.new!(%{name: \"person_name\", type: :string}),\n      FunctionParam.new!(%{name: \"person_age\", type: :number}),\n      FunctionParam.new!(%{name: \"person_hair_color\", type: :string}),\n      FunctionParam.new!(%{name: \"dog_name\", type: :string}),\n      FunctionParam.new!(%{name: \"dog_breed\", type: :string})\n    ]\n    |> FunctionParam.to_parameters_schema()","ref":"LangChain.Chains.DataExtractionChain.html#module-example"},{"type":"function","title":"LangChain.Chains.DataExtractionChain.build_extract_function/1","doc":"Build the function to expose to the LLM that can be called for data\nextraction.","ref":"LangChain.Chains.DataExtractionChain.html#build_extract_function/1"},{"type":"function","title":"LangChain.Chains.DataExtractionChain.run/4","doc":"Run the data extraction chain.","ref":"LangChain.Chains.DataExtractionChain.html#run/4"},{"type":"module","title":"LangChain.Chains.LLMChain","doc":"","ref":"LangChain.Chains.LLMChain.html"},{"type":"function","title":"LangChain.Chains.LLMChain.__struct__/0","doc":"Define an LLMChain. This is the heart of the LangChain library.\n\nThe chain deals with tools, a tool map, delta tracking, tracking the messages\nexchanged during a run, the last_message tracking, conversation messages, and\nverbose logging. Messages and tool results support multi-modal ContentParts,\nenabling richer responses (text, images, files, thinking, etc.). ToolResult\ncontent can be a list of ContentParts. The chain also supports\n`async_tool_timeout` and improved fallback handling.","ref":"LangChain.Chains.LLMChain.html#__struct__/0"},{"type":"function","title":"Callbacks - LangChain.Chains.LLMChain.__struct__/0","doc":"Callbacks are fired as specific events occur in the chain as it is running.\nThe set of events are defined in `LangChain.Chains.ChainCallbacks`.\n\nTo be notified of an event you care about, register a callback handler with\nthe chain. Multiple callback handlers can be assigned. The callback handler\nassigned to the `LLMChain` is not provided to an LLM chat model. For callbacks\non a chat model, set them there.","ref":"LangChain.Chains.LLMChain.html#__struct__/0-callbacks"},{"type":"function","title":"Registering a callback handler - LangChain.Chains.LLMChain.__struct__/0","doc":"A handler is a map with key name for the callback to fire. A function is\nassigned to the map key. Refer to the documentation for each function as they\narguments vary.\n\nIf we want to be notified when an LLM Assistant chat response message has been\nprocessed and it is complete, this is how we could receive that event in our\nrunning LiveView:\n\n    live_view_pid = self()\n\n    handler = %{\n      on_message_processed: fn _chain, message ->\n        send(live_view_pid, {:new_assistant_response, message})\n      end\n    }\n\n    LLMChain.new!(%{...})\n    |> LLMChain.add_callback(handler)\n    |> LLMChain.run()\n\nIn the LiveView, a `handle_info` function executes with the received message.","ref":"LangChain.Chains.LLMChain.html#__struct__/0-registering-a-callback-handler"},{"type":"function","title":"Fallbacks - LangChain.Chains.LLMChain.__struct__/0","doc":"When running a chain, the `:with_fallbacks` option can be used to provide a\nlist of fallback chat models to try when a failure is encountered.\n\nWhen working with language models, you may often encounter issues from the\nunderlying APIs, whether these be rate limiting, downtime, or something else.\nTherefore, as you go to move your LLM applications into production it becomes\nmore and more important to safeguard against these. That's what fallbacks are\ndesigned to provide.\n\nA **fallback** is an alternative plan that may be used in an emergency.\n\nA `before_fallback` function can be provided to alter or return a different\nchain to use with the fallback LLM model. This is important because often, the\nprompts needed for will differ for a fallback LLM. This means if your OpenAI\ncompletion fails, a different prompt may be needed when retrying it with an\nAnthropic fallback.","ref":"LangChain.Chains.LLMChain.html#__struct__/0-fallbacks"},{"type":"function","title":"Fallback for LLM API Errors - LangChain.Chains.LLMChain.__struct__/0","doc":"This is perhaps the most common use case for fallbacks. A request to an LLM\nAPI can fail for a variety of reasons - the API could be down, you could have\nhit rate limits, any number of things. Therefore, using fallbacks can help\nprotect against these types of failures.","ref":"LangChain.Chains.LLMChain.html#__struct__/0-fallback-for-llm-api-errors"},{"type":"function","title":"Fallback Examples - LangChain.Chains.LLMChain.__struct__/0","doc":"A simple fallback that tries a different LLM chat model\n\n    fallback_llm = ChatAnthropic.new!(%{stream: false})\n\n    {:ok, updated_chain} =\n      %{llm: ChatOpenAI.new!(%{stream: false})}\n      |> LLMChain.new!()\n      |> LLMChain.add_message(Message.new_system!(\"OpenAI system prompt\"))\n      |> LLMChain.add_message(Message.new_user!(\"Why is the sky blue?\"))\n      |> LLMChain.run(with_fallbacks: [fallback_llm])\n\nNote the `with_fallbacks: [fallback_llm]` option when running the chain.\n\nThis example uses the `:before_fallback` option to provide a function that can\nmodify or return an alternate chain when used with a certain LLM. Also note\nthe utility function `LangChain.Utils.replace_system_message!/2` is used for\nswapping out the system message when falling back to a different LLM.\n\n    fallback_llm = ChatAnthropic.new!(%{stream: false})\n\n    {:ok, updated_chain} =\n      %{llm: ChatOpenAI.new!(%{stream: false})}\n      |> LLMChain.new!()\n      |> LLMChain.add_message(Message.new_system!(\"OpenAI system prompt\"))\n      |> LLMChain.add_message(Message.new_user!(\"Why is the sky blue?\"))\n      |> LLMChain.run(\n        with_fallbacks: [fallback_llm],\n        before_fallback: fn chain ->\n          case chain.llm do\n            %ChatAnthropic{} ->\n              # replace the system message\n              %LLMChain{\n                chain\n                | messages:\n                    Utils.replace_system_message!(\n                      chain.messages,\n                      Message.new_system!(\"Anthropic system prompt\")\n                    )\n              }\n\n            _open_ai ->\n              chain\n          end\n        end\n      )\n\nSee `LangChain.Chains.LLMChain.run/2` for more details.","ref":"LangChain.Chains.LLMChain.html#__struct__/0-fallback-examples"},{"type":"function","title":"Run Until Tool Used - LangChain.Chains.LLMChain.__struct__/0","doc":"The `run_until_tool_used/3` function makes it easy to instruct an LLM to use a\nset of tools and then call a specific tool to present the results. This is\nparticularly useful for complex workflows where you want the LLM to perform\nmultiple operations and then finalize with a specific action.\n\nThis works well for receiving a final structured output after multiple tools\nare used.\n\nWhen the specified tool is successfully called, the chain stops processing and\nreturns the result. This prevents unnecessary additional LLM calls and\nprovides a clear termination point for your workflow.\n\n    {:ok, %LLMChain{} = updated_chain, %ToolResult{} = tool_result} =\n      %{llm: ChatOpenAI.new!(%{stream: false})}\n      |> LLMChain.new!()\n      |> LLMChain.add_tools([special_search, report_results])\n      |> LLMChain.add_message(Message.new_system!())\n      |> LLMChain.add_message(Message.new_user!(\"...\"))\n      |> LLMChain.run_until_tool_used(\"final_summary\")\n\nThe function returns a tuple with three elements:\n- `:ok` - Indicating success\n- The updated chain with all messages and tool calls\n- The specific tool result that matched the requested tool name\n\nTo prevent runaway function calls, a default `max_runs` value of 25 is set.\nYou can adjust this as needed:\n\n    # Allow up to 50 runs before timing out\n    LLMChain.run_until_tool_used(chain, \"final_summary\", max_runs: 50)\n\nThe function also supports fallbacks, allowing you to gracefully handle LLM\nfailures:\n\n    LLMChain.run_until_tool_used(chain, \"final_summary\",\n      max_runs: 10,\n      with_fallbacks: [fallback_llm],\n      before_fallback: fn chain ->\n        # Modify chain before using fallback LLM\n        chain\n      end\n    )\n\nSee `LangChain.Chains.LLMChain.run_until_tool_used/3` for more details.","ref":"LangChain.Chains.LLMChain.html#__struct__/0-run-until-tool-used"},{"type":"function","title":"LangChain.Chains.LLMChain.add_callback/2","doc":"Add another callback to the list of callbacks.","ref":"LangChain.Chains.LLMChain.html#add_callback/2"},{"type":"function","title":"LangChain.Chains.LLMChain.add_message/2","doc":"Add a received Message struct to the chain. The LLMChain tracks the\n`last_message` received and the complete list of messages exchanged. Depending\non the message role, the chain may be in a pending or incomplete state where\na response from the LLM is anticipated.","ref":"LangChain.Chains.LLMChain.html#add_message/2"},{"type":"function","title":"LangChain.Chains.LLMChain.add_messages/2","doc":"Add a set of Message structs to the chain. This enables quickly building a chain\nfor submitting to an LLM.","ref":"LangChain.Chains.LLMChain.html#add_messages/2"},{"type":"function","title":"LangChain.Chains.LLMChain.add_tools/2","doc":"Add a tool to an LLMChain.","ref":"LangChain.Chains.LLMChain.html#add_tools/2"},{"type":"function","title":"LangChain.Chains.LLMChain.apply_deltas/2","doc":"Apply a list of deltas to the chain. When the final delta is received that\ncompletes the message, the LLMChain is updated to clear the `delta` and the\n`last_message` and list of messages are updated. The message is processed and\nfires any registered callbacks.","ref":"LangChain.Chains.LLMChain.html#apply_deltas/2"},{"type":"function","title":"LangChain.Chains.LLMChain.apply_prompt_templates/3","doc":"Apply a set of PromptTemplates to the chain. The list of templates can also\ninclude Messages with no templates. Provide the inputs to apply to the\ntemplates for rendering as a message. The prepared messages are applied to the\nchain.","ref":"LangChain.Chains.LLMChain.html#apply_prompt_templates/3"},{"type":"function","title":"LangChain.Chains.LLMChain.cancel_delta/2","doc":"Remove an incomplete MessageDelta from `delta` and add a Message with the\ndesired status to the chain.","ref":"LangChain.Chains.LLMChain.html#cancel_delta/2"},{"type":"function","title":"LangChain.Chains.LLMChain.common_validation/1","doc":"","ref":"LangChain.Chains.LLMChain.html#common_validation/1"},{"type":"function","title":"LangChain.Chains.LLMChain.delta_to_message_when_complete/1","doc":"Convert any hanging delta of the chain to a message and append to the chain.\n\nIf the delta is `nil`, the chain is returned unmodified.","ref":"LangChain.Chains.LLMChain.html#delta_to_message_when_complete/1"},{"type":"function","title":"LangChain.Chains.LLMChain.drop_delta/1","doc":"Drop the current delta. This is useful when needing to ignore a partial or\ncomplete delta because the message may be handled in a different way.","ref":"LangChain.Chains.LLMChain.html#drop_delta/1"},{"type":"function","title":"LangChain.Chains.LLMChain.execute_tool_call/3","doc":"Execute the tool call with the tool. Returns the tool's message response.","ref":"LangChain.Chains.LLMChain.html#execute_tool_call/3"},{"type":"function","title":"LangChain.Chains.LLMChain.execute_tool_calls/2","doc":"If the `last_message` from the Assistant includes one or more `ToolCall`s, then the linked\ntool is executed. If there is no `last_message` or the `last_message` is\nnot a `tool_call`, the LLMChain is returned with no action performed.\nThis makes it safe to call any time.\n\nThe `context` is additional data that will be passed to the executed tool.\nThe value given here will override any `custom_context` set on the LLMChain.\nIf not set, the global `custom_context` is used.","ref":"LangChain.Chains.LLMChain.html#execute_tool_calls/2"},{"type":"function","title":"LangChain.Chains.LLMChain.increment_current_failure_count/1","doc":"Increments the internal current_failure_count. Returns and incremented and\nupdated struct.","ref":"LangChain.Chains.LLMChain.html#increment_current_failure_count/1"},{"type":"function","title":"LangChain.Chains.LLMChain.merge_delta/2","doc":"Merge a received MessageDelta struct into the chain's current delta. The\nLLMChain tracks the current merged MessageDelta state. This is able to merge\nin TokenUsage received after the final delta.","ref":"LangChain.Chains.LLMChain.html#merge_delta/2"},{"type":"function","title":"LangChain.Chains.LLMChain.merge_deltas/2","doc":"Merge a list of deltas into the chain.","ref":"LangChain.Chains.LLMChain.html#merge_deltas/2"},{"type":"function","title":"LangChain.Chains.LLMChain.message_processors/2","doc":"Register a set of processors to be applied to received assistant messages.","ref":"LangChain.Chains.LLMChain.html#message_processors/2"},{"type":"function","title":"LangChain.Chains.LLMChain.new/1","doc":"Start a new LLMChain configuration.\n\n    {:ok, chain} = LLMChain.new(%{\n      llm: %ChatOpenAI{model: \"gpt-3.5-turbo\", stream: true},\n      messages: [%Message.new_system!(\"You are a helpful assistant.\")]\n    })","ref":"LangChain.Chains.LLMChain.html#new/1"},{"type":"function","title":"LangChain.Chains.LLMChain.new!/1","doc":"Start a new LLMChain configuration and return it or raise an error if invalid.\n\n    chain = LLMChain.new!(%{\n      llm: %ChatOpenAI{model: \"gpt-3.5-turbo\", stream: true},\n      messages: [%Message.new_system!(\"You are a helpful assistant.\")]\n    })","ref":"LangChain.Chains.LLMChain.html#new!/1"},{"type":"function","title":"LangChain.Chains.LLMChain.process_message/2","doc":"Process a newly message received from the LLM. Messages with a role of\n`:assistant` may be processed through the `message_processors` before being\ngenerally available or being notified through a callback.","ref":"LangChain.Chains.LLMChain.html#process_message/2"},{"type":"function","title":"LangChain.Chains.LLMChain.quick_prompt/2","doc":"Convenience function for setting the prompt text for the LLMChain using\nprepared text.","ref":"LangChain.Chains.LLMChain.html#quick_prompt/2"},{"type":"function","title":"LangChain.Chains.LLMChain.reset_current_failure_count/1","doc":"Reset the internal current_failure_count to 0. Useful after receiving a\nsuccessfully returned and processed message from the LLM.","ref":"LangChain.Chains.LLMChain.html#reset_current_failure_count/1"},{"type":"function","title":"LangChain.Chains.LLMChain.reset_current_failure_count_if/2","doc":"Reset the internal current_failure_count to 0 if the function provided returns\n`true`. Helps to make the change conditional.","ref":"LangChain.Chains.LLMChain.html#reset_current_failure_count_if/2"},{"type":"function","title":"LangChain.Chains.LLMChain.run/2","doc":"Run the chain on the LLM using messages and any registered functions. This\nformats the request for a ChatLLMChain where messages are passed to the API.\n\nWhen successful, it returns `{:ok, updated_chain}`","ref":"LangChain.Chains.LLMChain.html#run/2"},{"type":"function","title":"Options - LangChain.Chains.LLMChain.run/2","doc":"- `:mode` - It defaults to run the chain one time, stopping after receiving a\n  response from the LLM. Supports `:until_success` and\n  `:while_needs_response`.\n\n- `mode: :until_success` - (for non-interactive processing done by the LLM\n  where it may repeatedly fail and need to re-try) Repeatedly evaluates a\n  received message through any message processors, returning any errors to the\n  LLM until it either succeeds or exceeds the `max_retry_count`. This includes\n  evaluating received `ToolCall`s until they succeed. If an LLM makes 3\n  ToolCalls in a single message and 2 succeed while 1 fails, the success\n  responses are returned to the LLM with the failure response of the remaining\n  `ToolCall`, giving the LLM an opportunity to resend the failed `ToolCall`,\n  and only the failed `ToolCall` until it succeeds or exceeds the\n  `max_retry_count`. In essence, once we have a successful response from the\n  LLM, we don't return any more to it and don't want any further responses.\n\n- `mode: :while_needs_response` - (for interactive chats that make\n  `ToolCalls`) Repeatedly evaluates functions and submits to the LLM so long\n  as we still expect to get a response. Best fit for conversational LLMs where\n  a `ToolResult` is used by the LLM to continue. After all `ToolCall` messages\n  are evaluated, the `ToolResult` messages are returned to the LLM giving it\n  an opportunity to use the `ToolResult` information in an assistant response\n  message. In essence, this mode always gives the LLM the last word.\n\n- `with_fallbacks: [...]` - Provide a list of chat models to use as a fallback\n  when one fails. This helps a production system remain operational when an\n  API limit is reached, an LLM service is overloaded or down, or something\n  else new an exciting goes wrong.\n\n  When all fallbacks fail, a `%LangChainError{type: \"all_fallbacks_failed\"}`\n  is returned in the error response.\n\n- `before_fallback: fn chain -> modified_chain end` - A `before_fallback`\n  function is called before the LLM call is made. **NOTE: When provided, it\n  also fires for the first attempt.** This allows a chain to be modified or\n  replaced before running against the configured LLM. This is helpful, for\n  example, when a different system prompt is needed for Anthropic vs OpenAI.","ref":"LangChain.Chains.LLMChain.html#run/2-options"},{"type":"function","title":"Mode Examples - LangChain.Chains.LLMChain.run/2","doc":"**Use Case**: A chat with an LLM where functions are available to the LLM:\n\n    LLMChain.run(chain, mode: :while_needs_response)\n\nThis will execute any LLM called functions, returning the result to the LLM,\nand giving it a chance to respond to the results.\n\n**Use Case**: An application that exposes a function to the LLM, but we want\nto stop once the function is successfully executed. When errors are\nencountered, the LLM should be given error feedback and allowed to try again.\n\n    LLMChain.run(chain, mode: :until_success)","ref":"LangChain.Chains.LLMChain.html#run/2-mode-examples"},{"type":"function","title":"LangChain.Chains.LLMChain.run_until_tool_used/3","doc":"Run the chain until a specific tool call is made. This makes it easy for an\nLLM to make multiple tool calls and call a specific tool to return a result,\nsignaling the end of the operation.","ref":"LangChain.Chains.LLMChain.html#run_until_tool_used/3"},{"type":"function","title":"Options - LangChain.Chains.LLMChain.run_until_tool_used/3","doc":"- `max_runs`: The maximum number of times to run the chain. To prevent runaway\n  calls, it defaults to 25. When exceeded, a `%LangChainError{type: \"exceeded_max_runs\"}`\n  is returned in the error response.\n\n- `with_fallbacks: [...]` - Provide a list of chat models to use as a fallback\n  when one fails. This helps a production system remain operational when an\n  API limit is reached, an LLM service is overloaded or down, or something\n  else new an exciting goes wrong.\n\n  When all fallbacks fail, a `%LangChainError{type: \"all_fallbacks_failed\"}`\n  is returned in the error response.\n\n- `before_fallback: fn chain -> modified_chain end` - A `before_fallback`\n  function is called before the LLM call is made. **NOTE: When provided, it\n  also fires for the first attempt.** This allows a chain to be modified or\n  replaced before running against the configured LLM. This is helpful, for\n  example, when a different system prompt is needed for Anthropic vs OpenAI.","ref":"LangChain.Chains.LLMChain.html#run_until_tool_used/3-options"},{"type":"function","title":"LangChain.Chains.LLMChain.update_custom_context/3","doc":"Update the LLMChain's `custom_context` map. Passing in a `context_update` map\nwill by default merge the map into the existing `custom_context`.\n\nUse the `:as` option to:\n- `:merge` - Merge update changes in. Default.\n- `:replace` - Replace the context with the `context_update`.","ref":"LangChain.Chains.LLMChain.html#update_custom_context/3"},{"type":"type","title":"LangChain.Chains.LLMChain.message_processor/0","doc":"A message processor is an arity 2 function that takes an\n`LangChain.Chains.LLMChain` and a `LangChain.Message`. It is used to\n\"pre-process\" the received message from the LLM. Processors can be chained\ntogether to perform a sequence of transformations.\n\nThe return of the processor is a tuple with a keyword and a message. The\nkeyword is either `:cont` or `:halt`. If `:cont` is returned, the\nmessage is used as the next message in the chain. If `:halt` is returned, the\nhalting message is returned to the LLM as an error and no further processors\nwill handle the message.\n\nAn example of this is the `LangChain.MessageProcessors.JsonProcessor` which\nparses the message content as JSON and returns the parsed data as a map. If\nthe content is not valid JSON, the processor returns a halting message with an\nerror message for the LLM to respond to.","ref":"LangChain.Chains.LLMChain.html#t:message_processor/0"},{"type":"type","title":"LangChain.Chains.LLMChain.processor_return/0","doc":"The expected return types for a Message processor function. When successful,\nit returns a `:cont` with an Message to use as a replacement. When it\nfails, a `:halt` is returned along with an updated `LLMChain.t()` and a new\nuser message to be returned to the LLM reporting the error.","ref":"LangChain.Chains.LLMChain.html#t:processor_return/0"},{"type":"type","title":"LangChain.Chains.LLMChain.t/0","doc":"","ref":"LangChain.Chains.LLMChain.html#t:t/0"},{"type":"module","title":"LangChain.Chains.SummarizeConversationChain","doc":"When an AI conversation has many back-and-forth messages (from user to\nassistant to user to assistant, etc.), the number of messages and the total\ntoken count can be large. Large token counts present the following problems:\n\n- Increased cost/price per generation\n- Increased generation times\n- Risk of exceeding the total token limit, resulting in an error\n\nThis chain is run as a separate process to summarize and condense a separate\nconversation chain. It is assumed that the chain the user sees in the UI\nretains all their original messages and they are not seeing the full, raw\nmessage list.\n\nWe don't want to perform more work than we need to, so we'll only kick off the\nsummary process once the number of messages has reached some threshold, then\nwe'll retain a configured number of the most recent messages to help retain\ncontinuity for the conversation.","ref":"LangChain.Chains.SummarizeConversationChain.html"},{"type":"module","title":"Options - LangChain.Chains.SummarizeConversationChain","doc":"- `:llm` - The LLM to use for performing the summarization. There is no need for streaming.\n- `:keep_count` - The number of raw messages to retain. It will be the\n  most recent messages and defaults to 2 (a user and assistant message).\n- `:threshold_count` - The total number of messages (excluding the system\n  message) that must be present before the summarizing operation is performed.\n  Running the summarization on a short conversation chain will return the\n  chain unchanged and not make any calls to an LLM.\n- `:override_system_prompt` - When the system prompt should be customized for the instructions on how to summarize, this can be used to provide a customized replacement of the system prompt.\n- `:messages` - When explicit control of multiple messages is needed, they can be provided as a list. They can be `LangChain.PromptTemplate`s and the concatenated list of messages will be in the `@conversation` param. When this is used, any value in  `:override_system_prompt` is ignored.","ref":"LangChain.Chains.SummarizeConversationChain.html#module-options"},{"type":"module","title":"Examples - LangChain.Chains.SummarizeConversationChain","doc":"A basic example that processes the messages in a separate LLMChain, returning\nan updated chain with summarized contents.\n\n\n    {:ok, summarized_chain} =\n      %{\n        llm: ChatOpenAI.new!(%{model: \"gpt-4o-mini\", stream: false}),\n        keep_count: 2,\n        threshold_count: 6\n      }\n      |> SummarizeConversationChain.new!()\n      |> SummarizeConversationChain.summarize(chain_to_summarize)\n\nUsing a `:with_fallback` option to still try and summarize if the LLM errors from the Azure host OpenAI.\n\n    # Azure configured OpenAI LLM\n    fallback_llm =\n      ChatOpenAI.new!(%{\n        stream: false,\n        endpoint: System.fetch_env!(\"AZURE_OPENAI_ENDPOINT\"),\n        api_key: System.fetch_env!(\"AZURE_OPENAI_KEY\")\n      })\n\n    {:ok, summarized_chain} =\n      %{\n        llm: ChatOpenAI.new!(%{model: \"gpt-4o-mini\", stream: false}),\n        keep_count: 2,\n        threshold_count: 6\n      }\n      |> SummarizeConversationChain.new!()\n      |> SummarizeConversationChain.evaluate(chain_to_summarize, with_fallbacks: [fallback_llm])","ref":"LangChain.Chains.SummarizeConversationChain.html#module-examples"},{"type":"function","title":"LangChain.Chains.SummarizeConversationChain.combine_messages_for_summary_text/2","doc":"Create a single text message to represent the current set of messages being\nsummarized from the `to_summarize` chain. Uses the settings from SummarizeConversationChain.\nA `nil` is returned when the threshold has not been reached for running the\nsummary procedure.\n\nThis combines the user and assistant messages into a single string that can be summarized. This does not summarize a `system` message and does not include the last `n` messages for the `keep_count`.","ref":"LangChain.Chains.SummarizeConversationChain.html#combine_messages_for_summary_text/2"},{"type":"function","title":"LangChain.Chains.SummarizeConversationChain.new/1","doc":"Start a new SummarizeConversationChain configuration.\n\n    {:ok, summarizer} = SummarizeConversationChain.new(%{\n      llm: %ChatOpenAI{model: \"gpt-3.5-turbo\", stream: false},\n      keep_count: 2,\n      threshold_count: 6\n    })","ref":"LangChain.Chains.SummarizeConversationChain.html#new/1"},{"type":"function","title":"LangChain.Chains.SummarizeConversationChain.new!/1","doc":"Start a new SummarizeConversationChain and return it or raise an error if invalid.\n\n    chain = SummarizeConversationChain.new!(%{\n      llm: %ChatOpenAI{model: \"gpt-3.5-turbo\", stream: false},\n      keep_count: 2,\n      threshold_count: 6\n    })","ref":"LangChain.Chains.SummarizeConversationChain.html#new!/1"},{"type":"function","title":"LangChain.Chains.SummarizeConversationChain.run/3","doc":"Run a SummarizeConversationChain to summarize a text representation of a sequence of user and assistant messages.\n\n    new_title = SummarizeConversationChain.new!(%{\n      llm: %ChatOpenAI{model: \"gpt-3.5-turbo\", stream: false},\n      keep_count: 2,\n      threshold_count: 6\n    })\n    |> SummarizeConversationChain.run(text_to_summarize)","ref":"LangChain.Chains.SummarizeConversationChain.html#run/3"},{"type":"function","title":"LangChain.Chains.SummarizeConversationChain.summarize/3","doc":"Summarize the `to_summarize` LLMChain using the\n`%SummarizeConversationChain{}` configuration and `opts`. Returns a new,\npotentially modified `LLMChain` after completing the summarization process.\n\nIf the `threshold_count` is greater than the current number of summarizable\nmessages (ie user and assistant roles), then nothing is modified and the\noriginal LLMChain is returned.","ref":"LangChain.Chains.SummarizeConversationChain.html#summarize/3"},{"type":"function","title":"Options - LangChain.Chains.SummarizeConversationChain.summarize/3","doc":"- `:with_fallbacks` - An optional set of fallback LLMs to use if the\n  summarization process fails. See `LangChain.Chains.LLMChain.run/2` for\n  details.","ref":"LangChain.Chains.SummarizeConversationChain.html#summarize/3-options"},{"type":"type","title":"LangChain.Chains.SummarizeConversationChain.t/0","doc":"","ref":"LangChain.Chains.SummarizeConversationChain.html#t:t/0"},{"type":"module","title":"LangChain.Chains.TextToTitleChain","doc":"A convenience chain for turning a user's prompt text into a summarized title\nfor the anticipated conversation.","ref":"LangChain.Chains.TextToTitleChain.html"},{"type":"module","title":"Basic Examples - LangChain.Chains.TextToTitleChain","doc":"A basic example that generates a title\n\n    llm = ChatOpenAI.new!(%{model: \"gpt-3.5-turbo\", stream: false, seed: 0})\n    user_text = \"Let's start a new blog post about the magical properties of pineapple cookies.\"\n\n    %{\n      llm: llm,\n      input_text: user_text\n    }\n    |> TextToTitleChain.new!()\n    |> TextToTitleChain.evaluate()\n\n    #=> \"Magical Properties of Pineapple Cookies Blog Post\"","ref":"LangChain.Chains.TextToTitleChain.html#module-basic-examples"},{"type":"module","title":"Examples using Title Examples - LangChain.Chains.TextToTitleChain","doc":"Want to get more consistent titles?\n\nLLMs are pretty bad at following instructions for text length. However, we can\nprovide examples titles for the LLM to follow in format style and length. We\nget the added benefit of getting more consistently formatted titles.\n\nThis is the same example, however now we provide other title examples to the\nLLM to follow for consistency.\n\n   llm = ChatOpenAI.new!(%{model: \"gpt-3.5-turbo\", stream: false, seed: 0})\n    user_text = \"Let's start a new blog post about the magical properties of\n    pineapple cookies.\"\n\n    %{\n      llm: llm,\n      input_text: user_text,\n      examples: [\n        \"Blog Post: Making Delicious and Healthy Smoothies\",\n        \"System Email: Notifying Users of Planned Downtime\"\n      ]\n    }\n    |> TextToTitleChain.new!()\n    |> TextToTitleChain.evaluate()\n\n    #=> \"Blog Post: Exploring the Magic of Pineapple Cookies\"","ref":"LangChain.Chains.TextToTitleChain.html#module-examples-using-title-examples"},{"type":"module","title":"Overriding the System Prompt - LangChain.Chains.TextToTitleChain","doc":"For more explicit control of how titles are generated, an `override_system_prompt` can be provided.\n\n    %{\n      llm: llm,\n      input_text: user_text,\n      override_system_prompt: ~s|\n        You expertly summarize the User Text into a short 3 or 4 word title to represent a conversation in a positive way.|\n    }\n    |> TextToTitleChain.new!()\n    |> TextToTitleChain.evaluate()","ref":"LangChain.Chains.TextToTitleChain.html#module-overriding-the-system-prompt"},{"type":"module","title":"Using a Fallback - LangChain.Chains.TextToTitleChain","doc":"If the primary LLM fails to respond successfully, one or more fallback LLMs can be specified.\n\n    %{\n      llm: primary_llm,\n      input_text: user_text\n    }\n    |> TextToTitleChain.new!()\n    |> TextToTitleChain.evaluate(with_fallbacks: [fallback_llm])","ref":"LangChain.Chains.TextToTitleChain.html#module-using-a-fallback"},{"type":"function","title":"LangChain.Chains.TextToTitleChain.evaluate/2","doc":"Runs the TextToTitleChain and evaluates the result to return the final answer.\n\nIf unable to generate a title, the `fallback_title` is returned.","ref":"LangChain.Chains.TextToTitleChain.html#evaluate/2"},{"type":"function","title":"Option - LangChain.Chains.TextToTitleChain.evaluate/2","doc":"- `:with_fallbacks` - Supports the `with_fallbacks: [fallback_llm]` where one or more additional LLMs can be specified as a backup when the preferred LLM fails.","ref":"LangChain.Chains.TextToTitleChain.html#evaluate/2-option"},{"type":"function","title":"LangChain.Chains.TextToTitleChain.new/1","doc":"Start a new TextToTitleChain configuration.\n\n    {:ok, chain} = TextToTitleChain.new(%{\n      llm: %ChatOpenAI{model: \"gpt-3.5-turbo\", stream: false},\n      input_text: \"Let's create a marketing blog post about our new product 'Fuzzy Furries'\"\n    })","ref":"LangChain.Chains.TextToTitleChain.html#new/1"},{"type":"function","title":"LangChain.Chains.TextToTitleChain.new!/1","doc":"Start a new TextToTitleChain and return it or raise an error if invalid.\n\n    chain = TextToTitleChain.new!(%{\n      llm: %ChatOpenAI{model: \"gpt-3.5-turbo\", stream: false},\n      input_text: \"Let's create a marketing blog post about our new product 'Fuzzy Furries'\"\n    })","ref":"LangChain.Chains.TextToTitleChain.html#new!/1"},{"type":"function","title":"LangChain.Chains.TextToTitleChain.run/2","doc":"Run a simple LLMChain to summarize the user's prompt into a title for the\nconversation. Uses the provided model. Recommend faster, simpler LLMs without\nstreaming.\n\nIf it fails to summarize to a title, it returns the default text.\n\n    new_title = TextToTitleChain.new!(%{\n      llm: %ChatOpenAI{model: \"gpt-3.5-turbo\", stream: false},\n      input_text: \"Let's create a marketing blog post about our new product 'Fuzzy Furries'\"\n    })\n    |> TextToTitleChain.run()","ref":"LangChain.Chains.TextToTitleChain.html#run/2"},{"type":"type","title":"LangChain.Chains.TextToTitleChain.t/0","doc":"","ref":"LangChain.Chains.TextToTitleChain.html#t:t/0"},{"type":"module","title":"LangChain.Message","doc":"Models a complete `Message` for a chat LLM.","ref":"LangChain.Message.html"},{"type":"module","title":"Roles - LangChain.Message","doc":"- `:system` - a system message. Typically just one and it occurs first as a\n  primer for how the LLM should behave.\n\n- `:user` - The user or application responses. Typically represents the\n  \"human\" element of the exchange.\n\n- `:assistant` - Responses coming back from the LLM. This includes one or more\n  \"tool calls\", requesting that the system execute a tool on behalf of the LLM\n  and return the response.\n\n- `:tool` - A message for returning the result of executing a `tool` request.","ref":"LangChain.Message.html#module-roles"},{"type":"module","title":"Tools - LangChain.Message","doc":"A `tool_call` comes from the `:assistant` role. The `tool_id` identifies which\nof the available tool's to execute.\n\nCreate a message of role `:tool` to provide the system responses for one or\nmore tool requests. A `ToolResult` handles the response back to the LLM.","ref":"LangChain.Message.html#module-tools"},{"type":"module","title":"User Content Parts - LangChain.Message","doc":"Some LLMs support multi-modal messages. This means the user's message content\ncan be text and/or image data. Within the LLMs, these are often referred to as\n\"Vision\", meaning you can provide text like \"Please identify the what this is\nan image of\" and provide an image.\n\nUser Content Parts are implemented through `LangChain.Message.ContentPart`. A\nlist of them can be supplied as the \"content\" for a message. Only a few LLMs\nsupport it, and they may require using specific models trained for it. See the\ndocumentation for the LLM or service for details on their level of support.","ref":"LangChain.Message.html#module-user-content-parts"},{"type":"module","title":"Assistant Content Parts - LangChain.Message","doc":"Assistant Content Parts are implemented through\n`LangChain.Message.ContentPart`. A list of them can be supplied as the\n\"content\" for a message. Only a few LLMs support it, and they may require\nusing specific models trained for it. See the documentation for the LLM or\nservice for details on their level of support.\n\nAn example of this is Anthropic's Claude 3.7 Sonnet and it's \"thinking\"\ncontent parts. The service may also return redacted_thinking content parts\nthat can be sent back to maintain continuity of the conversation.\n\nThis also supports the idea of an assistant returning an image along with\ntext.","ref":"LangChain.Message.html#module-assistant-content-parts"},{"type":"module","title":"Processed Content - LangChain.Message","doc":"The `processed_content` field is a handy place to store the results of\nprocessing a message and needing to hold on to the processed value and store\nit with the message.\n\nThis is particularly helpful for a `LangChain.MessageProcessors.JsonProcessor`\nthat can process an assistant message and store the processed value on the\nmessage itself.\n\nIt is intended for assistant messages when a message processor is applied.\nThis contains the results of the processing. This allows the `content` to\nreflect what was actually returned from the LLM so it can easily be sent back\nto the LLM as a part of the entire conversation.","ref":"LangChain.Message.html#module-processed-content"},{"type":"module","title":"Examples - LangChain.Message","doc":"A basic system message example:\n\n    alias LangChain.Message\n\n    Message.new_system!(\"You are a helpful assistant.\")\n\nA basic user message:\n\n    Message.new_user!(\"Who is Prime Minister of the moon?\")\n\nA multi-part user message:\n\n    alias LangChain.Message.ContentPart\n\n    Message.new_user!([\n      ContentPart.text!(\"What is in this picture?\"),\n      ContentPart.image_url!(\"https://upload.wikimedia.org/wikipedia/commons/thumb/d/dd/Gfp-wisconsin-madison-the-nature-boardwalk.jpg/2560px-Gfp-wisconsin-madison-the-nature-boardwalk.jpg\")\n    ]","ref":"LangChain.Message.html#module-examples"},{"type":"function","title":"LangChain.Message.append_tool_result/2","doc":"Append a `ToolResult` to a message. A result can only be added to a `:tool`\nrole message.","ref":"LangChain.Message.html#append_tool_result/2"},{"type":"function","title":"LangChain.Message.is_empty?/1","doc":"Determines if a message is considered \"empty\" and likely indicates a failure.\n\nThis is particularly useful for detecting failure patterns with Anthropic models\nwhere the LLM may return empty responses when it encounters issues.","ref":"LangChain.Message.html#is_empty?/1"},{"type":"function","title":"Examples - LangChain.Message.is_empty?/1","doc":"iex> alias LangChain.Message\n    iex> Message.is_empty?(%Message{role: :assistant, content: [], tool_calls: [], status: :complete})\n    true\n\n    iex> alias LangChain.Message\n    iex> Message.is_empty?(%Message{role: :assistant, content: \"Hello\", tool_calls: [], status: :complete})\n    false\n\n    iex> alias LangChain.Message\n    iex> Message.is_empty?(%Message{role: :user, content: [], status: :complete})\n    false","ref":"LangChain.Message.html#is_empty?/1-examples"},{"type":"function","title":"LangChain.Message.is_tool_call?/1","doc":"Return if a Message is a tool_call.","ref":"LangChain.Message.html#is_tool_call?/1"},{"type":"function","title":"LangChain.Message.is_tool_related?/1","doc":"Return if a Message is tool related. It may be a tool call or a tool result.","ref":"LangChain.Message.html#is_tool_related?/1"},{"type":"function","title":"LangChain.Message.new/1","doc":"Build a new message and return an `:ok`/`:error` tuple with the result.","ref":"LangChain.Message.html#new/1"},{"type":"function","title":"LangChain.Message.new!/1","doc":"Build a new message and return it or raise an error if invalid.","ref":"LangChain.Message.html#new!/1"},{"type":"function","title":"LangChain.Message.new_assistant/1","doc":"Create a new assistant message which represents a response from the AI or LLM.","ref":"LangChain.Message.html#new_assistant/1"},{"type":"function","title":"LangChain.Message.new_assistant!/1","doc":"Create a new assistant message which represents a response from the AI or LLM.","ref":"LangChain.Message.html#new_assistant!/1"},{"type":"function","title":"Examples - LangChain.Message.new_assistant!/1","doc":"# Create a simple text message\n    iex> new_assistant!(\"I'll help you with that.\")\n    %LangChain.Message{role: :assistant, content: [LangChain.Message.ContentPart.text!(\"I'll help you with that.\")], tool_calls: []}\n\n    # Create with content parts\n    iex> new_assistant!([\n    ...>   LangChain.Message.ContentPart.text!(\"Here's my response\"),\n    ...>   LangChain.Message.ContentPart.text!(\"And some additional thoughts\")\n    ...> ])\n    %LangChain.Message{role: :assistant, content: [%LangChain.Message.ContentPart{content: \"Here's my response\"}, %LangChain.Message.ContentPart{content: \"And some additional thoughts\"}], tool_calls: []}\n\n    # Create with tool calls\n    iex> new_assistant!(%{\n    ...>   tool_calls: [\n    ...>     LangChain.Message.ToolCall.new!(%{call_id: \"1\", name: \"calculator\", arguments: %{x: 2, y: 3}})\n    ...>   ]\n    ...> })\n    %LangChain.Message{role: :assistant, tool_calls: [%LangChain.Message.ToolCall{call_id: \"1\", name: \"calculator\", arguments: %{x: 2, y: 3}, status: :complete}]}","ref":"LangChain.Message.html#new_assistant!/1-examples"},{"type":"function","title":"LangChain.Message.new_system/1","doc":"Create a new system message which can prime the AI/Assistant for how to\nrespond.","ref":"LangChain.Message.html#new_system/1"},{"type":"function","title":"LangChain.Message.new_system!/1","doc":"Create a new system message which can prime the AI/Assistant for how to\nrespond.","ref":"LangChain.Message.html#new_system!/1"},{"type":"function","title":"LangChain.Message.new_tool_result/1","doc":"Create a new `tool` message to represent the result of a tool's execution.","ref":"LangChain.Message.html#new_tool_result/1"},{"type":"function","title":"Attributes - LangChain.Message.new_tool_result/1","doc":"- `:tool_results` - a list of tool `ToolResult` structs.\n- `:content` - Text content returned from the LLM.","ref":"LangChain.Message.html#new_tool_result/1-attributes"},{"type":"function","title":"LangChain.Message.new_tool_result!/1","doc":"Create a new tool response message to return the result of an executed\ntool.","ref":"LangChain.Message.html#new_tool_result!/1"},{"type":"function","title":"Attributes - LangChain.Message.new_tool_result!/1","doc":"- `:tool_results` - a list of tool `ToolResult` structs.\n- `:content` - Text content returned from the LLM.","ref":"LangChain.Message.html#new_tool_result!/1-attributes"},{"type":"function","title":"LangChain.Message.new_user/1","doc":"Create a new user message which represents a human message or a message from\nthe application.","ref":"LangChain.Message.html#new_user/1"},{"type":"function","title":"LangChain.Message.new_user!/1","doc":"Create a new user message which represents a human message or a message from\nthe application.","ref":"LangChain.Message.html#new_user!/1"},{"type":"function","title":"LangChain.Message.tool_had_errors?/1","doc":"Return `true` if the message is a `tool` response and any of the `ToolResult`s\nended in an error. Returns `false` if not a `tool` response or all\n`ToolResult`s succeeded.","ref":"LangChain.Message.html#tool_had_errors?/1"},{"type":"function","title":"LangChain.Message.validate_tool_results_list_type/1","doc":"","ref":"LangChain.Message.html#validate_tool_results_list_type/1"},{"type":"function","title":"LangChain.Message.validate_tool_results_required/1","doc":"","ref":"LangChain.Message.html#validate_tool_results_required/1"},{"type":"type","title":"LangChain.Message.status/0","doc":"","ref":"LangChain.Message.html#t:status/0"},{"type":"type","title":"LangChain.Message.t/0","doc":"","ref":"LangChain.Message.html#t:t/0"},{"type":"module","title":"LangChain.Message.ContentPart","doc":"Models a `ContentPart`. ContentParts are now used for multi-modal support in\nboth messages and tool results. This enables richer responses, allowing text,\nimages, files, and thinking blocks to be combined in a single message or tool\nresult.","ref":"LangChain.Message.ContentPart.html"},{"type":"module","title":"Types - LangChain.Message.ContentPart","doc":"- `:text` - The message part is text.\n- `:image_url` - The message part is a URL to an image.\n- `:image` - The message part is image data that is base64 encoded text.\n- `:file` - The message part is file data that is base64 encoded text.\n- `:file_url` - The message part is a URL to a file.\n- `:thinking` - A thinking block from a reasoning model like Anthropic.\n- `:unsupported` - A part that is not supported but may need to be present.\n  This includes Anthropic's `redacted_thinking` block which has no value in\n  being displayed because it is encrypted, but can be provided back to the LLM\n  to maintain reasoning continuity. The specific parts of the data are stored\n  in `:options`.","ref":"LangChain.Message.ContentPart.html#module-types"},{"type":"module","title":"Fields - LangChain.Message.ContentPart","doc":"- `:content` - Text content.\n- `:options` - Options are a keyword list of values that may be specific to\n  the LLM for a particular message type. For example, multi-modal message\n  (ones that include image data) use the `:media` option to specify the\n  mimetype information. Options may also contain key-value settings like\n  `cache_control: true` for models like Anthropic that support caching.\n\n  When receiving content parts like with Anthropic Claude's thinking model,\n  the options may contain LLM specific data that is recommended to be\n  preserved like a `signature` or `redacted_thinking` data used by the LLM.","ref":"LangChain.Message.ContentPart.html#module-fields"},{"type":"module","title":"Image mime types - LangChain.Message.ContentPart","doc":"The `:media` option is used to specify the mime type of the image. Various\nLLMs handle this differently or perhaps not at all.\n\nExamples:\n\n- `media: :jpg` - turns into `\"image/jpeg\"` or `\"image/jpg\"`, depending on\n  what the LLM accepts.\n- `media: :png` - turns into `\"image/png\"`\n- `media: \"image/webp\"` - stays as `\"image/webp\"`. Any specified string value\n  is passed through unchanged. This allows for future formats to be supported\n  quickly.\n- When omitted, the LLM may error or some will accept it but may require the\n  `base64` encoded content data to be prefixed with the mime type information.\n  Basically, you must handle the content needs yourself.","ref":"LangChain.Message.ContentPart.html#module-image-mime-types"},{"type":"function","title":"LangChain.Message.ContentPart.content_to_string/2","doc":"Convert \"content\" to a string. Content may be `nil`, a string, or a list of ContentParts.","ref":"LangChain.Message.ContentPart.html#content_to_string/2"},{"type":"function","title":"LangChain.Message.ContentPart.file!/2","doc":"Create a new ContentPart that contains a file encoded as base64 data.","ref":"LangChain.Message.ContentPart.html#file!/2"},{"type":"function","title":"LangChain.Message.ContentPart.file_url!/2","doc":"Create a new ContentPart that contains a URL to an file. Raises an exception if not valid.","ref":"LangChain.Message.ContentPart.html#file_url!/2"},{"type":"function","title":"LangChain.Message.ContentPart.image!/2","doc":"Create a new ContentPart that contains an image encoded as base64 data. Raises\nan exception if not valid.","ref":"LangChain.Message.ContentPart.html#image!/2"},{"type":"function","title":"Options - LangChain.Message.ContentPart.image!/2","doc":"- `:media` - Provide the \"media type\" for the image. Examples: \"image/jpeg\",\n  \"image/png\", etc. ChatGPT does not require this but other LLMs may.\n- `:detail` - if the LLM supports it, most images must be resized or cropped\n  before given to the LLM for analysis. A detail option may specify the level\n  detail of the image to present to the LLM. The higher the detail, the more\n  tokens consumed. Currently only supported by OpenAI and the values of \"low\",\n  \"high\", and \"auto\".\n\nChatGPT requires media type information to prefix the base64 content. Setting\nthe `media: \"image/jpeg\"` type will do that. Otherwise the data must be\nprovided with the required prefix.\n\nAnthropic requires the media type information to be submitted as separate\ninformation with the JSON request. This media option provides an abstraction\nto normalize the behavior.","ref":"LangChain.Message.ContentPart.html#image!/2-options"},{"type":"function","title":"LangChain.Message.ContentPart.image_url!/2","doc":"Create a new ContentPart that contains a URL to an image. Raises an exception if not valid.","ref":"LangChain.Message.ContentPart.html#image_url!/2"},{"type":"function","title":"LangChain.Message.ContentPart.merge_part/2","doc":"Merge two `ContentPart` structs for the same index in a MessageDelta. The\nfirst `ContentPart` is the `primary` one that smaller deltas are merged into.\nThe primary is what is being accumulated.\n\nA set of ContentParts can be merged like this:\n\n    Enum.reduce(list_of_content_parts, nil, fn new_part, acc ->\n      ContentPart.merge_part(acc, new_part)\n    end)","ref":"LangChain.Message.ContentPart.html#merge_part/2"},{"type":"function","title":"LangChain.Message.ContentPart.new/1","doc":"Build a new message and return an `:ok`/`:error` tuple with the result.","ref":"LangChain.Message.ContentPart.html#new/1"},{"type":"function","title":"LangChain.Message.ContentPart.new!/1","doc":"Build a new message and return it or raise an error if invalid.","ref":"LangChain.Message.ContentPart.html#new!/1"},{"type":"function","title":"Example - LangChain.Message.ContentPart.new!/1","doc":"ContentPart.new!(%{type: :text, content: \"Greetings!\"})\n\n    ContentPart.new!(%{type: :image_url, content: \"https://example.com/images/house.jpg\"})\n\n    ContentPart.new!(%{type: :thinking, content: \"I've been asked...\", options: [signature: \"SIGNATURE_DATA\"]}\n\n    ContentPart.new!(%{type: :unsupported, content: \"redacted_data\", options: [type: \"redacted_thinking\"]}","ref":"LangChain.Message.ContentPart.html#new!/1-example"},{"type":"function","title":"LangChain.Message.ContentPart.parts_to_string/2","doc":"Helper function for easily getting plain text from a list of ContentParts.\n\nThis function processes a list of ContentParts and joins the text parts together\nusing \"\n\n\" characters. Only parts where `type: :text` are used. All other parts\nare ignored.","ref":"LangChain.Message.ContentPart.html#parts_to_string/2"},{"type":"function","title":"Examples - LangChain.Message.ContentPart.parts_to_string/2","doc":"iex> parts = [\n    ...>   text!(\"Hello\"),\n    ...>   image!(\"base64data\"),\n    ...>   text!(\"world\")\n    ...> ]\n    iex> parts_to_string(parts)\n    \"Hello\\n\\nworld\"\n\n    iex> parts_to_string([])\n    nil","ref":"LangChain.Message.ContentPart.html#parts_to_string/2-examples"},{"type":"function","title":"LangChain.Message.ContentPart.set_option_on_last_part/3","doc":"Sets an option on the last text part in a list of ContentParts. Returns the updated content parts.","ref":"LangChain.Message.ContentPart.html#set_option_on_last_part/3"},{"type":"function","title":"LangChain.Message.ContentPart.text!/2","doc":"Create a new ContentPart that contains text. Raises an exception if not valid.","ref":"LangChain.Message.ContentPart.html#text!/2"},{"type":"type","title":"LangChain.Message.ContentPart.t/0","doc":"","ref":"LangChain.Message.ContentPart.html#t:t/0"},{"type":"module","title":"LangChain.Message.ToolCall","doc":"Represents an LLM's request to use tool. It specifies the tool to execute and\nmay provide arguments for the tool to use.","ref":"LangChain.Message.ToolCall.html"},{"type":"module","title":"Example - LangChain.Message.ToolCall","doc":"ToolCall.new!(%{\n      call_id: \"call_123\",\n      name: \"give_greeting\"\n      arguments: %{\"name\" => \"Howard\"}\n    })","ref":"LangChain.Message.ToolCall.html#module-example"},{"type":"function","title":"LangChain.Message.ToolCall.complete/1","doc":"Ensure the ToolCall's status is set to `:complete`. The process of completing\nit parses the tool arguments, which may be invalid. Any problems parsing are\nreturned as a changeset error.","ref":"LangChain.Message.ToolCall.html#complete/1"},{"type":"function","title":"LangChain.Message.ToolCall.merge/2","doc":"The left side, or `primary`, is the ToolCall that is being accumulated. The\n`call_part` is being merged into the `primary`.\n\nUsed to process streamed deltas where a single tool call can be split over\nmany smaller parts.","ref":"LangChain.Message.ToolCall.html#merge/2"},{"type":"function","title":"LangChain.Message.ToolCall.new/1","doc":"Build a new ToolCall and return an `:ok`/`:error` tuple with the result.","ref":"LangChain.Message.ToolCall.html#new/1"},{"type":"function","title":"LangChain.Message.ToolCall.new!/1","doc":"Build a new ToolCall and return it or raise an error if invalid.","ref":"LangChain.Message.ToolCall.html#new!/1"},{"type":"type","title":"LangChain.Message.ToolCall.t/0","doc":"","ref":"LangChain.Message.ToolCall.html#t:t/0"},{"type":"module","title":"LangChain.Message.ToolResult","doc":"Represents a the result of running a requested tool. The LLM's requests a tool\nuse through a `ToolCall`. A `ToolResult` returns the answer or result from the\napplication back to the AI.","ref":"LangChain.Message.ToolResult.html"},{"type":"module","title":"Content - LangChain.Message.ToolResult","doc":"The `content` can be a string or a list of ContentParts for multi-modal\nresponses (text, images, etc.) that gets returned to the LLM as the result.","ref":"LangChain.Message.ToolResult.html#module-content"},{"type":"module","title":"Processed Content - LangChain.Message.ToolResult","doc":"The `processed_content` field is optional. When you want to keep the results\nof the Elixir function call as a native Elixir data structure,\n`processed_content` can hold it.\n\nAdvanced use: You can use the `options` field for LLM-specific features like\ncache control.\n\nTo do this, the Elixir function's result should be a `{:ok, \"String response\nfor LLM\", native_elixir_data}`. See `LangChain.Function` for details and\nexamples.","ref":"LangChain.Message.ToolResult.html#module-processed-content"},{"type":"function","title":"LangChain.Message.ToolResult.new/1","doc":"Build a new ToolResult and return an `:ok`/`:error` tuple with the result.","ref":"LangChain.Message.ToolResult.html#new/1"},{"type":"function","title":"LangChain.Message.ToolResult.new!/1","doc":"Build a new ToolResult and return it or raise an error if invalid.","ref":"LangChain.Message.ToolResult.html#new!/1"},{"type":"type","title":"LangChain.Message.ToolResult.t/0","doc":"","ref":"LangChain.Message.ToolResult.html#t:t/0"},{"type":"module","title":"LangChain.MessageDelta","doc":"Models a \"delta\" message from a chat LLM. A delta is a small chunk, or piece\nof a much larger complete message. A series of deltas are used to construct\nthe complete message.\n\nDelta messages must be applied in order for them to be valid. Delta messages\ncan be combined and transformed into a `LangChain.Message` once the final\npiece is received.","ref":"LangChain.MessageDelta.html"},{"type":"module","title":"Roles - LangChain.MessageDelta","doc":"* `:unknown` - The role data is missing for the delta.\n* `:assistant` - Responses coming back from the LLM.","ref":"LangChain.MessageDelta.html#module-roles"},{"type":"module","title":"Tool Usage - LangChain.MessageDelta","doc":"Tools can be used or called by the assistant (LLM). A tool call is also split\nacross many message deltas and must be fully assembled before it can be\nexecuted.","ref":"LangChain.MessageDelta.html#module-tool-usage"},{"type":"module","title":"Metadata - LangChain.MessageDelta","doc":"The `metadata` field is a map that can contain any additional information\nabout the message delta. It is used to store token usage, model, and other\nLLM-specific information.","ref":"LangChain.MessageDelta.html#module-metadata"},{"type":"module","title":"Content Fields - LangChain.MessageDelta","doc":"The `content` field may contain:\n- A string (for backward compatibility)\n- A `LangChain.Message.ContentPart` struct\n- An empty list `[]` that is received from some services like Anthropic, which\n  is a signal that the content will be a list of content parts\n\nThe module uses two content-related fields:\n\n* `content` - The raw content received from the LLM. This can be either a\n  string (for backward compatibility), a `LangChain.Message.ContentPart`\n  struct, or a `[]` indicating it will be a list of content parts. This field\n  is cleared (set to `nil`) after merging into `merged_content`.\n\n* `merged_content` - The accumulated list of `ContentPart`s after merging\n  deltas. This is the source of truth for the message content and is used when\n  converting to a `LangChain.Message`. When merging deltas:\n  - For string content, it's converted to a `ContentPart` of type `:text`\n  - For `ContentPart` content, it's merged based on the `index` field\n  - Multiple content parts can be maintained in the list to support\n    multi-modal responses (text, images, audio) or separate thinking content\n    from final text","ref":"LangChain.MessageDelta.html#module-content-fields"},{"type":"function","title":"LangChain.MessageDelta.accumulate_token_usage/2","doc":"Accumulates token usage from delta messages. Uses `LangChain.TokenUsage.add/2` to combine\nthe usage data from both deltas.","ref":"LangChain.MessageDelta.html#accumulate_token_usage/2"},{"type":"function","title":"Example - LangChain.MessageDelta.accumulate_token_usage/2","doc":"iex> alias LangChain.TokenUsage\n    iex> alias LangChain.MessageDelta\n    iex> delta1 = %MessageDelta{\n    ...>   metadata: %{\n    ...>     usage: %TokenUsage{input: 10, output: 5}\n    ...>   }\n    ...> }\n    iex> delta2 = %MessageDelta{\n    ...>   metadata: %{\n    ...>     usage: %TokenUsage{input: 5, output: 15}\n    ...>   }\n    ...> }\n    iex> result = MessageDelta.accumulate_token_usage(delta1, delta2)\n    iex> result.metadata.usage.input\n    15\n    iex> result.metadata.usage.output\n    20","ref":"LangChain.MessageDelta.html#accumulate_token_usage/2-example"},{"type":"function","title":"LangChain.MessageDelta.content_to_string/2","doc":"Convert the MessageDelta's merged content to a string. Specify the type of\ncontent to convert so it can return just the text parts or thinking parts,\netc. Defaults to `:text`.","ref":"LangChain.MessageDelta.html#content_to_string/2"},{"type":"function","title":"LangChain.MessageDelta.merge_delta/2","doc":"Merge two `MessageDelta` structs. The first `MessageDelta` is the `primary`\none that smaller deltas are merged into.\n\nThe merging process:\n1. Migrates any string content to `ContentPart`s for backward compatibility\n2. Merges the content into `merged_content` based on the `index` field\n3. Clears the `content` field (sets to `nil`) after merging\n4. Updates other fields (tool_calls, status, etc.)","ref":"LangChain.MessageDelta.html#merge_delta/2"},{"type":"function","title":"Examples - LangChain.MessageDelta.merge_delta/2","doc":"iex> delta_1 =\n    ...>   %LangChain.MessageDelta{\n    ...>     content: nil,\n    ...>     index: 0,\n    ...>     tool_calls: [],\n    ...>     role: :assistant,\n    ...>     status: :incomplete\n    ...>   }\n    iex> delta_2 =\n    ...>   %LangChain.MessageDelta{\n    ...>     content: \"Hello\",\n    ...>     index: 0,\n    ...>     tool_calls: [],\n    ...>     role: :unknown,\n    ...>     status: :incomplete\n    ...>   }\n    iex> LangChain.MessageDelta.merge_delta(delta_1, delta_2)\n    %LangChain.MessageDelta{\n      content: nil,\n      merged_content: [%LangChain.Message.ContentPart{type: :text, content: \"Hello\"}],\n      status: :incomplete,\n      index: 0,\n      role: :assistant,\n      tool_calls: []\n    }\n\nA set of deltas can be easily merged like this:\n\n      MessageDelta.merge_deltas(list_of_delta_messages)","ref":"LangChain.MessageDelta.html#merge_delta/2-examples"},{"type":"function","title":"LangChain.MessageDelta.merge_deltas/1","doc":"Merges a list of `MessageDelta`s into a single `MessageDelta`. The deltas\nare merged in order, with each delta being merged into the result of the\nprevious merge.","ref":"LangChain.MessageDelta.html#merge_deltas/1"},{"type":"function","title":"Examples - LangChain.MessageDelta.merge_deltas/1","doc":"iex> deltas = [\n    ...>   %LangChain.MessageDelta{content: \"Hello\", role: :assistant},\n    ...>   %LangChain.MessageDelta{content: \" world\", role: :assistant},\n    ...>   %LangChain.MessageDelta{content: \"!\", role: :assistant, status: :complete}\n    ...> ]\n    iex> LangChain.MessageDelta.merge_deltas(deltas)\n    %LangChain.MessageDelta{\n      content: nil,\n      merged_content: [%LangChain.Message.ContentPart{type: :text, content: \"Hello world!\"}],\n      status: :complete,\n      role: :assistant\n    }","ref":"LangChain.MessageDelta.html#merge_deltas/1-examples"},{"type":"function","title":"LangChain.MessageDelta.migrate_to_content_parts/1","doc":"Migrates a MessageDelta's string content to use `LangChain.Message.ContentPart`.\nThis is for backward compatibility with models that don't yet support ContentPart streaming.","ref":"LangChain.MessageDelta.html#migrate_to_content_parts/1"},{"type":"function","title":"Examples - LangChain.MessageDelta.migrate_to_content_parts/1","doc":"iex> delta = %LangChain.MessageDelta{content: \"Hello world\"}\n    iex> upgraded = migrate_to_content_parts(delta)\n    iex> upgraded.content\n    %LangChain.Message.ContentPart{type: :text, content: \"Hello world\"}","ref":"LangChain.MessageDelta.html#migrate_to_content_parts/1-examples"},{"type":"function","title":"LangChain.MessageDelta.new/1","doc":"Create a new `MessageDelta` that represents a message chunk.","ref":"LangChain.MessageDelta.html#new/1"},{"type":"function","title":"LangChain.MessageDelta.new!/1","doc":"Create a new `MessageDelta` that represents a message chunk and return it or\nraise an error if invalid.","ref":"LangChain.MessageDelta.html#new!/1"},{"type":"function","title":"LangChain.MessageDelta.to_message/1","doc":"Convert the MessageDelta to a Message. Can only convert a fully complete\nMessageDelta.\n\nThis is assumed to be the result of merging all the received `MessageDelta`s.\nAn error is returned if the `status` is `:incomplete`.\n\nIf the `MessageDelta` fails to convert to a `LangChain.Message`, an error is\nreturned with the reason.","ref":"LangChain.MessageDelta.html#to_message/1"},{"type":"type","title":"LangChain.MessageDelta.t/0","doc":"","ref":"LangChain.MessageDelta.html#t:t/0"},{"type":"module","title":"LangChain.MessageProcessors.JsonProcessor","doc":"A built-in Message processor that processes a received Message for JSON\ncontents.\n\nWhen successful, the assistant message's JSON contents are processed into a\nmap and set on `processed_content`. No additional validation or processing of\nthe data is done by this processor.\n\nWhen JSON data is expected but not received, or the received JSON is invalid\nor incomplete, a new user `Message` struct is returned with a text error\nmessage for the LLM so it can try again to correct the error and return a\nvalid response.\n\nThere are multiple ways to extract JSON content.\n\nWhen the JSON data is reliably returned as the only response, this extracts it\nto an Elixir Map:\n\n    message = Message.new_assistant!(%{content: \"{\"value\": 123}\"})\n\n    # process the message for JSON content\n    {:cont, updated_chain, updated_message} =\n      JsonProcessor.run(chain, message)\n\nThe updated message will be an assistant message where content is a map:\n\n    updated_message.content\n    #=> %{\"value\" => 123}\n\nSome models are unable to reliably return a JSON response without adding some\ncommentary. For that situation, instruct the model how to format the JSON\ncontent. Depending on the model, one of these formats may work better than\nanother:\n\n    # bracketing the data with XML tags\n     \n    {\"value\": 123}\n     \n\n    # markdown style code fences with json language\n    ```json\n    {\"value\": 123}\n    ```\n\n    # markdown style code fences (no language)\n    ```\n    {\"value\": 123}\n    ```\n\nWhen the LLM adds commentary with the data, it may appear like this:\n\n    The answer to your question in JSON is:\n\n    ```json\n    {\"value\": 123}\n    ```\n\nWe can still extract the JSON data in a situation like this. We provide a\nRegex to use for extracting the data from whatever starting and ending text\nthe LLM was instructed to use.\n\nExamples:\n\n    ~r/ (.*?) /s\n    ~r/```json(.*?)```/s\n    ~r/```(.*?)```/s\n\nThe `\"```json\"` formatted one is processed like this:\n\n    {:cont, updated_chain, updated_message} =\n      JsonProcessor.run(chain, message, ~r/```json(.*?)```/s)","ref":"LangChain.MessageProcessors.JsonProcessor.html"},{"type":"module","title":"JsonProcessor vs Tool Usage - LangChain.MessageProcessors.JsonProcessor","doc":"The `JsonProcessor` is not compatible with tool usage. Some simpler models do\nnot support tool usage and the `JsonProcessor` can help get structured\nresponses out of them more easily.\n\nThe `JsonProcessor` is run on the assistant's response. If the response is a\nToolCall, then the response does not contain text JSON contents to process.","ref":"LangChain.MessageProcessors.JsonProcessor.html#module-jsonprocessor-vs-tool-usage"},{"type":"function","title":"LangChain.MessageProcessors.JsonProcessor.new!/0","doc":"Returns a function for use in a `LangChain.Chains.LLMChain.message_processors/2`.","ref":"LangChain.MessageProcessors.JsonProcessor.html#new!/0"},{"type":"function","title":"LangChain.MessageProcessors.JsonProcessor.new!/1","doc":"Returns a wrapped (curried) function for use in a\n`LangChain.Chains.LLMChain.message_processors/2` that includes the configured\nRegex to use for extracting JSON content.\n\nThe Regex pattern is used with the `:all_but_first` capture option to extract\njust the internal capture text.","ref":"LangChain.MessageProcessors.JsonProcessor.html#new!/1"},{"type":"function","title":"LangChain.MessageProcessors.JsonProcessor.run/2","doc":"Run the JSON Processor on a message. The response indicates what should be\ndone with the message.\n\nResponse values:\n\n- `{:cont, %Message{}}` - The returned message replaces the one being\n  processed and no additional processors are run.\n- `{:halt, %Message{}}` - Future processors are skipped. The Message is\n  returned as a response to the LLM for reporting errors.","ref":"LangChain.MessageProcessors.JsonProcessor.html#run/2"},{"type":"function","title":"LangChain.MessageProcessors.JsonProcessor.run/3","doc":"","ref":"LangChain.MessageProcessors.JsonProcessor.html#run/3"},{"type":"module","title":"LangChain.PromptTemplate","doc":"Enables defining a prompt, optionally as a template, but delaying the final\nbuilding of it until a later time when input values are substituted in.\n\nThis also supports the ability to create a Message from a PromptTemplate.\n\nAn LLM conversation is made up of a set of messages. PromptTemplates are a\ntool to help build messages.\n\n    # Create a template and convert it to a message\n    prompt = PromptTemplate.new!(%{text: \"My template\", role: :user})\n    %LangChain.Message{} = message = PromptTemplate.to_message!(prompt)\n\nPromptTemplates are powerful because they support Elixir's EEx templates\nallowing for parameter substitution. This is helpful when we want to prepare a\ntemplate message and plan to later substitute in information from the user.\n\nHere's an example of setting up a template using a parameter then later\nproviding the input value.\n\n    prompt = PromptTemplate.from_template!(\"What's a name for a company that makes <%= @product %>?\")\n\n    # later, format the final text after after applying the values.\n    PromptTemplate.format(prompt, %{product: \"colorful socks\"})\n    #=> \"What's a name for a company that makes colorful socks?\"","ref":"LangChain.PromptTemplate.html"},{"type":"function","title":"LangChain.PromptTemplate.format/2","doc":"Format the prompt template with inputs to replace with assigns. It returns the\nformatted text.\n\n    prompt = PromptTemplate.from_template!(\"Suggest a good name for a company that makes <%= @product %>?\")\n    PromptTemplate.format(prompt, %{product: \"colorful socks\"})\n    #=> \"Suggest a good name for a company that makes colorful socks?\"\n\nA PromptTemplate supports storing input values on the struct. These could be\nset when the template is defined. If an input value is not provided when the\n`format` function is called, any inputs on the struct will be used.","ref":"LangChain.PromptTemplate.html#format/2"},{"type":"function","title":"LangChain.PromptTemplate.format_composed/3","doc":"Formats a PromptTemplate at two levels. Supports providing a list of\n`composed_of` templates that are all combined into a `full_template`.\n\nFor this example, we'll use an overall template layout like this:\n\n    full_prompt =\n      PromptTemplate.from_template!(~s(<%= @introduction %>\n\n      <%= @example %>\n\n      <%= @start %>))\n\nThis template is made up of 3 more specific templates. Let's start with the\nintroduction sub-template.\n\n    introduction_prompt =\n      PromptTemplate.from_template!(\"You are impersonating <%= @person %>.\")\n\nThe `introduction` takes a parameter for which person it should impersonate.\nThe desired person is not provided here and will come in later.\n\nLet's next look at the `example` prompt:\n\n    example_prompt =\n      PromptTemplate.from_template!(~s(Here's an example of an interaction:\n        Q: <%= @example_q %>\n        A: <%= @example_a %>))\n\nThis defines a sample interaction for the LLM as a model of what we're looking\nfor. Primarily, this template is used to define the pattern we want to use for\nthe interaction. Again, this template takes parameters for the sample question\nand answer.\n\nFinally, there is the `start` section of the overall prompt. In this example,\nthat might be a question presented by a user asking a question of our\nimpersonating AI.\n\n    start_prompt =\n      PromptTemplate.from_template!(~s(Now, do this for real!\n      Q: <%= @input %>\n      A:))\n\nWe have the overall template defined and templates that define each of the\nsmaller portions. The `format_composed` function let's us combine those all\ntogether and build the complete text to pass to an LLM.\n\n      formatted_prompt =\n        PromptTemplate.format_composed(\n          full_prompt,\n          %{\n            introduction: introduction_prompt,\n            example: example_prompt,\n            start: start_prompt\n          },\n          %{\n            person: \"Elon Musk\",\n            example_q: \"What's your favorite car?\",\n            example_a: \"Tesla\",\n            input: \"What's your favorite social media site?\"\n          }\n        )\n\nWe provide the PromptTemplate for the overall prompt, then provide the inputs,\nwhich are themselves prompts.\n\nFinally, we provide a map of values for all the parameters that still need\nvalues. For this example, this is what the final prompt looks like that is\npresented to the LLM.\n\n    ~s(You are impersonating Elon Musk.\n\n    Here's an example of an interaction:\n    Q: What's your favorite car?\n    A: Tesla\n\n    Now, do this for real!\n    Q: What's your favorite social media site?\n    A:)\n\nUsing a setup like this, we can easily swap out who we are impersonating and\nallow the user to interact with that persona.\n\nWith everything defined, this is all it takes to now talk with an Abraham\nLincoln impersonation:\n\n    formatted_prompt =\n      PromptTemplate.format_composed(\n        full_prompt,\n        %{\n          introduction: introduction_prompt,\n          example: example_prompt,\n          start: start_prompt\n        },\n        %{\n          person: \"Abraham Lincoln\",\n          example_q: \"What is your nickname?\",\n          example_a: \"Honest Abe\",\n          input: \"What is one of your favorite pastimes?\"\n        }\n      )","ref":"LangChain.PromptTemplate.html#format_composed/3"},{"type":"function","title":"LangChain.PromptTemplate.format_text/2","doc":"Format the prompt template with inputs to replace embeds. The final replaced\ntext is returned.\n\nOperates directly on text to apply the inputs. This does not take the\nPromptTemplate struct.\n\n    PromptTemplate.format_text(\"Hi! My name is <%= @name %>.\", %{name: \"Jose\"})\n    #=> \"Hi! My name is Jose.\"","ref":"LangChain.PromptTemplate.html#format_text/2"},{"type":"function","title":"LangChain.PromptTemplate.from_template/1","doc":"Build a PromptTemplate struct from a template string.\n\nShortcut function for building a user prompt.\n\n    {:ok, prompt} = PromptTemplate.from_template(\"Suggest a good name for a company that makes <%= @product %>?\")","ref":"LangChain.PromptTemplate.html#from_template/1"},{"type":"function","title":"LangChain.PromptTemplate.from_template!/1","doc":"Build a PromptTemplate struct from a template string and return the struct or error if invalid.\n\nShortcut function for building a user prompt.\n\n    prompt = PromptTemplate.from_template!(\"Suggest a good name for a company that makes <%= @product %>?\")","ref":"LangChain.PromptTemplate.html#from_template!/1"},{"type":"function","title":"LangChain.PromptTemplate.new/1","doc":"","ref":"LangChain.PromptTemplate.html#new/1"},{"type":"function","title":"LangChain.PromptTemplate.new!/1","doc":"Create a new PromptTemplate struct using the attributes. If invalid, an\nexception is raised with the reason.","ref":"LangChain.PromptTemplate.html#new!/1"},{"type":"function","title":"Example - LangChain.PromptTemplate.new!/1","doc":"A template is created using a simple map with `text` and `role` keys. The\ncreated template can be be converted to a `LangChain.Message`.\n\n    PromptTemplate.new!(%{text: \"My template\", role: :user})\n\nTypically a `PromptTemplate` is used with parameter substitution as that's\nit's primary purpose. EEx is used to render the final text.\n\n    PromptTemplate.new!(%{text: \"My name is <%= @user_name %>. Warmly welcome me.\", role: :user})","ref":"LangChain.PromptTemplate.html#new!/1-example"},{"type":"function","title":"LangChain.PromptTemplate.to_content_part/2","doc":"Transform a PromptTemplate to a `LangChain.Message.ContentPart` of type\n`text`. Provide the inputs at the time of transformation to render the final\ncontent.","ref":"LangChain.PromptTemplate.html#to_content_part/2"},{"type":"function","title":"LangChain.PromptTemplate.to_content_part!/2","doc":"Transform a PromptTemplate to a `LangChain.Message.ContentPart` of type\n`text`. Provide the inputs at the time of transformation to render the final\ncontent. Raises an exception if invalid.","ref":"LangChain.PromptTemplate.html#to_content_part!/2"},{"type":"function","title":"LangChain.PromptTemplate.to_message/2","doc":"Transform a `PromptTemplate` to a `LangChain.Message`. Provide the inputs at the time of\ntransformation to render the final content.","ref":"LangChain.PromptTemplate.html#to_message/2"},{"type":"function","title":"LangChain.PromptTemplate.to_message!/2","doc":"Transform a PromptTemplate to a `LangChain.Message`. Provide the inputs at the time of\ntransformation to render the final content. Raises an exception if invalid.","ref":"LangChain.PromptTemplate.html#to_message!/2"},{"type":"function","title":"Example - LangChain.PromptTemplate.to_message!/2","doc":"It renders a `PromptTemplate`'s `text` by applying the inputs.\n\n    template = PromptTemplate.new!(%{text: \"My name is <%= @user_name %>.\", role: :user})\n    messages = PromptTemplate.to_messages!([template], %{user_name: \"Tim\"})\n\nWhere the final message has the contents `\"My name is Tim.\"`","ref":"LangChain.PromptTemplate.html#to_message!/2-example"},{"type":"function","title":"LangChain.PromptTemplate.to_messages!/2","doc":"Transform a list of PromptTemplates into a list of `LangChain.Message`s.\nApplies the inputs to the list of prompt templates. If any of the prompt\nentries are invalid or fail, an exception is raised.","ref":"LangChain.PromptTemplate.html#to_messages!/2"},{"type":"type","title":"LangChain.PromptTemplate.t/0","doc":"","ref":"LangChain.PromptTemplate.html#t:t/0"},{"type":"module","title":"LangChain.TokenUsage","doc":"Contains token usage information returned from an LLM.","ref":"LangChain.TokenUsage.html"},{"type":"module","title":"Example - LangChain.TokenUsage","doc":"%TokenUsage{\n      input: 30,\n      output: 15,\n      raw: %{\n        \"total_tokens\" => 29\n      }\n    }\n\nInput is the tokens from the prompt. Output is the completion or generated\ntokens returned.\n\nRefer to the `raw` token usage information for access to LLM-specific information that may be available.","ref":"LangChain.TokenUsage.html#module-example"},{"type":"function","title":"LangChain.TokenUsage.add/2","doc":"Combines two TokenUsage structs by adding their respective input and output values.\nThe raw maps are merged, with values being added if they are numeric.\n\nIf both arguments are nil, returns nil.\nIf one argument is nil, returns the non-nil argument.","ref":"LangChain.TokenUsage.html#add/2"},{"type":"function","title":"Example - LangChain.TokenUsage.add/2","doc":"iex> usage1 = LangChain.TokenUsage.new!(%{input: 10, output: 20, raw: %{\"total_tokens\" => 30}})\n    iex> usage2 = LangChain.TokenUsage.new!(%{input: 5, output: 15, raw: %{\"total_tokens\" => 20}})\n    iex> combined = LangChain.TokenUsage.add(usage1, usage2)\n    iex> combined.input\n    15\n    iex> combined.output\n    35\n    iex> combined.raw[\"total_tokens\"]\n    50","ref":"LangChain.TokenUsage.html#add/2-example"},{"type":"function","title":"LangChain.TokenUsage.get/1","doc":"Extracts token usage information from a `LangChain.Message` or\n`LangChain.MessageDelta` struct's metadata. Returns nil if no token usage\ninformation is found.","ref":"LangChain.TokenUsage.html#get/1"},{"type":"function","title":"Example - LangChain.TokenUsage.get/1","doc":"iex> message = %LangChain.Message{metadata: %{usage: %LangChain.TokenUsage{input: 10, output: 20}}}\n    iex> LangChain.TokenUsage.get(message)\n    %LangChain.TokenUsage{input: 10, output: 20}\n\n    iex> message = %LangChain.Message{metadata: %{}}\n    iex> LangChain.TokenUsage.get(message)\n    nil","ref":"LangChain.TokenUsage.html#get/1-example"},{"type":"function","title":"LangChain.TokenUsage.new/1","doc":"Build a new TokenUsage and return an `:ok`/`:error` tuple with the result.","ref":"LangChain.TokenUsage.html#new/1"},{"type":"function","title":"LangChain.TokenUsage.new!/1","doc":"Build a new TokenUsage and return it or raise an error if invalid.","ref":"LangChain.TokenUsage.html#new!/1"},{"type":"function","title":"LangChain.TokenUsage.set/2","doc":"Sets the token usage information on a `LangChain.Message` or\n`LangChain.MessageDelta` struct in the `metadata` under the `:usage` key.","ref":"LangChain.TokenUsage.html#set/2"},{"type":"function","title":"Example - LangChain.TokenUsage.set/2","doc":"iex> message = %LangChain.Message{metadata: %{}}\n    iex> token_usage = %LangChain.TokenUsage{input: 10, output: 20}\n    iex> LangChain.TokenUsage.set(message, token_usage)\n    %LangChain.Message{metadata: %{usage: %LangChain.TokenUsage{input: 10, output: 20}}}","ref":"LangChain.TokenUsage.html#set/2-example"},{"type":"function","title":"LangChain.TokenUsage.set_wrapped/2","doc":"Sets the token usage information on a `LangChain.Message` or\n`LangChain.MessageDelta` struct when wrapped in an :ok,:error tuple in the `metadata` under the `:usage` key.","ref":"LangChain.TokenUsage.html#set_wrapped/2"},{"type":"function","title":"LangChain.TokenUsage.total/1","doc":"Return the total token usage amount. The total is the sum of input and output.","ref":"LangChain.TokenUsage.html#total/1"},{"type":"type","title":"LangChain.TokenUsage.t/0","doc":"","ref":"LangChain.TokenUsage.html#t:t/0"},{"type":"module","title":"LangChain.Function","doc":"Defines a \"function\" that can be provided to an LLM for the LLM to optionally\nexecute and pass argument data to.\n\nA function is defined using a schema.\n\n* `name` - The name of the function given to the LLM.\n* `description` - A description of the function provided to the LLM. This\n  should describe what the function is used for or what it returns. This\n  information is used by the LLM to decide which function to call and for what\n  purpose.\n* ` parameters` - A list of `Function.FunctionParam` structs that are\n  converted to a JSONSchema format. (Use in place of `parameters_schema`)\n* ` parameters_schema` - A [JSONSchema\n  structure](https://json-schema.org/learn/getting-started-step-by-step.html)\n  that describes the required data structure format for how arguments are\n  passed to the function. (Use if greater control or unsupported features are\n  needed.)\n* `function` - An Elixir function to execute when an LLM requests to execute\n  the function. The function can return a string, a tuple, or a\n  `%ToolResult{}` struct for advanced control. Returning a ToolResult allows\n  for multi-modal responses (list of ContentParts), cache control, and\n  processed_content.\n* `async` - Boolean value that flags if this can function can be executed\n  asynchronously, potentially concurrently with other calls to the same\n  function. Defaults to `false`.\n* `options` - A Keyword list of options that can be passed to the LLM. For\n  example, this can be used for passing caching config to Anthropic.\n\nWhen passing arguments from an LLM to a function, they go through a single\n`map` argument. This allows for multiple keys or named parameters.","ref":"LangChain.Function.html"},{"type":"module","title":"Example - LangChain.Function","doc":"This example defines a function that an LLM can execute for performing basic\nmath calculations. **NOTE:** This is a partial implementation of the\n`LangChain.Tools.Calculator`.\n\n    Function.new(%{\n      name: \"calculator\",\n      description: \"Perform basic math calculations\",\n      parameters_schema: %{\n        type: \"object\",\n        properties: %{\n          expression: %{type: \"string\", description: \"A simple mathematical expression.\"}\n        },\n        required: [\"expression\"]\n      },\n      function:\n        fn(%{\"expression\" => expr} = _args, _context) ->\n          {:ok, \"42?\"}\n        end)\n    })\n\nThe `function` attribute is an Elixir function that can be executed when the\nfunction is \"called\" by the LLM.\n\nThe `args` argument is the JSON data passed by the LLM after being parsed to a\nmap.\n\nThe `context` argument is passed through as the `context` on a\n`LangChain.Chains.LLMChain`. This is whatever context data is needed for the\nfunction to do it's work.\n\nContext examples could be data like user_id, account_id, account struct,\nbilling level, etc.","ref":"LangChain.Function.html#module-example"},{"type":"module","title":"Function Parameters - LangChain.Function","doc":"The `parameters` field is a list of `LangChain.FunctionParam` structs. This is\na convenience for defining the parameters to the function. If it does not work\nfor more complex use-cases, then use the `parameters_schema` to declare it as\nneeded.\n\nThe `parameters_schema` is an Elixir map that follows a\n[JSONSchema](https://json-schema.org/learn/getting-started-step-by-step.html)\nstructure. It is used to define the required data structure format for\nreceiving data to the function from the LLM.\n\nNOTE: Only use `parameters` or `parameters_schema`, not both.","ref":"LangChain.Function.html#module-function-parameters"},{"type":"module","title":"Expanded Parameter Examples - LangChain.Function","doc":"Function with no arguments:\n\n    alias LangChain.Function\n\n    Function.new!(%{name: \"get_current_user_info\"})\n\nFunction that takes a simple required argument:\n\n    alias LangChain.FunctionParam\n\n    Function.new!(%{name: \"set_user_name\", parameters: [\n      FunctionParam.new!(%{name: \"user_name\", type: :string, required: true})\n    ]})\n\nFunction that takes an array of strings:\n\n    Function.new!(%{name: \"set_tags\", parameters: [\n      FunctionParam.new!(%{name: \"tags\", type: :array, item_type: \"string\"})\n    ]})\n\nFunction that takes two arguments and one is an object/map:\n\n    Function.new!(%{name: \"update_preferences\", parameters: [\n      FunctionParam.new!(%{name: \"unique_code\", type: :string, required: true})\n      FunctionParam.new!(%{name: \"data\", type: :object, object_properties: [\n        FunctionParam.new!(%{name: \"auto_complete_email\", type: :boolean}),\n        FunctionParam.new!(%{name: \"items_per_page\", type: :integer}),\n      ]})\n    ]})\n\nThe `LangChain.FunctionParam` is nestable allowing for arrays of object and\nobjects with nested objects.","ref":"LangChain.Function.html#module-expanded-parameter-examples"},{"type":"module","title":"Example that also stores the Elixir result - LangChain.Function","doc":"Sometimes we want to process a `ToolCall` from the LLM and keep the processed\nElixir data for ourselves. This is particularly useful when using an LLM to\nperform structured data extraction. Our Elixir function may even process that\ndata into a newly created Ecto Schema database entry. The result of the\n`ToolCall` that goes back to the LLM must be in a string form. That typically\nmeans returning a JSON string of the result data.\n\nTo make it easier to process the data, return a string response to the LLM,\nbut **keep** the original Elixir data as well, our Elixir function can return\na 3-tuple result.\n\n    Function.new!(%{name: \"create_invoice\",\n      parameters: [\n        FunctionParam.new!(%{name: \"vendor_name\", type: :string, required: true})\n        FunctionParam.new!(%{name: \"total_amount\", type: :string, required: true})\n      ],\n      function: &execute_create_invoice/2\n    })\n\n    # ...\n\n    def execute_create_invoice(args, %{account_id: account_id} = _context) do\n      case MyApp.Invoices.create_invoice(account_id, args) do\n        {:ok, invoice} ->\n          {:ok, \"SUCCESS\", invoice}\n\n        {:error, changeset} ->\n          {:error, \"ERROR: \" <> LangChain.Utils.changeset_error_to_string(changeset)}\n      end\n    end\n\nIn this example, the `LangChain.Function` is tied to the\n`MyApp.Invoices.create_invoice/2` function in our application.\n\nThe Elixir function returns a 3-tuple result. The `\"SUCCESS\"` is returned to\nthe LLM. In our scenario, we don't care to return a JSON version of the\ninvoice. The important part is we return the actual\n`%MyApp.Invoices.Invoice{}` struct in the tuple. This is stored on the\n`LangChain.ToolResult`'s `processed_content` field.\n\nThis is really helpful when all we want is the final, fully processed Elixir\nresult. This pairs well with the `LLMChain.run(chain, mode: :until_success)`.\nThis is when we want the LLM to perform some data extraction and it should be\nre-run until it succeeds and we have our final, processed result in the\n`ToolResult`.\n\nNote: The LLM may issue one or more `ToolCall`s in a single assistant message.\nEach Elixir function's `ToolResult` may contain a `processed_content`.","ref":"LangChain.Function.html#module-example-that-also-stores-the-elixir-result"},{"type":"module","title":"Explicit ToolResult Control - LangChain.Function","doc":"For advanced use cases where you need explicit control over the `ToolResult`\nstructure or want to set LLM-specific options, your Elixir function can return\na fully constructed `%ToolResult{}` struct. The `content` field can be a list\nof ContentParts for multi-modal responses.\n\nThis approach is particularly useful when you need to:\n\n* Set LLM-specific options (like Anthropic's `cache_control`)\n* Set custom error states beyond simple string responses\n* Customize the `display_text` for the ToolResult\n* Provide detailed metadata in the `options` field\n\nThe `options` field can contain any LLM-specific configuration that gets\npassed through to the chat model's API conversion layer. If an LLM does not\nsupport it, it will be ignored.","ref":"LangChain.Function.html#module-explicit-toolresult-control"},{"type":"function","title":"LangChain.Function.execute/3","doc":"Execute the function passing in arguments and additional optional context.\nThis is called by a `LangChain.Chains.LLMChain` when a `Function` execution is\nrequested by the LLM.","ref":"LangChain.Function.html#execute/3"},{"type":"function","title":"LangChain.Function.get_display_text/3","doc":"Given a list of functions, return the `display_text` for the named function.\nIf it not found, return the fallback text.","ref":"LangChain.Function.html#get_display_text/3"},{"type":"function","title":"LangChain.Function.new/1","doc":"Build a new function.","ref":"LangChain.Function.html#new/1"},{"type":"function","title":"LangChain.Function.new!/1","doc":"Build a new function and return it or raise an error if invalid.","ref":"LangChain.Function.html#new!/1"},{"type":"type","title":"LangChain.Function.arguments/0","doc":"","ref":"LangChain.Function.html#t:arguments/0"},{"type":"type","title":"LangChain.Function.context/0","doc":"","ref":"LangChain.Function.html#t:context/0"},{"type":"type","title":"LangChain.Function.t/0","doc":"","ref":"LangChain.Function.html#t:t/0"},{"type":"module","title":"LangChain.FunctionParam","doc":"Define a function parameter as a struct. Used to generate the expected\nJSONSchema data for describing one or more arguments being passed to a\n`LangChain.Function`.\n\nNote: This is not intended to be a fully compliant implementation of\n[JSONSchema\ntypes](https://json-schema.org/understanding-json-schema/reference/type). This\nis intended to be a convenience for working with the most common situations\nwhen working with an LLM that understands JSONSchema.\n\nSupports:\n\n* simple values - string, integer, number, boolean\n* enum values - `enum: [\"alpha\", \"beta\"]`. The values can be strings,\n  integers, etc.\n* array values - `type: :array` couples with `item_type: \"string\"` to express\n  it is an array of.\n  * `item_type` is optional. When omitted, it can be a mixed array.\n  * `item_type: \"object\"` allows for creating an array of objects. Use\n    `object_properties: [...]` to describe the structure of the objects.\n* objects - Define the object's expected values or supported structure using\n  `object_properties`.\n\nThe function `to_parameters_schema/1` is used to convert a list of\n`FunctionParam` structs into a JSONSchema formatted data map.","ref":"LangChain.FunctionParam.html"},{"type":"module","title":"Examples - LangChain.FunctionParam","doc":"Basic string field.\n\n    FunctionParam.new!(%{name: \"name\", type: :string})\n\nBasic string field with description\n\n    FunctionParam.new!(%{name: \"name\",\n                         type: :string,\n                         description: \"User's name\"})\n\nBasic required string field with description\n\n    FunctionParam.new!(%{name: \"name\", type: :string, required: true})\n\nRequired boolean field with description\n\n    FunctionParam.new!(%{name: \"active\", type: :boolean, required: true})\n\nA string Enum field. The field's value can only be one of the values specified\nby the enum.\n\n    FunctionParam.new!(%{name: \"color\",\n                         type: :string,\n                         enum: [\"red\", \"yellow\", \"blue\"],\n                         description: \"The specified primary color})\n\nArray of strings field example. Defines a set of tags that an LLM may assign.\n\n    FunctionParam.new!(%{name: \"tags\", type: :array, item_type: \"string\")\n\nA field that represents an object with nested fields.\n\n    FunctionParam.new!(%{name: \"person\",\n                         type: :object,\n                         object_properties: [\n                           FunctionParam.new!(%{name: \"name\", type: :string, required: true}),\n                           FunctionParam.new!(%{name: \"age\", type: :integer}),\n                         ]\n                      })","ref":"LangChain.FunctionParam.html#module-examples"},{"type":"function","title":"LangChain.FunctionParam.new/1","doc":"Build a new FunctionParam struct.","ref":"LangChain.FunctionParam.html#new/1"},{"type":"function","title":"LangChain.FunctionParam.new!/1","doc":"Build a new `FunctionParam` struct and return it or raise an error if invalid.","ref":"LangChain.FunctionParam.html#new!/1"},{"type":"function","title":"LangChain.FunctionParam.required_properties/1","doc":"Return the list of required property names.","ref":"LangChain.FunctionParam.html#required_properties/1"},{"type":"function","title":"LangChain.FunctionParam.to_json_schema/2","doc":"Transform a `FunctionParam` to a JSONSchema compatible definition that is\nadded to the passed in `data` map.","ref":"LangChain.FunctionParam.html#to_json_schema/2"},{"type":"function","title":"LangChain.FunctionParam.to_parameters_schema/1","doc":"Transform a list of `FunctionParam` structs into a map expressing the structure\nin a JSONSchema compatible way.","ref":"LangChain.FunctionParam.html#to_parameters_schema/1"},{"type":"type","title":"LangChain.FunctionParam.t/0","doc":"","ref":"LangChain.FunctionParam.html#t:t/0"},{"type":"module","title":"LangChain.Callbacks","doc":"Defines the structure of callbacks and provides utilities for executing them.\n\nSee `LangChain.Chains.ChainCallbacks` for the list of callbacks that can be\nused.","ref":"LangChain.Callbacks.html"},{"type":"function","title":"LangChain.Callbacks.fire/3","doc":"Fire a named callback with the list of arguments to pass. Takes a list of\ncallback handlers and will execute the callback for each handler that defines\na handler function for it.","ref":"LangChain.Callbacks.html#fire/3"},{"type":"module","title":"LangChain.Chains.ChainCallbacks","doc":"Defines the callbacks fired by an LLMChain and LLM module.\n\nA callback handler is a map that defines the specific callback event with a\nfunction to execute for that event.","ref":"LangChain.Chains.ChainCallbacks.html"},{"type":"module","title":"Example - LangChain.Chains.ChainCallbacks","doc":"A sample configured callback handler that forwards received data to a specific\nLiveView.\n\n    live_view_pid = self()\n\n    my_handlers = %{\n      on_llm_new_message: fn _chain, new_message -> send(live_view_pid, {:received_message, new_message}) end,\n      on_llm_new_delta: fn _chain, new_deltas -> send(live_view_pid, {:received_delta, new_deltas}) end,\n      on_error_message_created: fn _chain, new_message -> send(live_view_pid, {:received_message, new_message}) end\n    }\n\n    model = SomeLLM.new!(%{...})\n\n    chain =\n      %{llm: model}\n      |> LLMChain.new!()\n      |> LLMChain.add_callback(my_handlers)","ref":"LangChain.Chains.ChainCallbacks.html#module-example"},{"type":"type","title":"LangChain.Chains.ChainCallbacks.chain_callback_handler/0","doc":"The supported set of callbacks for an LLM module.","ref":"LangChain.Chains.ChainCallbacks.html#t:chain_callback_handler/0"},{"type":"type","title":"LangChain.Chains.ChainCallbacks.chain_error_message_created/0","doc":"Executed when an LLMChain, in response to an error from the LLM, generates a\nnew, automated response message intended to be returned to the LLM.\n\nThe handler's return value is discarded.","ref":"LangChain.Chains.ChainCallbacks.html#t:chain_error_message_created/0"},{"type":"type","title":"Example - LangChain.Chains.ChainCallbacks.chain_error_message_created/0","doc":"A function declaration that matches the signature.\n\n    def handles_chain_error_message_created(chain, new_message) do\n      IO.inspect(new_message)\n    end","ref":"LangChain.Chains.ChainCallbacks.html#t:chain_error_message_created/0-example"},{"type":"type","title":"LangChain.Chains.ChainCallbacks.chain_message_processed/0","doc":"Executed when an LLMChain has completed processing a received assistant\nmessage.\n\nThe handler's return value is discarded.","ref":"LangChain.Chains.ChainCallbacks.html#t:chain_message_processed/0"},{"type":"type","title":"Example - LangChain.Chains.ChainCallbacks.chain_message_processed/0","doc":"A function declaration that matches the signature.\n\n    def handle_chain_message_processed(chain, message) do\n      IO.inspect(message)\n    end","ref":"LangChain.Chains.ChainCallbacks.html#t:chain_message_processed/0-example"},{"type":"type","title":"LangChain.Chains.ChainCallbacks.chain_message_processing_error/0","doc":"Executed when processing a received message errors or fails. The erroring\nmessage is included in the callback with the state of processing that was\ncompleted before erroring.\n\nThe handler's return value is discarded.","ref":"LangChain.Chains.ChainCallbacks.html#t:chain_message_processing_error/0"},{"type":"type","title":"Example - LangChain.Chains.ChainCallbacks.chain_message_processing_error/0","doc":"A function declaration that matches the signature.\n\n    def handle_chain_message_processing_error(chain, new_message) do\n      IO.inspect(new_message)\n    end","ref":"LangChain.Chains.ChainCallbacks.html#t:chain_message_processing_error/0-example"},{"type":"type","title":"LangChain.Chains.ChainCallbacks.chain_retries_exceeded/0","doc":"Executed when the chain failed multiple times used up the `max_retry_count`\nresulting in the process aborting and returning an error.\n\nThe handler's return value is discarded.","ref":"LangChain.Chains.ChainCallbacks.html#t:chain_retries_exceeded/0"},{"type":"type","title":"Example - LangChain.Chains.ChainCallbacks.chain_retries_exceeded/0","doc":"A function declaration that matches the signature.\n\n    def handle_retries_exceeded(chain) do\n      IO.inspect(chain)\n    end","ref":"LangChain.Chains.ChainCallbacks.html#t:chain_retries_exceeded/0-example"},{"type":"type","title":"LangChain.Chains.ChainCallbacks.chain_tool_response_created/0","doc":"Executed when the chain uses one or more tools and the resulting ToolResults\nare generated as part of a tool response message.\n\nThe handler's return value is discarded.","ref":"LangChain.Chains.ChainCallbacks.html#t:chain_tool_response_created/0"},{"type":"type","title":"Example - LangChain.Chains.ChainCallbacks.chain_tool_response_created/0","doc":"A function declaration that matches the signature.\n\n    def handle_chain_tool_response_created(chain, new_message) do\n      IO.inspect(new_message)\n    end","ref":"LangChain.Chains.ChainCallbacks.html#t:chain_tool_response_created/0-example"},{"type":"type","title":"LangChain.Chains.ChainCallbacks.llm_new_delta/0","doc":"Executed when an LLM is streaming a response and a new MessageDelta (or token)\nwas received.\n\n- `:index` is optionally present if the LLM supports sending `n` versions of a\n  response.\n\nThe return value is discarded.","ref":"LangChain.Chains.ChainCallbacks.html#t:llm_new_delta/0"},{"type":"type","title":"Example - LangChain.Chains.ChainCallbacks.llm_new_delta/0","doc":"A function declaration that matches the signature.\n\n    def handle_llm_new_delta(chain, delta) do\n      IO.write(delta)\n    end","ref":"LangChain.Chains.ChainCallbacks.html#t:llm_new_delta/0-example"},{"type":"type","title":"LangChain.Chains.ChainCallbacks.llm_new_message/0","doc":"Executed when an LLM is not streaming and a full message was received.\n\nThe return value is discarded.","ref":"LangChain.Chains.ChainCallbacks.html#t:llm_new_message/0"},{"type":"type","title":"Example - LangChain.Chains.ChainCallbacks.llm_new_message/0","doc":"A function declaration that matches the signature.\n\n    def handle_llm_new_message(chain, message) do\n      IO.inspect(message)\n    end","ref":"LangChain.Chains.ChainCallbacks.html#t:llm_new_message/0-example"},{"type":"type","title":"LangChain.Chains.ChainCallbacks.llm_ratelimit_info/0","doc":"Executed when an LLM (typically a service) responds with rate limiting\ninformation.\n\nThe specific rate limit information depends on the LLM. It returns a map with\nall the available information included.\n\nThe return value is discarded.","ref":"LangChain.Chains.ChainCallbacks.html#t:llm_ratelimit_info/0"},{"type":"type","title":"Example - LangChain.Chains.ChainCallbacks.llm_ratelimit_info/0","doc":"A function declaration that matches the signature.\n\n    def handle_llm_ratelimit_info(chain, %{} = info) do\n      IO.inspect(info)\n    end","ref":"LangChain.Chains.ChainCallbacks.html#t:llm_ratelimit_info/0-example"},{"type":"type","title":"LangChain.Chains.ChainCallbacks.llm_token_usage/0","doc":"Executed when an LLM response reports the token usage in a\n`LangChain.TokenUsage` struct. The data returned depends on the LLM.\n\nThe return value is discarded.","ref":"LangChain.Chains.ChainCallbacks.html#t:llm_token_usage/0"},{"type":"type","title":"Example - LangChain.Chains.ChainCallbacks.llm_token_usage/0","doc":"A function declaration that matches the signature.\n\n    def handle_llm_token_usage(chain, %TokenUsage{} = usage) do\n      IO.inspect(usage)\n    end","ref":"LangChain.Chains.ChainCallbacks.html#t:llm_token_usage/0-example"},{"type":"module","title":"LangChain.Chains.RoutingChain","doc":"Run a router based on a user's initial prompt to determine what category best\nmatches from the given options. If there is no good match, the value \"DEFAULT\"\nis returned.\n\nHere's an example:\n\n    routes = [\n      PromptRoute.new!(%{\n        name: \"marketing_email\",\n        description: \"Create a marketing focused email\",\n        chain: marketing_email_chain\n      }),\n      PromptRoute.new!(%{\n        name: \"blog_post\",\n        description: \"Create a blog post that will be linked from the company's landing page\",\n        chain: blog_post_chain\n      }),\n    ]\n\n    selected_route =\n      RoutingChain.new!(%{\n        llm: ChatOpenAI.new!(%{model: \"gpt-40-mini\", stream: false}),\n        input_text: \"Let's create a marketing blog post about our new product 'Fuzzy Furies'\",\n        routes: routes,\n        default_route: PromptRoute.new!(%{name: \"DEFAULT\", chain: fallback_chain})\n      })\n      |> RoutingChain.evaluate()\n\n    # The PromptRoute for the `blog_post` should be returned as the `selected_route`.\n\nThe `llm` is the model used to make the determination of which route is the\nbest match. A smaller, faster LLM may be a great choice for the routing\ndecision, then a more complex LLM may be used for a selected route.\n\nThe `default_route` is required and is used as a fallback if the user's prompt\ndoesn't match any of the specified routes. It may also be used in some\nfallback error situations as well.","ref":"LangChain.Chains.RoutingChain.html"},{"type":"function","title":"LangChain.Chains.RoutingChain.evaluate/2","doc":"Runs the RoutingChain and evaluates the result to return the selected chain.","ref":"LangChain.Chains.RoutingChain.html#evaluate/2"},{"type":"function","title":"LangChain.Chains.RoutingChain.new/1","doc":"Start a new RoutingChain.","ref":"LangChain.Chains.RoutingChain.html#new/1"},{"type":"function","title":"LangChain.Chains.RoutingChain.new!/1","doc":"Start a new RoutingChain and return it or raise an error if invalid.","ref":"LangChain.Chains.RoutingChain.html#new!/1"},{"type":"function","title":"LangChain.Chains.RoutingChain.run/2","doc":"Run a simple RoutingChain to analyze the input_text and determine which of the\ngiven routes is the best match.\n\nA simpler, faster LLM may be a great fit for running the analysis. If it fails\nto find a good match, the `default_route` is used. The `default_route`'s name\nis supplied to the LLM as well. The name \"DEFAULT\" is suggested for this\nroute.","ref":"LangChain.Chains.RoutingChain.html#run/2"},{"type":"type","title":"LangChain.Chains.RoutingChain.t/0","doc":"","ref":"LangChain.Chains.RoutingChain.html#t:t/0"},{"type":"module","title":"LangChain.Routing.PromptRoute","doc":"Defines a route or direction a prompting interaction with an LLM can take.\n\nThis helps add complexity and specificity to an LLM's instructions without\nusing many tokens and helps avoid combining many things into a single prompt\nsetup. This works by letting the user's initial prompt define how the next\nprompt is setup. A first pass is run on the prompt to determine which of the\nprovided PromptRoutes is the best match.\n\nTo better understand, let's see example of taking the user's input and\nclassifying what supported activity or specialty it is best matched with.\n\nUser prompt:\n> Let's create a marketing focused blog post comparing three types of our\n> trailer hitch products.\n\nWe want our assistant to use different prompts for the different types of\nactivities it's capable of doing. Let's define the activities like this:\n\n    [\n      PromptRoute.new!(%{\n        name: \"marketing_email\",\n        description: \"Create a marketing focused email\",\n        chain: marketing_email_chain\n      }),\n      PromptRoute.new!(%{\n        name: \"blog_post\",\n        description: \"Create a blog post that will be linked from the company's landing page\",\n        chain: blog_post_chain\n      }),\n      PromptRoute.new!(%{\n        name: \"support_triage\",\n        description: \"Triage a customer support request for severity and requested resolution\",\n        chain: support_triage_chain\n      })\n    ]\n\nGiven the user's prompt and the routes we provided, it will choose\n\"blog_post\". This leads to an LLMChain being setup for that purpose where the\nuser's initial prompt can be re-run against the prompts we define for that\nroute.\n\nWhen a `chain` is linked to each route, we can specify the model to use, the\nprompt templates we want, and associate functions that can be used to\naccomplish the task.\n\nThe selected LLMChain can then be run through a UI for chat interactions where\nthe user and AI work together.\n\nThe `name` will be returned by the LLM when a particular route is selected.\n\nThe `description` is given to the LLM to help it determine if the route should\nbe selected based on the current input prompt.","ref":"LangChain.Routing.PromptRoute.html"},{"type":"function","title":"LangChain.Routing.PromptRoute.get_selected/2","doc":"Return the selected route based on the route name.","ref":"LangChain.Routing.PromptRoute.html#get_selected/2"},{"type":"function","title":"LangChain.Routing.PromptRoute.new/1","doc":"Build a new PromptRoute struct.","ref":"LangChain.Routing.PromptRoute.html#new/1"},{"type":"function","title":"LangChain.Routing.PromptRoute.new!/1","doc":"Build a new `PromptRoute` struct and return it or raise an error if invalid.","ref":"LangChain.Routing.PromptRoute.html#new!/1"},{"type":"type","title":"LangChain.Routing.PromptRoute.t/0","doc":"","ref":"LangChain.Routing.PromptRoute.html#t:t/0"},{"type":"module","title":"LangChain.Images","doc":"Functions for working with `LangChain.GeneratedImage` files.","ref":"LangChain.Images.html"},{"type":"function","title":"LangChain.Images.save_images/3","doc":"Save a list of `%GeneratedImage{}` images.\n\nPipe friendly for handling the result a `LangChain.Images.OpenAIImage.call/1`\nwhere it handles a success or passes the error result through.","ref":"LangChain.Images.html#save_images/3"},{"type":"function","title":"LangChain.Images.save_to_file/2","doc":"Save the generated image file to a local directory. If the GeneratedFile is an\nURL, it is first downloaded then saved. If the is a Base64 encoded image, it\nis decoded and saved.","ref":"LangChain.Images.html#save_to_file/2"},{"type":"module","title":"LangChain.Images.GeneratedImage","doc":"Represents a generated image where we have either the base64 encoded contents\nor a temporary URL to it.","ref":"LangChain.Images.GeneratedImage.html"},{"type":"module","title":"Example - LangChain.Images.GeneratedImage","doc":"Created when an image generation request completes and we have an image.\n\n    GeneratedImage.new!(%{\n      image_type: :png,\n      type: :url,\n      content: \"https://example.com/my_image.png\",\n      prompt: \"The prompt used for image generation\"\n    })","ref":"LangChain.Images.GeneratedImage.html#module-example"},{"type":"function","title":"LangChain.Images.GeneratedImage.new/1","doc":"Build a new GeneratedImage and return an `:ok`/`:error` tuple with the result.","ref":"LangChain.Images.GeneratedImage.html#new/1"},{"type":"function","title":"LangChain.Images.GeneratedImage.new!/1","doc":"Build a new GeneratedImage and return it or raise an error if invalid.","ref":"LangChain.Images.GeneratedImage.html#new!/1"},{"type":"type","title":"LangChain.Images.GeneratedImage.t/0","doc":"","ref":"LangChain.Images.GeneratedImage.html#t:t/0"},{"type":"module","title":"LangChain.Images.OpenAIImage","doc":"Represents the [OpenAI Images API\nendpoint](https://platform.openai.com/docs/api-reference/images) for working\nwith DALL-E-2 and DALL-E-3.\n\nParses and validates inputs for making a request from the OpenAI Image API.\n\nConverts responses into more specialized `LangChain` data structures and\nprovide functions for saving generated images.","ref":"LangChain.Images.OpenAIImage.html"},{"type":"function","title":"LangChain.Images.OpenAIImage.call/1","doc":"Calls the OpenAI API passing the OpenAIImage struct with configuration.\n\nWhen successful, it returns `{:ok, generated_images}` where that is a list of\n`LangChain.Images.GeneratedImage` structs.","ref":"LangChain.Images.OpenAIImage.html#call/1"},{"type":"function","title":"LangChain.Images.OpenAIImage.for_api/1","doc":"Return the params formatted for an API request.","ref":"LangChain.Images.OpenAIImage.html#for_api/1"},{"type":"function","title":"LangChain.Images.OpenAIImage.new/1","doc":"Setup a OpenAIImage client configuration.","ref":"LangChain.Images.OpenAIImage.html#new/1"},{"type":"function","title":"LangChain.Images.OpenAIImage.new!/1","doc":"Setup a OpenAIImage client configuration and return it or raise an error if\ninvalid.","ref":"LangChain.Images.OpenAIImage.html#new!/1"},{"type":"type","title":"LangChain.Images.OpenAIImage.t/0","doc":"","ref":"LangChain.Images.OpenAIImage.html#t:t/0"},{"type":"module","title":"LangChain.TextSplitter.CharacterTextSplitter","doc":"The `CharacterTextSplitter` is a length based text splitter\nthat divides text based on specified characters.\nThis splitter provides consistent chunk sizes.\nIt operates as follows:\n\n- It splits the text at specified `separator` characters.\n- It takes a `chunk_size` parameter that determines the maximum number of tokens\n  in each chunk.\n- If no separator is found within the `chunk_size`,\n  it will create a chunk larger than the specified size.\n\nThe purpose is to prepare text for processing\nby large language models with limited context windows,\nor where a shorter context window is desired.\n\nA `CharacterTextSplitter` is defined using a schema.\n* `separator` - String that splits a given text.\n* `chunk_size` - Integer number of tokens that a chunk should have.\n* `chunk_overlap` - Integer number of tokens that two consecutive chunks should share.\n* `keep_separator` - Either `:discard_separator`, `:start` or `:end`. If `:discard_separator`, the separator is discarded from the output chunks. `:start` and `:end` keep the separator at the start or end of the output chunks. Defaults to `:discard_separator`.\n* `is_separator_regex` - Boolean defaulting to `false`. If `true`, the `separator` string is not escaped. Defaults to `false`\n* `tokenizer` - Function that takes a string and returns the number of tokens. Defaults to `&String.length/1`.","ref":"LangChain.TextSplitter.CharacterTextSplitter.html"},{"type":"function","title":"LangChain.TextSplitter.CharacterTextSplitter.new/1","doc":"Build a new CharacterTextSplitter and return an `:ok`/`:error` tuple with the result.","ref":"LangChain.TextSplitter.CharacterTextSplitter.html#new/1"},{"type":"function","title":"LangChain.TextSplitter.CharacterTextSplitter.new!/1","doc":"Build a new CharacterTextSplitter and return it or raise an error if invalid.","ref":"LangChain.TextSplitter.CharacterTextSplitter.html#new!/1"},{"type":"function","title":"LangChain.TextSplitter.CharacterTextSplitter.split_text/2","doc":"Splits text based on a given character.\nBy default, the `separator` character is discarded\n    iex> text_splitter = CharacterTextSplitter.new!(%{separator: \" \", chunk_size: 3, chunk_overlap: 0})\n    iex> text = \"foo bar baz\"\n    iex> CharacterTextSplitter.split_text(text_splitter, text)\n    [\"foo\", \"bar\", \"baz\"]\n\nWe can keep the separator at the end of a chunk, providing the\n`keep_separator: :end` option:\n    iex> text_splitter = CharacterTextSplitter.new!(%{separator: \".\", chunk_size: 3, chunk_overlap: 0, keep_separator: :end})\n    iex> text = \"foo.bar.baz\"\n    iex> CharacterTextSplitter.split_text(text_splitter, text)\n    [\"foo.\", \"bar.\", \"baz\"]\n\nIn order to keep the separator at the beginning of a chunk, provide the\n`keep_separator: :start` option:\n    iex> text_splitter = CharacterTextSplitter.new!(%{separator: \".\", chunk_size: 3, chunk_overlap: 0, keep_separator: :start})\n    iex> text = \"foo.bar.baz\"\n    iex> CharacterTextSplitter.split_text(text_splitter, text)\n    [\"foo\", \".bar\", \".baz\"]\n\nThe last two examples used a regex special character as a `separator`.\nPlain strings are escaped and parsed as regex before splitting.\nIf you want to use a complex regex as `separator` you can,\nbut make sure to pass the `is_separator_regex: true` option:\n    iex> text_splitter = CharacterTextSplitter.new!(%{separator: Regex.escape(\".\"), chunk_size: 3, chunk_overlap: 0, keep_separator: :start, is_separator_regex: true})\n    iex> text = \"foo.bar.baz\"\n    iex> CharacterTextSplitter.split_text(text_splitter, text)\n    [\"foo\", \".bar\", \".baz\"]\n\nYou can control the overlap of chunks trhough the `chunk_overlap` parameter:\n    iex> text_splitter = CharacterTextSplitter.new!(%{separator: \" \", chunk_size: 7, chunk_overlap: 3})\n    iex> text = \"foo bar baz\"\n    iex> CharacterTextSplitter.split_text(text_splitter, text)\n    [\"foo bar\", \"bar baz\"]","ref":"LangChain.TextSplitter.CharacterTextSplitter.html#split_text/2"},{"type":"type","title":"LangChain.TextSplitter.CharacterTextSplitter.t/0","doc":"","ref":"LangChain.TextSplitter.CharacterTextSplitter.html#t:t/0"},{"type":"module","title":"LangChain.TextSplitter.LanguageSeparators","doc":"Separators lists for programming and markdown languages.\nUseful to use with `LangChain.TextSplitter.RecursiveCharacterTextSplitter`.","ref":"LangChain.TextSplitter.LanguageSeparators.html"},{"type":"function","title":"LangChain.TextSplitter.LanguageSeparators.c/0","doc":"","ref":"LangChain.TextSplitter.LanguageSeparators.html#c/0"},{"type":"function","title":"LangChain.TextSplitter.LanguageSeparators.cobol/0","doc":"","ref":"LangChain.TextSplitter.LanguageSeparators.html#cobol/0"},{"type":"function","title":"LangChain.TextSplitter.LanguageSeparators.cpp/0","doc":"","ref":"LangChain.TextSplitter.LanguageSeparators.html#cpp/0"},{"type":"function","title":"LangChain.TextSplitter.LanguageSeparators.csharp/0","doc":"","ref":"LangChain.TextSplitter.LanguageSeparators.html#csharp/0"},{"type":"function","title":"LangChain.TextSplitter.LanguageSeparators.elixir/0","doc":"","ref":"LangChain.TextSplitter.LanguageSeparators.html#elixir/0"},{"type":"function","title":"LangChain.TextSplitter.LanguageSeparators.go/0","doc":"","ref":"LangChain.TextSplitter.LanguageSeparators.html#go/0"},{"type":"function","title":"LangChain.TextSplitter.LanguageSeparators.haskell/0","doc":"","ref":"LangChain.TextSplitter.LanguageSeparators.html#haskell/0"},{"type":"function","title":"LangChain.TextSplitter.LanguageSeparators.html/0","doc":"","ref":"LangChain.TextSplitter.LanguageSeparators.html#html/0"},{"type":"function","title":"LangChain.TextSplitter.LanguageSeparators.java/0","doc":"","ref":"LangChain.TextSplitter.LanguageSeparators.html#java/0"},{"type":"function","title":"LangChain.TextSplitter.LanguageSeparators.js/0","doc":"","ref":"LangChain.TextSplitter.LanguageSeparators.html#js/0"},{"type":"function","title":"LangChain.TextSplitter.LanguageSeparators.kotlin/0","doc":"","ref":"LangChain.TextSplitter.LanguageSeparators.html#kotlin/0"},{"type":"function","title":"LangChain.TextSplitter.LanguageSeparators.latex/0","doc":"","ref":"LangChain.TextSplitter.LanguageSeparators.html#latex/0"},{"type":"function","title":"LangChain.TextSplitter.LanguageSeparators.lua/0","doc":"","ref":"LangChain.TextSplitter.LanguageSeparators.html#lua/0"},{"type":"function","title":"LangChain.TextSplitter.LanguageSeparators.markdown/0","doc":"","ref":"LangChain.TextSplitter.LanguageSeparators.html#markdown/0"},{"type":"function","title":"LangChain.TextSplitter.LanguageSeparators.php/0","doc":"","ref":"LangChain.TextSplitter.LanguageSeparators.html#php/0"},{"type":"function","title":"LangChain.TextSplitter.LanguageSeparators.powershell/0","doc":"","ref":"LangChain.TextSplitter.LanguageSeparators.html#powershell/0"},{"type":"function","title":"LangChain.TextSplitter.LanguageSeparators.proto/0","doc":"","ref":"LangChain.TextSplitter.LanguageSeparators.html#proto/0"},{"type":"function","title":"LangChain.TextSplitter.LanguageSeparators.python/0","doc":"","ref":"LangChain.TextSplitter.LanguageSeparators.html#python/0"},{"type":"function","title":"LangChain.TextSplitter.LanguageSeparators.rst/0","doc":"","ref":"LangChain.TextSplitter.LanguageSeparators.html#rst/0"},{"type":"function","title":"LangChain.TextSplitter.LanguageSeparators.ruby/0","doc":"","ref":"LangChain.TextSplitter.LanguageSeparators.html#ruby/0"},{"type":"function","title":"LangChain.TextSplitter.LanguageSeparators.rust/0","doc":"","ref":"LangChain.TextSplitter.LanguageSeparators.html#rust/0"},{"type":"function","title":"LangChain.TextSplitter.LanguageSeparators.scala/0","doc":"","ref":"LangChain.TextSplitter.LanguageSeparators.html#scala/0"},{"type":"function","title":"LangChain.TextSplitter.LanguageSeparators.sol/0","doc":"","ref":"LangChain.TextSplitter.LanguageSeparators.html#sol/0"},{"type":"function","title":"LangChain.TextSplitter.LanguageSeparators.swift/0","doc":"","ref":"LangChain.TextSplitter.LanguageSeparators.html#swift/0"},{"type":"function","title":"LangChain.TextSplitter.LanguageSeparators.ts/0","doc":"","ref":"LangChain.TextSplitter.LanguageSeparators.html#ts/0"},{"type":"module","title":"LangChain.TextSplitter.RecursiveCharacterTextSplitter","doc":"The `RecursiveCharacterTextSplitter` is the recommended spliltter for generic text.\nIt splits the text based on a list of characters.\nIt uses each of these characters sequentially, until the text is split\ninto small enough chunks. The default list is `[\"\n\n\", \"\n\", \" \", \"\"]`.\n\nThe purpose is to prepare text for processing\nby large language models with limited context windows,\nor where a shorter context window is desired.\n\nThe main characterstinc of this splitter is that tries to keep\nparagraphs, sentences or code functions together as long as possible.\n\n`LangChain.TextSplitter.LanguageSeparators` provide separator lists for some programming and markup languages.\nTo use these Separators, it's recommended to set the `is_separator_regex` option to `true`.\n\nHow it works:\n- It splits the text at the first specified `separator` characters\n  from the given `separators` list.\n  It uses `LangChain.TextSplitter.CharacterTextSplitter` to do so.\n- For each of the above splits, it calls itself recursively\n  using the tail of the `separators` list.\n\nA `RecursiveCharacterTextSplitter` is defined using a schema.\n* `separators` - List of string that split a given text.\n  The default list is `[\"\n\n\", \"\n\", \" \", \"\"]`.\n* `chunk_size` - Integer number of tokens that a chunk should have.\n* `chunk_overlap` - Integer number of tokens that two consecutive chunks should share.\n* `keep_separator` - Either `:discard_separator`, `:start` or `:end`. If `nil`, the separator is discarded from the output chunks. `:start` and `:end` keep the separator at the start or end of the output chunks. Defaults to `start`.\n* `is_separator_regex` - Boolean defaulting to `false`. If `true`, the `separator` string is not escaped. Defaults to `false`\n* `tokenizer` - Function that takes a string and returns the number of tokens. Defaults to `&String.length/1`.","ref":"LangChain.TextSplitter.RecursiveCharacterTextSplitter.html"},{"type":"function","title":"LangChain.TextSplitter.RecursiveCharacterTextSplitter.new/1","doc":"Build a new RecursiveCharcterTextSplitter and return an `:ok`/`:error` tuple with the result.","ref":"LangChain.TextSplitter.RecursiveCharacterTextSplitter.html#new/1"},{"type":"function","title":"LangChain.TextSplitter.RecursiveCharacterTextSplitter.new!/1","doc":"Build a new RecursiveCharacterTextSplitter and return it or raise an error if invalid.","ref":"LangChain.TextSplitter.RecursiveCharacterTextSplitter.html#new!/1"},{"type":"function","title":"LangChain.TextSplitter.RecursiveCharacterTextSplitter.split_text/2","doc":"Splits text recursively based on a list of characters.\nBy default, the `separators` characters are kept at the start\n    iex> split_tags = [\",\", \".\"]\n    iex> base_params = %{chunk_size: 10, chunk_overlap: 0, separators: split_tags}\n    iex> query = \"Apple,banana,orange and tomato.\"\n    iex> splitter = RecursiveCharacterTextSplitter.new!(base_params)\n    iex> splitter |> RecursiveCharacterTextSplitter.split_text(query)\n    [\"Apple\", \",banana\", \",orange and tomato\", \".\"]\n\nWe can keep the separator at the end of a chunk, providing the\n`keep_separator: :end` option:\n    iex> split_tags = [\",\", \".\"]\n    iex> base_params = %{chunk_size: 10, chunk_overlap: 0, separators: split_tags, keep_separator: :end}\n    iex> query = \"Apple,banana,orange and tomato.\"\n    iex> splitter = RecursiveCharacterTextSplitter.new!(base_params)\n    iex> splitter |> RecursiveCharacterTextSplitter.split_text(query)\n    [\"Apple,\", \"banana,\", \"orange and tomato.\"]\n\nSee `LangChain.TextSplitter.CharacterTextSplitter` for the usage of the different options.\n\n`LanguageSeparators` provides `separators` for multiple\nprogramming and markdown languages.\nTo use these Separators, it's recommended to set the `is_separator_regex` option to `true`.\nTo split Python code:\n    iex> python_code = \"\n    ...>def hello_world():\n    ...>  print('Hello, World')\n    ...>\n    ...>\n    ...># Call the function\n    ...>hello_world()\"\n    iex> splitter =\n    ...>  RecursiveCharacterTextSplitter.new!(%{\n    ...>    separators: LanguageSeparators.python(),\n    ...>    keep_separator: :start,\n    ...>    is_separator_regex: :true,\n    ...>    chunk_size: 16,\n    ...>    chunk_overlap: 0})\n    iex> splitter |> RecursiveCharacterTextSplitter.split_text(python_code)\n    [\"def\", \"hello_world():\", \"print('Hello,\", \"World')\", \"# Call the\", \"function\", \"hello_world()\"]","ref":"LangChain.TextSplitter.RecursiveCharacterTextSplitter.html#split_text/2"},{"type":"type","title":"LangChain.TextSplitter.RecursiveCharacterTextSplitter.t/0","doc":"","ref":"LangChain.TextSplitter.RecursiveCharacterTextSplitter.html#t:t/0"},{"type":"module","title":"LangChain.Tools.Calculator","doc":"Defines a Calculator tool for performing basic math calculations.\n\nThis is an example of a pre-built `LangChain.Function` that is designed and\nconfigured for a specific purpose.\n\nThis defines a function to expose to an LLM and provides an implementation for\nthe `execute/2` function for evaluating when an LLM executes the function.\n\nWhen using the `Calculator` tool, you will either need to:\n\n* make repeated calls to run the chain as the tool is called and the results\nare then made available to the LLM before it returns the final result.\n* OR run the chain using the `mode: :until_success` option like this:\n  `LangChain.LLMChain.run(chain, mode: :until_success)`","ref":"LangChain.Tools.Calculator.html"},{"type":"module","title":"Example - LangChain.Tools.Calculator","doc":"The following is an example that uses a prompt where math is needed. What\nfollows is the verbose log output.\n\n    {:ok, updated_chain, %Message{} = message} =\n      %{llm: ChatOpenAI.new!(%{temperature: 0}), verbose: true}\n      |> LLMChain.new!()\n      |> LLMChain.add_message(\n        Message.new_user!(\"Answer the following math question: What is 100 + 300 - 200?\")\n      )\n      |> LLMChain.add_functions(Calculator.new!())\n      |> LLMChain.run(mode: :until_success)\n\nVerbose log output:\n\n    LLM: %LangChain.ChatModels.ChatOpenAI{\n      endpoint: \"https://api.openai.com/v1/chat/completions\",\n      model: \"gpt-3.5-turbo\",\n      api_key: nil,\n      temperature: 0.0,\n      frequency_penalty: 0.0,\n      receive_timeout: 60000,\n      seed: 0,\n      n: 1,\n      json_response: false,\n      stream: false,\n      max_tokens: nil,\n      user: nil\n    }\n    MESSAGES: [\n      %LangChain.Message{\n        content: \"Answer the following math question: What is 100 + 300 - 200?\",\n        index: nil,\n        status: :complete,\n        role: :user,\n        name: nil,\n        tool_calls: [],\n        tool_results: nil\n      }\n    ]\n    TOOLS: [\n      %LangChain.Function{\n        name: \"calculator\",\n        description: \"Perform basic math calculations or expressions\",\n        display_text: nil,\n        function: #Function<0.75045395/2 in LangChain.Tools.Calculator.execute>,\n        async: true,\n        parameters_schema: %{\n          type: \"object\",\n          required: [\"expression\"],\n          properties: %{\n            expression: %{\n              type: \"string\",\n              description: \"A simple mathematical expression\"\n            }\n          }\n        },\n        parameters: []\n      }\n    ]\n    SINGLE MESSAGE RESPONSE: %LangChain.Message{\n      content: nil,\n      index: 0,\n      status: :complete,\n      role: :assistant,\n      name: nil,\n      tool_calls: [\n        %LangChain.Message.ToolCall{\n          status: :complete,\n          type: :function,\n          call_id: \"call_NlHbo4R5NXTA6lHyjLdGQN9p\",\n          name: \"calculator\",\n          arguments: %{\"expression\" => \"100 + 300 - 200\"},\n          index: nil\n        }\n      ],\n      tool_results: nil\n    }\n    EXECUTING FUNCTION: \"calculator\"\n    FUNCTION RESULT: \"200\"\n    TOOL RESULTS: %LangChain.Message{\n      content: nil,\n      index: nil,\n      status: :complete,\n      role: :tool,\n      name: nil,\n      tool_calls: [],\n      tool_results: [\n        %LangChain.Message.ToolResult{\n          type: :function,\n          tool_call_id: \"call_NlHbo4R5NXTA6lHyjLdGQN9p\",\n          name: \"calculator\",\n          content: \"200\",\n          display_text: nil,\n          is_error: false\n        }\n      ]\n    }\n    SINGLE MESSAGE RESPONSE: %LangChain.Message{\n      content: \"The result of the math question \"100 + 300 - 200\" is 200.\",\n      index: 0,\n      status: :complete,\n      role: :assistant,\n      name: nil,\n      tool_calls: [],\n      tool_results: nil\n    }","ref":"LangChain.Tools.Calculator.html#module-example"},{"type":"function","title":"LangChain.Tools.Calculator.execute/2","doc":"Performs the calculation specified in the expression and returns the response\nto be used by the the LLM.","ref":"LangChain.Tools.Calculator.html#execute/2"},{"type":"function","title":"LangChain.Tools.Calculator.new/0","doc":"Define the \"calculator\" function. Returns a success/failure response.","ref":"LangChain.Tools.Calculator.html#new/0"},{"type":"function","title":"LangChain.Tools.Calculator.new!/0","doc":"Define the \"calculator\" function. Raises an exception if function creation fails.","ref":"LangChain.Tools.Calculator.html#new!/0"},{"type":"module","title":"LangChain.Config","doc":"Utility that handles interaction with the application's configuration.","ref":"LangChain.Config.html"},{"type":"function","title":"LangChain.Config.resolve/2","doc":"Resolves the given key from the application's configuration returning the\nwrapped expanded value. If the value is a function it gets evaluated, if the\nvalue is a tuple of three elements it gets applied.","ref":"LangChain.Config.html#resolve/2"},{"type":"module","title":"LangChain.Gettext","doc":"A module providing Internationalization with a gettext-based API.\n\nBy using [Gettext](https://hexdocs.pm/gettext),\nyour module gains a set of macros for translations, for example:\n\n    use Gettext, backend: LangChain.Gettext\n\n    # Simple translation\n    gettext(\"Here is the string to translate\")\n\n    # Plural translation\n    ngettext(\"Here is the string to translate\",\n             \"Here are the strings to translate\",\n             3)\n\n    # Domain-based translation\n    dgettext(\"errors\", \"Here is the error message to translate\")\n\n    # Context-based message\n    pgettext(\"email\", \"Email text to translate\")\n\nSee the [Gettext Docs](https://hexdocs.pm/gettext) for detailed usage.","ref":"LangChain.Gettext.html"},{"type":"function","title":"LangChain.Gettext.handle_missing_bindings/2","doc":"","ref":"LangChain.Gettext.html#handle_missing_bindings/2"},{"type":"function","title":"LangChain.Gettext.handle_missing_plural_translation/7","doc":"","ref":"LangChain.Gettext.html#handle_missing_plural_translation/7"},{"type":"function","title":"LangChain.Gettext.handle_missing_translation/5","doc":"","ref":"LangChain.Gettext.html#handle_missing_translation/5"},{"type":"module","title":"LangChain.Utils","doc":"Collection of helpful utilities mostly for internal use.","ref":"LangChain.Utils.html"},{"type":"function","title":"LangChain.Utils.assign_string_value/3","doc":"Changeset helper function for processing streamed text from an LLM.\n\nA delta of \" \" a single empty space is expected. The \"cast\" process of the\nchangeset turns this into `nil` causing us to lose data.\n\nWe want to take whatever we are given here.","ref":"LangChain.Utils.html#assign_string_value/3"},{"type":"function","title":"LangChain.Utils.changeset_error_to_string/1","doc":"Return changeset errors as text with comma separated description.","ref":"LangChain.Utils.html#changeset_error_to_string/1"},{"type":"function","title":"LangChain.Utils.conditionally_add_to_map/3","doc":"Only add the key to the map if the value is present. When the value is a list,\nthe key will not be added when the list is empty. If the value is `nil`, it\nwill not be added.","ref":"LangChain.Utils.html#conditionally_add_to_map/3"},{"type":"function","title":"LangChain.Utils.fire_streamed_callback/2","doc":"Fire a streaming callback if present.","ref":"LangChain.Utils.html#fire_streamed_callback/2"},{"type":"function","title":"LangChain.Utils.handle_stream_fn/3","doc":"Creates and returns an anonymous function to handle the streaming response\nfrom an API.\n\nAccepts the following functions that handle the API-specific requirements:\n\n- `decode_stream_fn` - a function that parses the raw results from an API. It\n  deals with the specifics or oddities of a data source. The results come back\n  as `{[list_of_parsed_json_maps], \"incomplete text to buffer\"}`. In some\n  cases, a API may span the JSON data response across messages. This function\n  assembles what is complete and returns any incomplete portion that is passed\n  in on the next iteration of the function.\n\n- `transform_data_fn` - a function that is executed to process the parsed\n  JSON data in the form of an Elixir map into a LangChain struct of the\n  appropriate type.\n\n- `callback_fn` - a function that receives a successful result of from the\n  `transform_data_fn`.","ref":"LangChain.Utils.html#handle_stream_fn/3"},{"type":"function","title":"LangChain.Utils.migrate_to_content_parts/1","doc":"Migrate a string content to use `LangChain.Message.ContentPart`. This is for\nbackward compatibility with models that don't yet support ContentPart while\nproviding a more consistent API.\n\nThis can be used with Message contents and ToolResult contents.","ref":"LangChain.Utils.html#migrate_to_content_parts/1"},{"type":"function","title":"LangChain.Utils.module_from_name/1","doc":"Return an `{:ok, module}` when the string successfully converts to an existing\nmodule.","ref":"LangChain.Utils.html#module_from_name/1"},{"type":"function","title":"LangChain.Utils.put_in_list/3","doc":"Put the value in the list at the desired index. If the index does not exist,\nreturn an updated list where it now exists with the value in that index.","ref":"LangChain.Utils.html#put_in_list/3"},{"type":"function","title":"LangChain.Utils.replace_system_message!/2","doc":"Replace the system message with a new system message. This retains all other\nmessages as-is. An error is raised if there are more than 1 system messages.","ref":"LangChain.Utils.html#replace_system_message!/2"},{"type":"function","title":"LangChain.Utils.split_system_message/2","doc":"Split the messages into \"system\" and \"other\".\nRaises an error with the specified error message if more than 1 system message present.\nReturns a tuple with the single system message and the list of other messages.","ref":"LangChain.Utils.html#split_system_message/2"},{"type":"function","title":"LangChain.Utils.stringify_keys/1","doc":"Convert map atom keys to strings\n\nOriginal source: https://gist.github.com/kipcole9/0bd4c6fb6109bfec9955f785087f53fb","ref":"LangChain.Utils.html#stringify_keys/1"},{"type":"function","title":"LangChain.Utils.to_serializable_map/3","doc":"Given a struct, create a map with the selected keys converted to strings.\nAdditionally includes a `version` number for the data.","ref":"LangChain.Utils.html#to_serializable_map/3"},{"type":"function","title":"LangChain.Utils.translate_error/1","doc":"Translates an error message using gettext.","ref":"LangChain.Utils.html#translate_error/1"},{"type":"function","title":"LangChain.Utils.translate_errors/2","doc":"Translates the errors for a field from a keyword list of errors.","ref":"LangChain.Utils.html#translate_errors/2"},{"type":"function","title":"LangChain.Utils.validate_llm_is_struct/1","doc":"Validation helper. Validates a struct changeset that the LLM is a struct.","ref":"LangChain.Utils.html#validate_llm_is_struct/1"},{"type":"type","title":"LangChain.Utils.callback_data/0","doc":"","ref":"LangChain.Utils.html#t:callback_data/0"},{"type":"module","title":"LangChain.Utils.BedrockConfig","doc":"Configuration for AWS Bedrock.","ref":"LangChain.Utils.BedrockConfig.html"},{"type":"module","title":"Examples - LangChain.Utils.BedrockConfig","doc":"For applications hosted in AWS, [ExAws](https://hex.pm/packages/ex_aws) caches\ntemporary credentials (when running on AWS), so in the credentials function\nyou can pull the current cached credentials from `ExAws`.\n\n    ChatAnthropic.new!(%{\n      model: \"anthropic.claude-3-5-sonnet-20241022-v2:0\",\n      bedrock: %{\n        credentials: fn ->\n          ExAws.Config.new(:s3)\n          |> Map.take([:access_key_id, :secret_access_key])\n          |> Map.to_list()\n        end,\n        region: \"us-west-2\"\n      }\n    })\n\nFor applications hosted anywhere, you can configure the Bedrock settings into\nthe LangChain config like this (recommended for `config/runtime.exs`):\n\n    config :langchain,\n      aws_access_key_id: System.fetch_env!(\"AWS_ACCESS_KEY_ID\"),\n      aws_secret_access_key: System.fetch_env!(\"AWS_SECRET_ACCESS_KEY\"),\n      aws_region: System.get_env(\"AWS_REGION\", \"us-west-1\")\n\nThen, when you want to later use a Bedrock Anthropic model, this is will load\nit:\n\n      ChatAnthropic.new!(%{\n        model: \"anthropic.claude-3-5-sonnet-20241022-v2:0\",\n        bedrock: BedrockConfig.from_application_env!()\n      })","ref":"LangChain.Utils.BedrockConfig.html#module-examples"},{"type":"function","title":"LangChain.Utils.BedrockConfig.aws_sigv4_opts/1","doc":"","ref":"LangChain.Utils.BedrockConfig.html#aws_sigv4_opts/1"},{"type":"function","title":"LangChain.Utils.BedrockConfig.changeset/2","doc":"","ref":"LangChain.Utils.BedrockConfig.html#changeset/2"},{"type":"function","title":"LangChain.Utils.BedrockConfig.from_application_env!/0","doc":"Loads the Bedrock config settings from the previously configured Application settings.\n\n`config/runtime.exs`:\n\n    config :langchain,\n      aws_access_key_id: System.fetch_env!(\"AWS_ACCESS_KEY_ID\"),\n      aws_secret_access_key: System.fetch_env!(\"AWS_SECRET_ACCESS_KEY\"),\n      aws_region: System.get_env(\"AWS_REGION\", \"us-west-1\")","ref":"LangChain.Utils.BedrockConfig.html#from_application_env!/0"},{"type":"function","title":"LangChain.Utils.BedrockConfig.url/2","doc":"","ref":"LangChain.Utils.BedrockConfig.html#url/2"},{"type":"type","title":"LangChain.Utils.BedrockConfig.t/0","doc":"","ref":"LangChain.Utils.BedrockConfig.html#t:t/0"},{"type":"module","title":"LangChain.Utils.ChainResult","doc":"Module to help when working with the results of a chain.","ref":"LangChain.Utils.ChainResult.html"},{"type":"function","title":"LangChain.Utils.ChainResult.to_map/3","doc":"Write the result to the given map as the value of the given key.","ref":"LangChain.Utils.ChainResult.html#to_map/3"},{"type":"function","title":"LangChain.Utils.ChainResult.to_map!/3","doc":"Write the result to the given map as the value of the given key. If invalid,\nan exception is raised.","ref":"LangChain.Utils.ChainResult.html#to_map!/3"},{"type":"function","title":"LangChain.Utils.ChainResult.to_string/1","doc":"Return the result of the chain as a string. Returned in an `:ok` tuple format.\nAn `{:error, reason}` is returned for various reasons. These include:\n- The last message of the chain is not an `:assistant` message.\n- The last message of the chain is incomplete.\n- There is no last message.\n\nThis supports passing in the final, updated LLMChain, or the result of the\n`LLMChain.run/2` function.","ref":"LangChain.Utils.ChainResult.html#to_string/1"},{"type":"function","title":"LangChain.Utils.ChainResult.to_string!/1","doc":"Return the last message's content when it is valid to use it. Otherwise it\nraises and exception with the reason why it cannot be used. See the docs for\n`to_string/2` for details.","ref":"LangChain.Utils.ChainResult.html#to_string!/1"},{"type":"module","title":"LangChain.Utils.ChatTemplates","doc":"Functions for converting messages into the various commonly used chat template\nformats.","ref":"LangChain.Utils.ChatTemplates.html"},{"type":"module","title":"Format examples - LangChain.Utils.ChatTemplates","doc":"There are currently no industry standards around model's chat formats. For any\ngiven model, it's documentation and config may need to be inspected for it's\nformat, and it may be something not supported at this time.\n\n### `:inst`\n\n```\n [INST] System message. User message. [/INST]\nAssistant response\n[INST] User message. [/INST] \nAssistant response\n```\n\nNote: The `:inst` format has no concept of a system message. It will be\ncombined with the first user message\n\n### `:im_start`\n\n```\n<|im_start|>user\nUser message.<|im_end|>\n<|im_start|>assistant\nAssistant response.<|im_end|>\n<|im_start|>user\nUser message.<|im_end|>\n<|im_start|>assistant\n```\n\nNote: The `:im_start` format has no concept of a system message. It will be\ncombined with the first user message.\n\n### `:llama_2`\n\n```\n [INST] < >\nSystem message.\n< >\n\nUser message [/INST] Assistant response   [INST] User message. [/INST]\n```\n\nNote: The `:llama_2` format supports specific system messages. It is a\nvariation of the `:inst` format.\n\n### `:llama_3`\n\n```\n<|begin_of_text|>\n<|start_header_id|>system<|end_header_id|>\n\nSystem message.<|eot_id|>\n<|start_header_id|>user<|end_header_id|>\n\nUser message.<|eot_id|>\n<|start_header_id|>assistant<|end_header_id|>\n\nAssistant message.<|eot_id|>\n```\n\n### `:zephyr`\n\n```\n<|system|>\nSystem message. \n<|user|>\nUser message. \n<|assistant|>\nAssistant message.\n<|user|>\nUser message. \n```\n\nNote: The `:zephyr` format supports specific system messages.\n\n\n### `:phi_4`\n\nThe `:phi_4` template format is also supported.","ref":"LangChain.Utils.ChatTemplates.html#module-format-examples"},{"type":"module","title":"Template callback - LangChain.Utils.ChatTemplates","doc":"It's possible to pass a callback as a template.\nThe function receives the list of messages as first argument and `opts` as second and must return a string.","ref":"LangChain.Utils.ChatTemplates.html#module-template-callback"},{"type":"function","title":"LangChain.Utils.ChatTemplates.apply_chat_template!/3","doc":"Transform a list of messages into a text prompt in the desired format for the\nLLM.","ref":"LangChain.Utils.ChatTemplates.html#apply_chat_template!/3"},{"type":"function","title":"Options - LangChain.Utils.ChatTemplates.apply_chat_template!/3","doc":"- `:add_generation_prompt` - Boolean. Defaults to `true` when the last message\n  is a user prompt. Depending on the format, when a user message is the last\n  message, then the text prompt should begin the portion for the assistant to\n  trigger the assistant's text generation.","ref":"LangChain.Utils.ChatTemplates.html#apply_chat_template!/3-options"},{"type":"function","title":"LangChain.Utils.ChatTemplates.apply_chat_template_with_tools!/4","doc":"Transform a list of messages into a text prompt in the desired format for the\nLLM.\nAnd adds tool configuration.","ref":"LangChain.Utils.ChatTemplates.html#apply_chat_template_with_tools!/4"},{"type":"function","title":"Options - LangChain.Utils.ChatTemplates.apply_chat_template_with_tools!/4","doc":"- `:add_generation_prompt` - Boolean. Defaults to `true` when the last message\n  is a user prompt. Depending on the format, when a user message is the last\n  message, then the text prompt should begin the portion for the assistant to\n  trigger the assistant's text generation.","ref":"LangChain.Utils.ChatTemplates.html#apply_chat_template_with_tools!/4-options"},{"type":"function","title":"LangChain.Utils.ChatTemplates.llama_3_1_custom_tool_calling_parameter_conversion/1","doc":"","ref":"LangChain.Utils.ChatTemplates.html#llama_3_1_custom_tool_calling_parameter_conversion/1"},{"type":"function","title":"LangChain.Utils.ChatTemplates.prep_and_validate_messages/1","doc":"Validate that the message are in a supported format. Returns the system\nmessage (or uses a default one).\n\nReturns: `{System Message, First User Message, Other Messages}`\n\nIf there is an issue, an exception is raised. Reasons for an exception:\n\n- Only 1 system message is allowed and, if included, it is the first message.\n- Non-system messages must begin with a user message or assistant message.\n\nRecent change:\n- Alternating messages between user / assistant / user / assistant are no longer enforced as not every model has issues.\n- It is up to the programmer to enforce this if this is something they need.","ref":"LangChain.Utils.ChatTemplates.html#prep_and_validate_messages/1"},{"type":"type","title":"LangChain.Utils.ChatTemplates.chat_format/0","doc":"","ref":"LangChain.Utils.ChatTemplates.html#t:chat_format/0"},{"type":"type","title":"LangChain.Utils.ChatTemplates.template_callback/0","doc":"","ref":"LangChain.Utils.ChatTemplates.html#t:template_callback/0"},{"type":"exception","title":"LangChain.LangChainError","doc":"Exception used for raising LangChain specific errors.\n\nIt stores the `:message`. Passing an Ecto.Changeset with an error\nconverts the error into a string message.\n\n    raise LangChainError, changeset\n\n    raise LangChainError, \"Message text\"\n\n    raise LangChainError, type: \"overloaded\", message: \"Message text\"\n\nThe error struct contains the following keys:\n\n- `:type` - A string code to make detecting and responding to specific errors easier. This may have values like \"length\" or \"overloaded\". The specific meaning of the type is dependent on the service or model.\n\n- `:message` - A string representation or explanation of the error.\n\n- `:original` - If a exception was caught and wrapped into a LangChainError, this may be the original message that was encountered.","ref":"LangChain.LangChainError.html"},{"type":"function","title":"LangChain.LangChainError.exception/1","doc":"Create the exception using either a message or a changeset who's errors are\nconverted to a message.","ref":"LangChain.LangChainError.html#exception/1"},{"type":"function","title":"LangChain.LangChainError.format_exception/3","doc":"Formats the exception as a string using a stacktrace to provide context.","ref":"LangChain.LangChainError.html#format_exception/3"},{"type":"type","title":"LangChain.LangChainError.t/0","doc":"","ref":"LangChain.LangChainError.html#t:t/0"},{"type":"extras","title":"README","doc":"[![Elixir CI](https://github.com/brainlid/langchain/actions/workflows/elixir.yml/badge.svg)](https://github.com/brainlid/langchain/actions/workflows/elixir.yml)\n[![Module Version](https://img.shields.io/hexpm/v/langchain.svg)](https://hex.pm/packages/langchain)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/langchain)\n\n# ![Logo with chat chain links](https://github.com/brainlid/langchain/blob/main/images/elixir-langchain-link-logo_32px.png?raw=true) Elixir LangChain\n\nElixir LangChain enables Elixir applications to integrate AI services and self-hosted models into an application.\n\n**Currently supported AI services:**\n\n| Model | v0.3.x | v0.4.x |\n|-------|---------|---------|\n| OpenAI ChatGPT | ✓ | ✓ |\n| OpenAI DALL-e 2 (image generation) | ✓ | ? |\n| Anthropic Claude | ✓ | ✓ |\n| Anthropic Claude (thinking) | X | ✓ |\n| Google Gemini | ✓ | X |\n| Google Vertex AI* | ✓ | X |\n| Ollama | ✓ | ? |\n| Mistral | ✓ | X |\n| Bumblebee self-hosted models** | ✓ | ? |\n| LMStudio*** | ✓ | ? |\n| Perplexity | ✓ | ? |\n\n- *Google Vertex AI is Google's enterprise offering\n- **Bumblebee self-hosted models - including Llama, Mistral and Zephyr\n- ***[LMStudio](https://lmstudio.ai/docs/api/endpoints/openai) via their OpenAI compatibility API\n\n**LangChain** is short for Language Chain. An LLM, or Large Language Model, is the \"Language\" part. This library makes it easier for Elixir applications to \"chain\" or connect different processes, integrations, libraries, services, or functionality together with an LLM.\n\n**LangChain** is a framework for developing applications powered by language models. It enables applications that are:\n\n- **Data-aware:** connect a language model to other sources of data\n- **Agentic:** allow a language model to interact with its environment\n\nThe main value props of LangChain are:\n\n1. **Components:** abstractions for working with language models, along with a collection of implementations for each abstraction. Components are modular and easy-to-use, whether you are using the rest of the LangChain framework or not\n1. **Off-the-shelf chains:** a structured assembly of components for accomplishing specific higher-level tasks\n\nOff-the-shelf chains make it easy to get started. For more complex applications and nuanced use-cases, components make it easy to customize existing chains or build new ones.","ref":"readme.html"},{"type":"extras","title":"What is this? - README","doc":"Large Language Models (LLMs) are emerging as a transformative technology, enabling developers to build applications that they previously could not. But using these LLMs in isolation is often not enough to create a truly powerful app - the real power comes when you can combine them with other sources of computation or knowledge.\n\nThis library is aimed at assisting in the development of those types of applications.","ref":"readme.html#what-is-this"},{"type":"extras","title":"Documentation - README","doc":"The online documentation can be [found here](https://hexdocs.pm/langchain).","ref":"readme.html#documentation"},{"type":"extras","title":"Demo - README","doc":"Check out the [demo project](https://github.com/brainlid/langchain_demo) that you can download and review.","ref":"readme.html#demo"},{"type":"extras","title":"Relationship with JavaScript and Python LangChain - README","doc":"This library is written in [Elixir](https://elixir-lang.org/) and intended to be used with Elixir applications. The original libraries are [LangChain JS/TS](https://js.langchain.com/) and [LangChain Python](https://python.langchain.com/).\n\nThe JavaScript and Python projects aim to integrate with each other as seamlessly as possible. The intended integration is so strong that that all objects (prompts, LLMs, chains, etc) are designed in a way where they can be serialized and shared between the two languages.\n\nThis Elixir version does not aim for parity with the JavaScript and Python libraries. Why not?\n\n- JavaScript and Python are both Object Oriented languages. Elixir is Functional. We're not going to force a design that doesn't apply.\n- The JS and Python versions started before conversational LLMs were standard. They put a lot of effort into preserving history (like a conversation) when the LLM didn't support it. We're not doing that here.\n\nThis library was heavily inspired by, and based on, the way the JavaScript library actually worked and interacted with an LLM.","ref":"readme.html#relationship-with-javascript-and-python-langchain"},{"type":"extras","title":"Installation - README","doc":"The package can be installed by adding `langchain` to your list of dependencies\nin `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:langchain, \"0.4.0-rc.1\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Configuration - README","doc":"Currently, the library is written to use the `Req` library for making API calls.\n\nYou can configure an _organization ID_, and _API key_ for OpenAI's API, but this library also works with [other compatible APIs](#alternative-openai-compatible-apis) as well as other services and even [local models running on Bumblebee](#bumblebee-chat-support).\n\n`config/runtime.exs`:\n\n```elixir\nconfig :langchain, openai_key: System.fetch_env!(\"OPENAI_API_KEY\")\nconfig :langchain, openai_org_id: System.fetch_env!(\"OPENAI_ORG_ID\")\n# OR\nconfig :langchain, openai_key: \"YOUR SECRET KEY\"\nconfig :langchain, openai_org_id: \"YOUR_OPENAI_ORG_ID\"\n\nconfig :langchain, :anthropic_key, System.fetch_env!(\"ANTHROPIC_API_KEY\")\n```\n\nIt's possible to use a function or a tuple to resolve the secret:\n\n```elixir\nconfig :langchain, openai_key: {MyApp.Secrets, :openai_api_key, []}\nconfig :langchain, openai_org_id: {MyApp.Secrets, :openai_org_id, []}\n# OR\nconfig :langchain, openai_key: fn -> System.fetch_env!(\"OPENAI_API_KEY\") end\nconfig :langchain, openai_org_id: fn -> System.fetch_env!(\"OPENAI_ORG_ID\") end\n```\n\nThe API keys should be treated as secrets and not checked into your repository.\n\nFor [fly.io](https://fly.io), adding the secrets looks like this:\n\n```\nfly secrets set OPENAI_API_KEY=MyOpenAIApiKey\nfly secrets set ANTHROPIC_API_KEY=MyAnthropicApiKey\n```\n\nA list of models to use:\n\n- [Anthropic Claude models](https://docs.anthropic.com/en/docs/about-claude/models)\n- [Anthropic models on AWS Bedrock](https://docs.anthropic.com/en/api/claude-on-amazon-bedrock#accessing-bedrock)\n- [OpenAI models](https://platform.openai.com/docs/models)\n- [OpenAI models on Azure](https://learn.microsoft.com/en-us/azure/ai-services/openai/concepts/models)\n- [Gemini AI models](https://ai.google.dev/gemini-api/docs/models/gemini)","ref":"readme.html#configuration"},{"type":"extras","title":"Prompt caching - README","doc":"ChatGPT and Claude both offer prefix-based prompt caching, which can offer cost and performance benefits for longer prompts. Gemini offers context caching, which is similar.\n\n- [ChatGPT's prompt caching](https://openai.com/index/api-prompt-caching/) is automatic for prompts longer than 1024 tokens, caching the longest common prefix.\n- [Claude's prompt caching](https://docs.anthropic.com/en/docs/build-with-claude/prompt-caching) is not automatic. It's prefixing processes tools, system, and then messages, in that order, up to and including the block designated with {\"cache_control\": {\"type\": \"ephemeral\"}} . See LangChain.ChatModels.ChatAnthropicTest and for an example.\n- [Gemini's context caching]((https://ai.google.dev/gemini-api/docs/caching?lang=python)) requires a separate call which is not supported by Langchain.","ref":"readme.html#prompt-caching"},{"type":"extras","title":"Usage - README","doc":"The central module in this library is `LangChain.Chains.LLMChain`. Most other pieces are either inputs to this, or structures used by it. For understanding how to use the library, start there.","ref":"readme.html#usage"},{"type":"extras","title":"Exposing a custom Elixir function to ChatGPT - README","doc":"A really powerful feature of LangChain is making it easy to integrate an LLM into your application and expose features, data, and functionality _from_ your application to the LLM.\n\n \n\nA `LangChain.Function` bridges the gap between the LLM and our application code. We choose what to expose and using `context`, we can ensure any actions are limited to what the user has permission to do and access.\n\nFor an interactive example, refer to the project [Livebook notebook \"LangChain: Executing Custom Elixir Functions\"](notebooks/custom_functions.livemd).\n\nThe following is an example of a function that receives parameter arguments.\n\n```elixir\nalias LangChain.Function\nalias LangChain.Message\nalias LangChain.Chains.LLMChain\nalias LangChain.ChatModels.ChatOpenAI\nalias LangChain.Utils.ChainResult\n\n# map of data we want to be passed as `context` to the function when\n# executed.\ncustom_context = %{\n  \"user_id\" => 123,\n  \"hairbrush\" => \"drawer\",\n  \"dog\" => \"backyard\",\n  \"sandwich\" => \"kitchen\"\n}\n\n# a custom Elixir function made available to the LLM\ncustom_fn =\n  Function.new!(%{\n    name: \"custom\",\n    description: \"Returns the location of the requested element or item.\",\n    parameters_schema: %{\n      type: \"object\",\n      properties: %{\n        thing: %{\n          type: \"string\",\n          description: \"The thing whose location is being requested.\"\n        }\n      },\n      required: [\"thing\"]\n    },\n    function: fn %{\"thing\" => thing} = _arguments, context ->\n      # our context is a pretend item/location location map\n      {:ok, context[thing]}\n    end\n  })\n\n# create and run the chain\n{:ok, updated_chain} =\n  LLMChain.new!(%{\n    llm: ChatOpenAI.new!(),\n    custom_context: custom_context,\n    verbose: true\n  })\n  |> LLMChain.add_tools(custom_fn)\n  |> LLMChain.add_message(Message.new_user!(\"Where is the hairbrush located?\"))\n  |> LLMChain.run(mode: :while_needs_response)\n\n# print the LLM's answer\nIO.puts(ChainResult.to_string!(updated_chain))\n# => \"The hairbrush is located in the drawer.\"\n```","ref":"readme.html#exposing-a-custom-elixir-function-to-chatgpt"},{"type":"extras","title":"Alternative OpenAI compatible APIs - README","doc":"There are several services or self-hosted applications that provide an OpenAI compatible API for ChatGPT-like behavior. To use a service like that, the `endpoint` of the `ChatOpenAI` struct can be pointed to an API compatible `endpoint` for chats.\n\nFor example, if a locally running service provided that feature, the following code could connect to the service:\n\n```elixir\n{:ok, updated_chain} =\n  LLMChain.new!(%{\n    llm: ChatOpenAI.new!(%{endpoint: \"http://localhost:1234/v1/chat/completions\"}),\n  })\n  |> LLMChain.add_message(Message.new_user!(\"Hello!\"))\n  |> LLMChain.run()\n```","ref":"readme.html#alternative-openai-compatible-apis"},{"type":"extras","title":"Bumblebee Chat Support - README","doc":"Bumblebee hosted chat models are supported. There is built-in support for Llama 2, Mistral, and Zephyr models.\n\nCurrently, function calling is only supported for llama 3.1 Json Tool calling for Llama 2, Mistral, and Zephyr is NOT supported.\nThere is an example notebook in the notebook folder.\n\n    ChatBumblebee.new!(%{\n      serving: @serving_name,\n      template_format: @template_format,\n      receive_timeout: @receive_timeout,\n      stream: true\n    })\n\nThe `serving` is the module name of the `Nx.Serving` that is hosting the model.\n\nSee the [`LangChain.ChatModels.ChatBumblebee` documentation](https://hexdocs.pm/langchain/LangChain.ChatModels.ChatBumblebee.html) for more details.","ref":"readme.html#bumblebee-chat-support"},{"type":"extras","title":"Testing - README","doc":"Before you can run the tests, make sure you have the environment variables set.\n\nYou can do this by running:\n\n```\nsource .envrc_template\n```\n\nOr you can copy it to `.envrc` and populate it with your private API values.\n\nTo run all the tests including the ones that perform live calls against the OpenAI API, use the following command:\n\n```\nmix test --include live_call\nmix test --include live_open_ai\nmix test --include live_ollama_ai\nmix test --include live_anthropic\nmix test --include live_mistral_ai\nmix test test/tools/calculator_test.exs --include live_call\n```\n\nNOTE: This will use the configured API credentials which creates billable events.\n\nOtherwise, running the following will only run local tests making no external API calls:\n\n```\nmix test\n```\n\nExecuting a specific test, whether it is a `live_call` or not, will execute it creating a potentially billable event.\n\nWhen doing local development on the `LangChain` library itself, rename the `.envrc_template` to `.envrc` and populate it with your private API values. This is only used when running live test when explicitly requested.\n\nUse a tool like [Direnv](https://direnv.net/) or [Dotenv](https://github.com/motdotla/dotenv) to load the API values into the ENV when using the library locally.\n\n**Multi-modal support:**\n\nLangChain now supports multi-modal messages and tool results. This means you can include text, images, files, and even \"thinking\" blocks in a single message using ContentParts. See module docs for details. Support for this depends on the LLM and service. Not all models may yet support all modalities.","ref":"readme.html#testing"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v0.4.0-rc.1 - Changelog","doc":"---","ref":"changelog.html#v0-4-0-rc-1"},{"type":"extras","title":"Breaking Changes - Changelog","doc":"- ToolResult `content` now supports a list of ContentParts, not just strings. Functions can return a ToolResult directly for advanced control (e.g., cache control, processed_content).\n- Expanded multi-modal support: messages and tool results can now include text, images, files, and thinking blocks as ContentParts.\n- LLMChain: Added `async_tool_timeout` config; improved fallback and error handling.\n- `LangChain.Function` changed the default for `async` to `false`. If you want async execution, set `async: true` explicitly when defining your function.\n- The `on_llm_new_delta` callback now receives a list of `MessageDelta` structs instead of a single one. To merge the received deltas into your chain for display, use:\n\n```elixir\nupdated_chain = LLMChain.merge_deltas(current_llm_chain, deltas)\n```","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Upgrading from v0.4.0-rc.0 - v0.4.0-rc.1 - Changelog","doc":"- If you return a ToolResult from a function, you can now use ContentParts for richer responses. See module docs for details.\n- If you use custom chunking logic, see the new tokenizer support in TextSplitter.\n- If you are displaying streamed MessageDelta results using the `on_llm_new_delta` callback, you will need to update your callback function to expect a list of MessageDeltas and you can use the new `LLMChain.merge_deltas` function for merging them into your chain. The resulting merged delta can be used for display.\n\n#### Model Compatibility\n- The following models have been verified with this version:\n  - ChatOpenAI\n  - ChatAnthropic\n  - ChatGoogleAI\n- There are known broken live tests with Perplexity and likely others. Not all models are currently verified or supported in this release.\n\n**Assistance is requested** for verifying/updating other models and their tests.","ref":"changelog.html#upgrading-from-v0-4-0-rc-0-v0-4-0-rc-1"},{"type":"extras","title":"Added - Changelog","doc":"- Telemetry to `LLMChain.run_until_tool_used` for better observability.\n- Google Gemini 2.0+ supports native Google Search as a tool.\n- MistralAI: Structured output support.\n- ChatGoogleAI: `verbose_api` option; updated default model to `gemini-2.5-pro`.\n- TextSplitter: Added configurable tokenizer support for chunking by tokens, not just characters.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- ChatOpenAI: Improved handling of ContentParts in ToolResults; better support for reasoning models and robust API options.\n- ChatGoogleAI: Improved ToolResult handling for ContentParts; better error and token usage reporting.\n- ChatAnthropic: Expanded prompt caching support and documentation; improved error and token usage handling.\n- LLMChain: Improved fallback and error handling; added async tool timeout config.\n- TextSplitter: Now supports custom tokenizers for chunking.","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- ToolCalls: Fixed issues with nil tool_calls and tool call processing.\n- Token Usage: Fixed token usage reporting for GoogleAI.\n- Bedrock Stream Decoder: Fixed chunk order issue.","ref":"changelog.html#fixed"},{"type":"extras","title":"v0.4.0-rc.0 - Changelog","doc":"This includes several breaking changes:\n\n- Not all chat models are supported and updated yet. Currently only **OpenAI** and **Claude**\n- Assistant messages are all assumed to be a list of `ContentPart` structs, supporting text, thinking, and more in the future like images\n- A Message includes the TokenUsage in `Message.metadata.usage` after received.\n- To display a MessageDelta as it is being streamed back, use `MessageDelta.merged_content`.\n\nUse the v0.3.x releases for models that are not yet supported.\n\n| Model | v0.3.x | v0.4.x |\n|-------|---------|---------|\n| OpenAI ChatGPT | ✓ | ✓ |\n| OpenAI DALL-e 2 (image generation) | ✓ | ? |\n| Anthropic Claude | ✓ | ✓ |\n| Anthropic Claude (thinking) | X | ✓ |\n| Google Gemini | ✓ | ✓ |\n| Google Vertex AI | ✓ | ✓ |\n| Ollama | ✓ | ? |\n| Mistral | ✓ | X |\n| Bumblebee self-hosted models | ✓ | ? |\n| LMStudio | ✓ | ? |\n| Perplexity | ✓ | ? |","ref":"changelog.html#v0-4-0-rc-0"},{"type":"extras","title":"Upgrade from v0.3.3 to v0.4.x - Changelog","doc":"As LLM services get more advanced, they have begun returning multi-modal responses. For some time, they have been accepting multi-modal requests, meaning an image and text could be submitted at the same time.\n\nNow, LLMs have changed to return multi-modal responses. This means they may return text along with an image. This is currently most common with receiving a \"thinking\" response separate from their text response.\n\nIn an effort to provide a consistent interface to many different LLMs, now **all** message responses with content (text, image, thinking, etc.) will be represented as a list of `ContentPart` structs.\n\nThis is a breaking change and may require application updates to adapt.","ref":"changelog.html#upgrade-from-v0-3-3-to-v0-4-x"},{"type":"extras","title":"Message Changes - Changelog","doc":"Where this was received before:\n\n```elixir\n%Message{content: \"this is a string\"}\n```\n\nThis is received now:\n\n```elixir\n%Message{content: [%ContentPart{type: :text, content: \"this is a string\"}]}\n```\n\nThis can be quickly turned back into plain text using `LangChain.Message.ContentPart.parts_to_string/1`.\n\nIt looks like this:\n```elixir\nmessage = %Message{content: [%ContentPart{type: :text, content: \"this is a string\"}]}\nContentPart.parts_to_string(message.content)\n#=> \"this is a string\"\n```\n\nThis also handles if multiple text content parts are received:\n```elixir\nmessage = %Message{content: [\n  %ContentPart{type: :text, content: \"this is a string\"},\n  %ContentPart{type: :text, content: \"this is another string\"},\n]}\nContentPart.parts_to_string(message.content)\n#=> \"this is a string\\n\\nthisis another string\"\n```\n\nFor constructing your own messages, this is auto-converted for you:\n\n```elixir\nMessage.new_user!(\"Howdy!\")\n#=> %Message{role: :user, content: [%ContentPart{type: :text, content: \"Howdy!\"}]}\n```\n\nThis can also be constructed like this:\n\n```elixir\nMessage.new_user!([ContentPart.text!(\"Howdy!\")])\n#=> %Message{role: :user, content: [%ContentPart{type: :text, content: \"Howdy!\"}]}\n```\n\nThe change is more significant when handling an assistant response message.","ref":"changelog.html#message-changes"},{"type":"extras","title":"MessageDelta Changes - Changelog","doc":"When streaming a response and getting back `MessageDelta`s, these now have a `merged_content` field that combines the different streamed back content types into their complete pieces. These pieces can represent different indexes in the list of received ContentParts.\n\nSee the MessageDelta module docs for more information on `merged_content`.\n\nThis is important because when needing to display the deltas as they are being received, it is now the `merged_content` field that should be used.","ref":"changelog.html#messagedelta-changes"},{"type":"extras","title":"TokenUsage - Changelog","doc":"Another significant change is the moving of TokenUsage from a separated callback to being directly attached to a Message's `metadata`. Token usage is accumulated, as it is split out typically on the first and last delta's received.\n\nAfter an LLMChain.run, the `updated_chain.last_message.metadata.usage` will contain the %TokenUsage{} information.\n\nA related change was to move the TokenUsage callback from the OpenAI and Anthropic chat models to the LLMChain. This means the same event will fire, but it will fire when it's fully received and assembled.","ref":"changelog.html#tokenusage"},{"type":"extras","title":"v0.3.3 (2025-03-17) - Changelog","doc":"This is a milestone release before staring v0.4.0 which introduces breaking changes, but importantly adds support for \"thinking\" models.","ref":"changelog.html#v0-3-3-2025-03-17"},{"type":"extras","title":"Added - Changelog","doc":"- Added telemetry support https://github.com/brainlid/langchain/pull/284\n- Added `LLMChain.run_until_tool_used/3` function https://github.com/brainlid/langchain/pull/292\n- Support for file uploads with file_id in ChatOpenAI https://github.com/brainlid/langchain/pull/283\n- Support for json_response in ChatGoogleAI https://github.com/brainlid/langchain/pull/277\n- Support for streaming responses from Mistral https://github.com/brainlid/langchain/pull/287\n- Support for file URLs in Google AI https://github.com/brainlid/langchain/pull/286\n- Support for PDF content with OpenAI model https://github.com/brainlid/langchain/pull/275\n- Support for caching tool results in Anthropic calls https://github.com/brainlid/langchain/pull/269\n- Support for choosing Anthropic beta headers https://github.com/brainlid/langchain/pull/273","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Fixed options being passed to the Ollama chat API https://github.com/brainlid/langchain/pull/179\n- Fixed media URIs for Google Vertex https://github.com/brainlid/langchain/pull/242\n- Fixed OpenAI verbose_api https://github.com/brainlid/langchain/pull/274\n- Improved documentation for callbacks and content parts\n- Upgraded gettext and migrated https://github.com/brainlid/langchain/pull/271","ref":"changelog.html#changed"},{"type":"extras","title":"Fixed - Changelog","doc":"- Added validation to check if requested tool_name exists in chain\n- Fixed various documentation issues and typos\n- Fixed callback links in documentation","ref":"changelog.html#fixed"},{"type":"extras","title":"v0.3.2 (2025-03-17) - Changelog","doc":"","ref":"changelog.html#v0-3-2-2025-03-17"},{"type":"extras","title":"Added - Changelog","doc":"- Support for Perplexity AI https://github.com/brainlid/langchain/pull/261\n- Enable tool support for ollama (if the model supports it and only when not streaming) https://github.com/brainlid/langchain/pull/164\n- Added `on_message_processed` callback when tool response is created: When a Tool response message is created, it already fired an on_tool_response_created, but it now also fires the more general on_message_processed, because a tool result can certainly be considered being processed. https://github.com/brainlid/langchain/pull/248\n- Added Tool Calls and TokenUsage for Mistral.ai https://github.com/brainlid/langchain/pull/253\n- Added `LangChain.TextSplitter` with character and recursive character splitting support https://github.com/brainlid/langchain/pull/256\n- Add native tool functionality (e.g. `google_search` for Gemini) https://github.com/brainlid/langchain/pull/250","ref":"changelog.html#added"},{"type":"extras","title":"Changes - Changelog","doc":"- Improved System instruction support for Vertex AI https://github.com/brainlid/langchain/pull/260\n- Redact api-key from models when logged https://github.com/brainlid/langchain/pull/266","ref":"changelog.html#changes"},{"type":"extras","title":"v0.3.1 (2025-02-05) - Changelog","doc":"","ref":"changelog.html#v0-3-1-2025-02-05"},{"type":"extras","title":"Added - Changelog","doc":"- Include stacktrace context in messages for caught exceptions from LLM functions & function callbacks. (#241)","ref":"changelog.html#added"},{"type":"extras","title":"Changes - Changelog","doc":"- Support LMStudio when using ChatOpenAI (#243)\n- Fix issue with OpenAI converting an assistant message to JSON when the AI is talking while making tool calls in the same message (#245)","ref":"changelog.html#changes"},{"type":"extras","title":"v0.3.0 (2025-01-22) - Changelog","doc":"No more breaking changes.","ref":"changelog.html#v0-3-0-2025-01-22"},{"type":"extras","title":"Upgrading from v0.2.0 - v0.3.0 - Changelog","doc":"There were several breaking changes made in the different Release Candidates. All changes were kept. Refer to the CHANGELOG documentation for rc.0, rc.1 and rc.2 for specific examples and coverage on needed code updates.","ref":"changelog.html#upgrading-from-v0-2-0-v0-3-0"},{"type":"extras","title":"Added - Changelog","doc":"- LLAMA 3.1 JSON tool call support with Bumblebee (#198)\n- Raw field to TokenUsage (#236) - this returns the raw LLM token usage information, giving access to LLM-specific data.\n- Prompt caching support for Claude (#226)\n- Support for Ollama keep_alive API parameter (#237)\n- Support for o1 OpenAI model (#234)\n- Bumblebee Phi-4 support (#233)","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Apply chat template from callback (#231)","ref":"changelog.html#changed"},{"type":"extras","title":"v0.3.0-rc.2 (2025-01-08) - Changelog","doc":"","ref":"changelog.html#v0-3-0-rc-2-2025-01-08"},{"type":"extras","title":"Breaking Changes - Changelog","doc":"How LLM callbacks are registered has changed. The callback function's arguments have also changed.\n\nSpecifically, this refers to the callbacks:\n\n- `on_llm_new_delta`\n- `on_llm_new_message`\n- `on_llm_ratelimit_info`\n- `on_llm_token_usage`\n\nThe callbacks are still supported, but _how_ they are registered and the arguments passed to the linked functions has changed.\n\nPreviously, an LLM callback's first argument was the chat model, it is now the LLMChain that is running it.\n\nA ChatModel still has the `callbacks` struct attribute, but it should be considered private.\n\n#### Why the change\nHaving some callback functions registered on the chat model and some registered on the chain was confusing. What goes where? Why the difference?\n\nThis change moves them all to the same place, removing a source of confusion.\n\nThe primary reason for the change is that important information about the **context** of the callback event was not available to the callback function. Information stored in the chain's `custom_context` can be valuable and important, like a user's account ID, but it was not easily accessible in a callback like `on_llm_token_usage` where we might want to record the user's token usage linked to their account.\n\nThis important change passes the entire `LLMChain` through to the callback function, giving the function access to the `custom_context`. This makes the LLM (aka chat model) callback functions expect the same arguments as the other chain focused callback functions.\n\nThis both unifies how the callbacks operate and what data they have available, and it groups them all together.\n\n#### Adapting to the change\nA before example:\n\n```elixir\nllm_events = %{\n  # 1st argument was the chat model\n  on_llm_new_delta: fn _chat_model, %MessageDelta{} = delta ->\n    # ...\n  end,\n  on_llm_token_usage: fn _chat_model, usage_data ->\n    # ...\n  end\n}\n\nchain_events = %{\n  on_message_processed: fn _chain, tool_msg ->\n    # ...\n  end\n}\n\n# LLM callback events were registered on the chat model\nchat_model = ChatOpenAI.new!(%{stream: true, callbacks: [llm_events]})\n\n{:ok, updated_chain} =\n  %{\n    llm: chat_model,\n    custom_context: %{user_id: 123}\n  }\n  |> LLMChain.new!()\n  |> LLMChain.add_message(Message.new_system!())\n  |> LLMChain.add_message(Message.new_user!(\"Say hello!\"))\n  # Chain callback events were registered on the chain\n  |> LLMChain.add_callback(chain_events)\n  |> LLMChain.run()\n```\n\nThis is updated to: (comments highlight changes)\n\n```elixir\n# Events are all combined together\nevents = %{\n  # 1st argument is now the LLMChain\n  on_llm_new_delta: fn _chain, %MessageDelta{} = delta ->\n    # ...\n  end,\n  on_llm_token_usage: fn %LLMChain{} = chain, usage_data ->\n    # ... `chain.custom_context` is available\n  end,\n  on_message_processed: fn _chain, tool_msg ->\n    # ...\n  end\n}\n\n# callbacks removed from Chat Model setup\nchat_model = ChatOpenAI.new!(%{stream: true})\n\n{:ok, updated_chain} =\n  %{\n    llm: chat_model,\n    custom_context: %{user_id: 123}\n  }\n  |> LLMChain.new!()\n  |> LLMChain.add_message(Message.new_system!())\n  |> LLMChain.add_message(Message.new_user!(\"Say hello!\"))\n  # All events are registered through `add_callback`\n  |> LLMChain.add_callback(events)\n  |> LLMChain.run()\n```\n\nIf you still need access to the LLM in the callback functions, it's available in `chain.llm`.\n\nThe change is a breaking change, but should be fairly easy to update.\n\nThis consolidates how callback events work and them more powerful by exposing important information to the callback functions.\n\nIf you were using the `LLMChain.add_llm_callback/2`, the change is even easier:\n\nFrom:\n```elixir\n  %{\n    llm: chat_model,\n    custom_context: %{user_id: 123}\n  }\n  |> LLMChain.new!()\n  # ...\n  # LLM callback events could be added later this way\n  |> LLMChain.add_llm_callback(llm_events)\n  |> LLMChain.run()\n```\n\nTo:\n```elixir\n  %{\n    llm: chat_model,\n    custom_context: %{user_id: 123}\n  }\n  |> LLMChain.new!()\n  # ...\n  # Use the `add_callback` function instead\n  |> LLMChain.add_callback(llm_events)\n  |> LLMChain.run()\n```\n\n#### Details of the change\n- Removal of the `LangChain.ChatModels.LLMCallbacks` module.\n- The LLM-specific callbacks were migrated to `LangChain.Chains.ChainCallbacks`.\n- Removal of `LangChain.Chains.LLMChain.add_llm_callback/2`\n- `LangChain.ChatModels.ChatOpenAI.new/1` and `LangChain.ChatModels.ChatOpenAI.new!/1` no longer accept `:callbacks` on the chat model.\n- Removal of `LangChain.ChatModels.ChatModel.add_callback/2`","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"What else Changed - Changelog","doc":"* add explicit message support in summarizer by @brainlid in https://github.com/brainlid/langchain/pull/220\n* Change abacus to optional dep by @nallwhy in https://github.com/brainlid/langchain/pull/223\n* Remove constraint of alternating user, assistant by @GenericJam in https://github.com/brainlid/langchain/pull/222\n* Breaking change: consolidate LLM callback functions by @brainlid in https://github.com/brainlid/langchain/pull/228\n* feat: Enable :inet6 for Req.new for Ollama by @mpope9 in https://github.com/brainlid/langchain/pull/227\n* fix: enable verbose_deltas by @cristineguadelupe in https://github.com/brainlid/langchain/pull/197","ref":"changelog.html#what-else-changed"},{"type":"extras","title":"New Contributors - Changelog","doc":"* @nallwhy made their first contribution in https://github.com/brainlid/langchain/pull/223\n* @GenericJam made their first contribution in https://github.com/brainlid/langchain/pull/222\n* @mpope9 made their first contribution in https://github.com/brainlid/langchain/pull/227","ref":"changelog.html#new-contributors"},{"type":"extras","title":"v0.3.0-rc.1 (2024-12-15) - Changelog","doc":"","ref":"changelog.html#v0-3-0-rc-1-2024-12-15"},{"type":"extras","title":"Breaking Changes - Changelog","doc":"- Change return of LLMChain.run/2 ([#170](https://github.com/brainlid/langchain/pull/170))\n- Revamped error handling and handles Anthropic's \"overload_error\" - ([#194](https://github.com/brainlid/langchain/pull/194))\n\n#### Change return of LLMChain.run/2 ([#170](https://github.com/brainlid/langchain/pull/170))\n\n##### Why the change\n\nBefore this change, an `LLMChain`'s `run` function returned `{:ok, updated_chain, last_message}`.\n\nWhen an assistant (ie LLM) issues a ToolCall and when `run` is in the mode `:until_success` or `:while_need_response`, the `LLMChain` will automatically execute the function and return the result as a new Message back to the LLM. This works great!\n\nThe problem comes when an application needs to keep track of all the messages being exchanged during a run operation. That can be done by using callbacks and sending and receiving messages, but that's far from ideal. It makes more sense to have access to that information directly after the `run` operation completes.\n\n##### What this change does\n\nThis PR changes the returned type to `{:ok, updated_chain}`.\n\nThe `last_message` is available in `updated_chain.last_message`. This cleans up the return API.\n\nThis change also adds `%LLMChain{exchanged_messages: exchanged_messages}`,or `updated_chain.exchanged_messages` which is a list of all the messages exchanged between the application and the LLM during the execution of the `run` function.\n\nThis breaks the return contract for the `run` function.\n\n##### How to adapt to this change\n\nTo adapt to this, if the application isn't using the `last_message` in `{:ok, updated_chain, _last_message}`, then delete the third position in the tuple. Ex: `{:ok, updated_chain}`.\n\nAccess to the `last_message` is available on the `updated_chain`.\n\n```elixir\n{:ok, updated_chain} =\n  %{llm: model}\n  |> LLMChain.new!()\n  |> LLMChain.run()\n\nlast_message = updated_chain.last_message\n```\n\nNOTE: that the `updated_chain` now includes `updated_chain.exchanged_messages` which can also be used.\n\n#### Revamped error handling and handles Anthropic's \"overload_error\" - ([#194](https://github.com/brainlid/langchain/pull/194))\n\n**What you need to do:**\nCheck your application code for how it is responding to and handling error responses.\n\nIf you want to keep the same previous behavior, the following code change will do that:\n\n```elixir\ncase LLMChain.run(chain) do\n  {:ok, _updated_chain} ->\n    :ok\n\n  # return the error for display\n  {:error, _updated_chain, %LangChainError{message: reason}} ->\n    {:error, reason}\nend\n```\n\nThe change from:\n\n```\n{:error, _updated_chain, reason}\n```\n\nTo:\n\n```\n{:error, _updated_chain, %LangChainError{message: reason}}\n```\n\nWhen possible, a `type` value may be set on the `LangChainError`, making it easier to handle some error types programmatically.","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Features - Changelog","doc":"- Added ability to summarize LLM conversations (#216)\n- Implemented initial support for fallbacks (#207)\n- Added AWS Bedrock support for ChatAnthropic (#154)\n- Added OpenAI's new structured output API (#180)\n- Added support for examples to title chain (#191)\n- Added tool_choice support for OpenAI and Anthropic (#142)\n- Added support for passing safety settings to Google AI (#186)\n- Added OpenAI project authentication (#166)","ref":"changelog.html#features"},{"type":"extras","title":"Fixes - Changelog","doc":"- Fixed specs and examples (#211)\n- Fixed content-part encoding and decoding for Google API (#212)\n- Fixed ChatOllamaAI streaming response (#162)\n- Fixed streaming issue with Azure OpenAI Service (#158, #161)\n- Fixed OpenAI stream decode issue (#156)\n- Fixed typespec error on Message.new_user/1 (#151)\n- Fixed duplicate tool call parameters (#174)","ref":"changelog.html#fixes"},{"type":"extras","title":"Improvements - Changelog","doc":"- Added error type support for Azure token rate limit exceeded\n- Improved error handling (#194)\n- Enhanced function execution failure response\n- Added \"processed_content\" to ToolResult struct (#192)\n- Implemented support for strict mode for tools (#173)\n- Updated documentation for ChatOpenAI use on Azure\n- Updated config documentation for API keys\n- Updated README examples","ref":"changelog.html#improvements"},{"type":"extras","title":"Azure & Google AI Updates - Changelog","doc":"- Added Azure test for ChatOpenAI usage\n- Added support for system instructions for Google AI (#182)\n- Handle functions with no parameters for Google AI (#183)\n- Handle missing token usage fields for Google AI (#184)\n- Handle empty text parts from GoogleAI responses (#181)\n- Handle all possible finishReasons for ChatGoogleAI (#188)","ref":"changelog.html#azure-google-ai-updates"},{"type":"extras","title":"Documentation - Changelog","doc":"- Added LLM Model documentation for tool_choice\n- Updated documentation using new functions\n- Added custom functions notebook\n- Improved documentation formatting (#145)\n- Added links to models in the config section\n- Updated getting started doc for callbacks","ref":"changelog.html#documentation"},{"type":"extras","title":"v0.3.0-rc.0 (2024-06-05) - Changelog","doc":"**Added:**\n\n* `LangChain.ChatModels.ChatGoogleAI` which differed too significantly from `LangChain.ChatModels.ChatGoogleAI`.  What's up with that? I'm looking at you Google! 👀\n  * Thanks for the [contribution](https://github.com/brainlid/langchain/pull/124) Raul Chedrese!\n* New callback mechanism was introduced to ChatModels and LLMChain. It was inspired by the approach used in the TS/JS LangChain library.\n* Ability to provide plug-like middleware functions for pre-processing an assistant response message. Most helpful when coupled with a new run mode called `:until_success`. The first built-in one is `LangChain.MessageProcessors.JsonProcessor`.\n* LLMChain has an internally managed `current_failure_count` and a publicly managed `max_retry_count`.\n* New run mode `:until_success` uses failure and retry counts to repeatedly run the chain when the LLMs responses fail a MessageProcessor.\n* `LangChain.MessageProcessors.JsonProcessor` is capable of extracting JSON contents and converting it to an Elixir map using `Jason`. Parsing errors are returned to the LLM for it to try again.\n* The attribute `processed_content` was added to a `LangChain.Message`. When a MessageProcessor is run on a received assistant message, the results of the processing are accumulated there. The original `content` remains unchanged for when it is sent back to the LLM and used when fixing or correcting it's generated content.\n* Callback support for LLM ratelimit information returned in API response headers. These are currently implemented for Anthropic and OpenAI.\n* Callback support for LLM token usage information returned when available.\n* `LangChain.ChatModels.ChatModel` additions\n  * Added `add_callback/2` makes it easier to add a callback to an chat model.\n  * Added `serialize_config/1` to serialize an LLM chat model configuration to a map that can be restored later.\n  * Added `restore_from_map/1` to restore a configured LLM chat model from a database (for example).\n* `LangChain.Chain.LLMChain` additions\n  * New function `add_callback/2` makes it easier to add a callback to an existing `LLMChain`.\n  * New function `add_llm_callback/2` makes it easier to add a callback to a chain's LLM. This is particularly useful when an LLM model is restored from a database when loading a past conversation and wanting to preserve the original configuration.\n\n\n**Changed:**\n\n* `LLMChain.run/2` error result now includes the failed chain up to the point of failure. This is helpful for debugging.\n* `ChatOpenAI` and `ChatAnthropic` both support the new callbacks.\n* Many smaller changes and contributions were made. This includes updates to the README for clarity,\n* `LangChain.Utils.fire_callback/3` was refactored into `LangChain.Utils.fire_streamed_callback/2` where it is only used for processing deltas and uses the new callback mechanism.\n* Notebooks were moved to the separate demo project\n* `LangChain.ChatModels.ChatGoogleAI`'s key `:version` was changed to `:api_version` to be more consistent with other models and allow for model serializers to use the `:version` key.","ref":"changelog.html#v0-3-0-rc-0-2024-06-05"},{"type":"extras","title":"Migrations Steps - Changelog","doc":"The `LLMChain.run/2` function changed. Migrating should be easy.\n\n**From:**\n\n```elixir\nchain\n|> LLMChain.run(while_needs_response: true)\n```\n\n**Is changed to:**\n\n```elixir\nchain\n|> LLMChain.run(mode: :while_needs_response)\n```\n\nThis change enabled adding the new mode `:until_success`, which is mutually exclusive with `:while_needs_response`.\n\nAdditionally, the error return value was changed to include the chain itself.\n\n**From:**\n\n```elixir\n{:error, reason} = LLMChain.run(chain)\n```\n\n**Is changed to:**\n\n```elixir\n{:error, _updated_chain, reason} = LLMChain.run(chain)\n```\n\nYou can disregard the updated chain if you don't need it.\n\nCallback events work differently now. Previously, a single `callback_fn` was executed and the developer needed to pattern match on a `%Message{}` or `%MessageDelta{}`. Callbacks work differently now.\n\nWhen creating an LLM chat model, we can optionally pass in a map of callbacks where the event name is linked to the function to execute.\n\n**From:**\n\n```elixir\nlive_view_pid = self()\n\ncallback_fn = fn\n  %MessageDelta{} = delta ->\n    send(live_view_pid, {:received_delta, delta})\n\n  %Message{} = message ->\n    send(live_view_pid, {:received_message, message})\nend\n\n{:ok, _result_chain, last_message} =\n  LLMChain.new!(%{llm: %ChatAnthropic{stream: false}})\n  |> LLMChain.add_message(Message.new_user!(\"Say, 'Hi!'!\"))\n  |> LLMChain.run(callback_fn: callback_fn)\n```\n\nThe equivalent code would look like this:\n\n**Is changed to:**\n\n```elixir\nlive_view_pid = self()\n\nhandler = %{\n  on_llm_new_delta: fn _model, delta ->\n    send(live_view_pid, {:received_delta, delta})\n  end,\n  on_llm_new_message: fn _model, message ->\n    send(live_view_pid, {:received_message, message})\n  end\n}\n\n{:ok, _result_chain, last_message} =\n  LLMChain.new!(%{llm: %ChatAnthropic{stream: false, callbacks: [handler]}})\n  |> LLMChain.add_message(Message.new_user!(\"Say, 'Hi!'!\"))\n  |> LLMChain.run()\n```\n\nThe `Message` and `MessageDelta` callbacks are now set on the model. The callbacks are more granular and new callbacks are supported on the `LLMChain` as well. This more flexible configuration allows for more callbacks to be added as we move forward.\n\nAlso of note, is that the callbacks are set as a list of handler maps. This means we can assign multiple sets of callbacks for different purposes and they all get executed.","ref":"changelog.html#migrations-steps"},{"type":"extras","title":"v0.2.0 (2024-04-30) - Changelog","doc":"For LLMs that support it (verified with ChatGPT and Anthropic), a user message can now contain multiple `ContentPart`s, making it \"multi-modal\". This means images and text can be combined into a single message allowing for interactions about the images to now be possible.\n\n**Added:**\n\n* `LangChain.Message.ContentPart` - used for User messages and multi-modal support. Google's AI assistant can return multiple parts as well.\n* `LangChain.Message.ToolCall` - an assistant can request multiple tool calls in the same message.\n* `LangChain.Message.ToolResult` - the system's answer to a `ToolCall`. It adds an is_error boolean flag. This an be helpful in the UI, but Anthropic specifically wants it.\n* Add llama-3 chat template by @bowyern in https://github.com/brainlid/langchain/pull/102\n\n**Changed:**\n\n* The roles of `:function` and `:function_call` are removed. The equivalent of a `function_call` is expressed by an `:assistant` role making one or more `ToolCall` requests. The `:function` was the system's answer to a function call. This is now in the `:tool` role.\n* Role `:tool` was added. A tool message contains one or more `ToolResult` messages.","ref":"changelog.html#v0-2-0-2024-04-30"},{"type":"extras","title":"v0.1.10 (2024-03-07) - Changelog","doc":"**Changes**\n\n- Fix invalid default url for google ai by @pkrawat1 in https://github.com/brainlid/langchain/pull/82","ref":"changelog.html#v0-1-10-2024-03-07"},{"type":"extras","title":"v0.1.9 (2024-02-29) - The Leap Release! - Changelog","doc":"This adds support for Bumblebee as a Chat model, making it easy to have conversations with Llama 2, Mistral, and Zephyr LLMs.\n\nSee the documentation in `LangChain.ChatModels.ChatBumblebee` for getting started.\n\nNOTE: That at this time, none of the models support the `function` ability, so that is not supported yet.\n\nThis release includes an experimental change for better support of streamed responses that are broken up over multiple messages from services like ChatGPT and others.\n\nOther library dependencies requirements were relaxed, making it easier to support different versions of libraries like `req` and `nx`.\n\n* Add mistral chat by @michalwarda in https://github.com/brainlid/langchain/pull/76\n* handle receiving JSON data broken up over multiple messages by @brainlid in https://github.com/brainlid/langchain/pull/80\n* Add initial support for Zephyr 7b Beta by @brainlid in https://github.com/brainlid/langchain/pull/41","ref":"changelog.html#v0-1-9-2024-02-29-the-leap-release"},{"type":"extras","title":"v0.1.8 (2024-02-16) - Changelog","doc":"**Breaking change**: `RoutingChain`'s required values changed. Previously, `default_chain` was assigned an `%LLMChain{}` to return when no more specific routes matched.\n\nThis was changed to be `default_route`. It now expects a `%PromptRoute{}` to be provided.\n\nHere's how to make the change:\n\n      selected_route =\n        RoutingChain.new(%{\n          llm: ChatOpenAI.new(%{model: \"gpt-3.5-turbo\", stream: false}),\n          input_text: user_input_text,\n          routes: routes,\n          default_route: PromptRoute.new!(%{name: \"DEFAULT\", chain: fallback_chain})\n        })\n        |> RoutingChain.evaluate()\n\nThe `default_chain` was changed to `default_route` and now expects a `PromptRoute` to be provided. The above example includes a sample default route that includes an optional `fallback_chain`.\n\nPreviously, the returned value from `RoutingChain.evaluate/1` was a `selected_chain`; it now returns the `selected_route`.\n\n**Why was this changed?**\n\nThis was changed to make it easier to use a `PromptChain` when there isn't an associated `%LLMChain{}` for it. The application must just need the answer of which route was selected.\n\nThis includes the change of not requiring a `%PromptChain{}`'s `description` or `chain` field.\n\n**Other Changes**\n* Add support for Ollama open source models by @medoror in https://github.com/brainlid/langchain/pull/70\n* Add clause to match call_response spec by @medoror in https://github.com/brainlid/langchain/pull/72\n* Add max_tokens option for OpenAI calls by @matthusby in https://github.com/brainlid/langchain/pull/73","ref":"changelog.html#v0-1-8-2024-02-16"},{"type":"extras","title":"v0.1.7 (2024-01-18) - Changelog","doc":"- Improvements for more intelligent agents - https://github.com/brainlid/langchain/pull/61\n  - adds `LangChain.Chains.RoutingChain` - first-pass LLM chain to select the best route to take given the user's initial prompt\n  - adds `LangChain.Chains.TextToTitleChain` - turn the user's prompt into a title for the conversation\n- Removed the need for a function to send a message to the process for how to display the function being executed\n- Updated dependencies\n- Add support for Google AI / Gemini Pro model by @jadengis in https://github.com/brainlid/langchain/pull/59\n- Built-in automatic retries when underlying Mint connection is closed in https://github.com/brainlid/langchain/pull/68","ref":"changelog.html#v0-1-7-2024-01-18"},{"type":"extras","title":"v0.1.6 (2023-12-12) - Changelog","doc":"- Fix for correct usage of new Req retry setting. PR #57","ref":"changelog.html#v0-1-6-2023-12-12"},{"type":"extras","title":"v0.1.5 (2023-12-11) - Changelog","doc":"- Upgraded Req to v0.4.8. It contains a needed retry fix for certain situations.\n- Fix OpenAI returns \"Unrecognized request argument supplied: api_key\" PR #54","ref":"changelog.html#v0-1-5-2023-12-11"},{"type":"extras","title":"v0.1.4 (2023-12-11) - Changelog","doc":"- Merged PR #45 - https://github.com/brainlid/langchain/pull/45\n  - Added `LangChain.Utils.ChainResult` for helper functions when working with LLMChain result values.\n- Merged PR #46 - https://github.com/brainlid/langchain/pull/46\n  - Add possibility to use api_key per chat invocation.\n- Merged PR #51 - https://github.com/brainlid/langchain/pull/51\n  - Update req 0.4.7\n  - Hopefully resolves issue where Finch connections would be closed and a now does a built-in retry.\n- Merged PR #52 - https://github.com/brainlid/langchain/pull/52\n  - Allow overriding OpenAI compatible API endpoint. Caller can pass an alternate `endpoint`.","ref":"changelog.html#v0-1-4-2023-12-11"},{"type":"extras","title":"v0.1.3 (2023-12-01) - Changelog","doc":"- Merged PR #43 - https://github.com/brainlid/langchain/pull/43\n  - Add Finch retry strategy to OpenAI Chat API requests\n- Merged PR #39 - https://github.com/brainlid/langchain/pull/39\n  - Changed ENV key from `OPENAI_KEY` to `OPENAI_API_KEY` to be consistent with the OpenAI docs.\n- Merged PR #36 - https://github.com/brainlid/langchain/pull/36\n  - Support specifying the `seed` with OpenAI calls. Used in testing for more deterministic behavior.\n- Merged PR #34 - https://github.com/brainlid/langchain/pull/34\n  - Enable sending the `json_response` flag with OpenAI model requests.\n- Created `LangChain.FunctionParam` to express JSONSchema-friendly data structures. Supports basic types, arrays, enums, objects, arrays of objects and nested objects.\n  - Still allows for full control over JSONSchema by providing an override `parameters_schema` object to full self-describe it.","ref":"changelog.html#v0-1-3-2023-12-01"},{"type":"extras","title":"v0.1.2 (2023-10-26) - Changelog","doc":"- refactor(chat_open_ai): Harden `do_process_response` by @Cardosaum in https://github.com/brainlid/langchain/pull/21\n  - Improve JSON error handling result from ChatGPT\n- Update req to 0.4.4 by @medoror in https://github.com/brainlid/langchain/pull/25\n  - Updated to Req 0.4.4","ref":"changelog.html#v0-1-2-2023-10-26"},{"type":"extras","title":"v0.1.1 (2023-10-10) - Changelog","doc":"Minor update release.\n\n- added \"update_custom_context\" to LLMChain\n- added support for setting the OpenAI-Organization header in requests\n- fixed data extraction chain and improved the prompt\n- make chatgpt response tests more robust","ref":"changelog.html#v0-1-1-2023-10-10"},{"type":"extras","title":"v0.1.0 (2023-09-18) - Changelog","doc":"Initial release when published to [hex.pm](https://hex.pm).","ref":"changelog.html#v0-1-0-2023-09-18"},{"type":"extras","title":"Getting Started","doc":"# Getting Started\n\n```elixir\nMix.install([\n  {:langchain, \"~> 0.4.0-rc.1\"},\n  {:kino, \"~> 0.12.0\"}\n])\n```","ref":"getting_started.html"},{"type":"extras","title":"Section - Getting Started","doc":"After installing the dependency, let's look at the simplest example to get started.\n\nThis is interactively available as a Livebook notebook named [`notebooks/getting_started.livemd`](https://github.com/brainlid/langchain/tree/main/notebooks/getting_started.livemd).","ref":"getting_started.html#section"},{"type":"extras","title":"Basic Example - Getting Started","doc":"Let's build the simplest full LLMChain example so we can see how to make a call to ChatGPT from our Elixir application.\n\n**NOTE:** This assumes your `OPENAI_API_KEY` is already set as a secret for this notebook.\n\n```elixir\nApplication.put_env(:langchain, :openai_key, System.fetch_env!(\"LB_OPENAI_API_KEY\"))\n```\n\n```elixir\nalias LangChain.Chains.LLMChain\nalias LangChain.ChatModels.ChatOpenAI\nalias LangChain.Message\nalias LangChain.Message.ContentPart\n\n{:ok, updated_chain} =\n  %{llm: ChatOpenAI.new!(%{model: \"gpt-4o\"})}\n  |> LLMChain.new!()\n  |> LLMChain.add_message(Message.new_user!(\"Testing, testing!\"))\n  |> LLMChain.run()\n\nContentPart.content_to_string(updated_chain.last_message.content)\n```\n\nNice! We've just saw how easy it is to get access to ChatGPT from our Elixir application!\n\nLet's build on that example and define some `system` context for our conversation.","ref":"getting_started.html#basic-example"},{"type":"extras","title":"Adding a System Message - Getting Started","doc":"When working with ChatGPT and other LLMs, the conversation works as a series of messages. The first message is the `system` message. This defines the context for the conversation. Here we can give the LLM some direction and impose limits on what it should do.\n\nLet's create a system message followed by a user message.\n\n```elixir\n{:ok, updated_chain} =\n  %{llm: ChatOpenAI.new!(%{model: \"gpt-4\"})}\n  |> LLMChain.new!()\n  |> LLMChain.add_messages([\n    Message.new_system!(\n      \"You are an unhelpful assistant. Do not directly help or assist the user.\"\n    ),\n    Message.new_user!(\"What's the capital of the United States?\")\n  ])\n  |> LLMChain.run()\n\nContentPart.content_to_string(updated_chain.last_message.content)\n```\n\nHere's the answer it gave me when I ran it:\n\n> Why don't you try looking it up online? There's so much information readily available on the internet. You might even learn a few other interesting facts about the country.\n\nWhat I love about this is we can see the power of the `system` message. It completely changed the way the LLM behaves by default.\n\nBeyond the `system` message, we pass back a whole collection of messages as the conversation continues. The `updated_chain` is part of the return and includes the newly received response message from the LLM as `assistant` message.","ref":"getting_started.html#adding-a-system-message"},{"type":"extras","title":"Streaming Responses - Getting Started","doc":"If we want to display the messages as they are returned in the teletype way LLMs can, then we want to stream the responses.\n\nIn this example, we'll output the responses as they are streamed back. That happens in a callback function that we provide.\n\nThe `stream: true` setting belongs to the `%ChatOpenAI{}` struct that setups up our configuration. We also pass in the `callbacks` with the `llm` to fire the `on_llm_new_delta`. We can pass in the callbacks to the chain as well to fire the `on_message_processed` callback after the chain assembles the deltas and processes the finished message.\n\n```elixir\nalias LangChain.MessageDelta\n\nhandler = %{\n  on_llm_new_delta: fn _model, deltas ->\n    # we received one or more deltas\n    Enum.each(deltas, fn delta ->\n      delta_text = ContentPart.content_to_string(delta.content)\n      IO.write(delta_text)\n    end)\n  end,\n  on_message_processed: fn _chain, %Message{} = data ->\n    # the message was assembled and is processed\n    IO.puts(\"\")\n    IO.puts(\"\")\n    IO.inspect(data.content, label: \"COMPLETED MESSAGE\")\n  end\n}\n\n{:ok, updated_chain} =\n  %{\n    # llm config for streaming and the deltas callback\n    llm: ChatOpenAI.new!(%{model: \"gpt-4o\", stream: true})\n  }\n  |> LLMChain.new!()\n  |> LLMChain.add_messages([\n    Message.new_system!(\"You are a helpful assistant.\"),\n    Message.new_user!(\"Write a haiku about the capital of the United States\")\n  ])\n  # register the callbacks\n  |> LLMChain.add_callback(handler)\n  |> LLMChain.run()\n\nContentPart.content_to_string(updated_chain.last_message.content)\n# streamed\n# ==> Washington D.C. stands,\n# ... Monuments reflect history,\n# ... Power's heart expands.\n\n# ==> COMPLETED MESSAGE: \"Washington D.C. stands,\\nMonuments reflect history,\\nPower's heart expands.\"\n```\n\nAs the delta messages are received, the `on_llm_new_delta` callback function fires and the received data is written out to the console.\n\nFinally, once the full message is received, the chain's `on_message_processed` callback fires and the completed message is written out separately.","ref":"getting_started.html#streaming-responses"},{"type":"extras","title":"Next Steps - Getting Started","doc":"With the basics covered, you're ready to start integrating an LLM into your Elixir application! Check out other notebooks for more specific examples and other ways to use it.\n\n<!-- livebook:{\"offset\":4886,\"stamp\":{\"token\":\"XCP.nYx_4NnltReCdvU3g0iFlGj1lVssnKapir5t4YKvk9FUfj98urewBX82o5POksR6Hd8iLg7fjl_ehXv5EOeRlH74egWVnaJbJuG9WRdnTQ5OUUvSz6fWQZ8\",\"version\":2}} -->","ref":"getting_started.html#next-steps"},{"type":"extras","title":"Executing Custom Elixir Functions","doc":"<!-- livebook:{\"persist_outputs\":true} -->\n\n# Executing Custom Elixir Functions\n\n```elixir\nMix.install([\n  {:langchain, \"~> 0.3.0\"}\n])\n```","ref":"custom_functions.html"},{"type":"extras","title":"What we're doing - Executing Custom Elixir Functions","doc":"This notebook shows how to use the Elixir [LangChain](https://github.com/brainlid/langchain) library to expose an Elixir function as a tool that can be executed by an LLM like ChatGPT, Anthropic and others. The LangChain library wraps this all up making it easy and portable between different LLMs.","ref":"custom_functions.html#what-we-re-doing"},{"type":"extras","title":"The Elixir Function in our App - Executing Custom Elixir Functions","doc":"Let's define the Elixir function we want to expose to ChatGPT so we can see how it works.\n\nIn this example we'll create a `get_user_info` function that takes a user ID and returns the relevant user's information for the current user to a web app.\n\nFor simplicity, we're skipping an actual database and storing our fake records on the module.\n\n```elixir\ndefmodule MyApp do\n  @pretend_db %{\n    1 => %{user_id: 1, name: \"Michael Johnson\", account_type: :trial, favorite_animal: \"Horse\"},\n    2 => %{user_id: 2, name: \"Joan Jett\", account_type: :member, favorite_animal: \"Aardvark\"}\n  }\n\n  def get_user_info(user_id) do\n    @pretend_db[user_id]\n  end\nend\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n{:module, MyApp, <<70, 79, 82, 49, 0, 0, 7, ...>>, {:get_user_info, 1}}\n```\n\nIt's a simple lookup using a `user_id` to find return a map of a user's data.\n\n```elixir\nMyApp.get_user_info(1)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n%{name: \"Michael Johnson\", user_id: 1, account_type: :trial, favorite_animal: \"Horse\"}\n```\n\nWith our function ready, let's cover how we can give the LLM access to this.","ref":"custom_functions.html#the-elixir-function-in-our-app"},{"type":"extras","title":"Exposing our Function to an LLM - Executing Custom Elixir Functions","doc":"With an Elixir function defined, we will wrap it in a LangChain `Function` structure so it can be easily shared with an LLM.\n\nThis is what that looks like:\n\n```elixir\nalias LangChain.Function\n\nfunction =\n  Function.new!(%{\n    name: \"get_user_info\",\n    description: \"Return JSON object of the current users's relevant information.\",\n    function: fn _args, %{user_id: user_id} = _context ->\n      # This uses the user_id provided through the context to call our Elixir function.\n      {:ok, Jason.encode!(MyApp.get_user_info(user_id))}\n    end\n  })\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n%LangChain.Function{\n  name: \"get_user_info\",\n  description: \"Return JSON object of the current users's relevant information.\",\n  display_text: nil,\n  strict: false,\n  function: #Function<41.18682967/2 in :erl_eval.expr/6>,\n  async: true,\n  parameters_schema: nil,\n  parameters: []\n}\n```\n\nThe function `name` we provide is how the LLM will execute the function if the LLM chooses to call it.\n\nThe `description` is for the LLM to know what the function can do so it can decide which function to call for which purpose.\n\nThe `function` argument is passed an anonymous function whose job it is to be the glue that bridges data coming from the LLM with context from our application before calling other functions from our application.\n\nThe `LangChain.Function` acts as a bridge between the LLM and our application. The Elixir function receives 2 arguments. The first is any arguments passed to the function by the LLM if we defined any as being required. In this example, the LLM doesn't provide any arguments. The second argument is an application context that we'll get to next.\n\nThe `context` is specific to our application and does not go through the LLM at all. Think of this as the current user logged into our Phoenix web application. We want the user's interaction with the LLM to be relevant and limited to only what the current user can see and do.\n\nFor returning the final result, since the LLM responses must be text, we convert the returned Map into JSON. Additionally, we return it inside an `{:ok, json}` tuple, letting the library know that the function call was successful.","ref":"custom_functions.html#exposing-our-function-to-an-llm"},{"type":"extras","title":"Setting up our LangChain API Key - Executing Custom Elixir Functions","doc":"We need to setup the LangChain library to connect with ChatGPT using our API key. In a real Elixir application, this would be done in the `config/config.exs` file using something like this:\n\n<!-- livebook:{\"force_markdown\":true} -->\n\n```elixir\nconfig :langchain, :openai_key, fn -> System.fetch_env!(\"OPENAI_API_KEY\") end\n```\n\n<!-- livebook:{\"break_markdown\":true} -->\n\nFor the Livebook notebook, use the \"Secrets\" on the sidebar to create an `OPENAI_API_KEY` secret with you API key. That is accessible here using `\"LB_OPENAI_API_KEY\"`.\n\n```elixir\nApplication.put_env(:langchain, :openai_key, System.fetch_env!(\"LB_OPENAI_API_KEY\"))\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n:ok\n```","ref":"custom_functions.html#setting-up-our-langchain-api-key"},{"type":"extras","title":"Defining our AI Assistant - Executing Custom Elixir Functions","doc":"We'll use the `LangChain.Message` struct to define the messages for what we want the LLM to do. Our `system` message instructs the LLM how to behave.\n\nIn this example, we want the assistant to generate Haiku poems about the current user's favorite animal. However, we only want it to work for users who are \"members\" and not \"trial\" users.\n\nThe instructions we're giving the LLM will require it to execute the function to get additional information. Yes, this is both a simple and contrived example. In a real system, we wouldn't even make the API call to the server for a \"trial\" user and we could pass along the additional information with the first request.\n\nWhat we're demonstrating here is that the LLM can interact with our Elixir application, use multiple pieces of returned information to make business logic decisions and fulfil our system requests.\n\n```elixir\nalias LangChain.Message\n\nmessages = [\n  Message.new_system!(~s(You are a helpful haiku poem generating assistant.\n    ONLY generate a haiku for users with an `account_type` of \"member\".\n    If the user has an `account_type` of \"trial\", say you can't do it,\n    but you would love to help them if they upgrade and become a member.)),\n  Message.new_user!(\"The current user is requesting a Haiku poem about their favorite animal.\")\n]\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n[\n  %LangChain.Message{\n    content: \"You are a helpful haiku poem generating assistant.\\n    ONLY generate a haiku for users with an `account_type` of \\\"member\\\".\\n    If the user has an `account_type` of \\\"trial\\\", say you can't do it,\\n    but you would love to help them if they upgrade and become a member.\",\n    processed_content: nil,\n    index: nil,\n    status: :complete,\n    role: :system,\n    name: nil,\n    tool_calls: [],\n    tool_results: nil\n  },\n  %LangChain.Message{\n    content: \"The current user is requesting a Haiku poem about their favorite animal.\",\n    processed_content: nil,\n    index: nil,\n    status: :complete,\n    role: :user,\n    name: nil,\n    tool_calls: [],\n    tool_results: nil\n  }\n]\n```","ref":"custom_functions.html#defining-our-ai-assistant"},{"type":"extras","title":"Defining our AI Model - Executing Custom Elixir Functions","doc":"For this example, we're talking to OpenAI's ChatGPT service. Let's setup that model. At this point, we can also specify which version of ChatGPT we want to talk with.\n\nFor the kind of work we're asking it to do, GPT-4o is the latest model that has great support for tools.\n\n```elixir\nalias LangChain.ChatModels.ChatOpenAI\n\nchat_model = ChatOpenAI.new!(%{model: \"gpt-4o\", temperature: 1, stream: false})\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n%LangChain.ChatModels.ChatOpenAI{\n  endpoint: \"https://api.openai.com/v1/chat/completions\",\n  model: \"gpt-4o\",\n  api_key: nil,\n  temperature: 1.0,\n  frequency_penalty: 0.0,\n  reasoning_mode: false,\n  reasoning_effort: \"medium\",\n  receive_timeout: 60000,\n  seed: nil,\n  n: 1,\n  json_response: false,\n  json_schema: nil,\n  stream: false,\n  max_tokens: nil,\n  stream_options: nil,\n  tool_choice: nil,\n  callbacks: [],\n  user: nil\n}\n```","ref":"custom_functions.html#defining-our-ai-model"},{"type":"extras","title":"Defining our Application's User Context - Executing Custom Elixir Functions","doc":"Here we'll define some special context that we want passed through to our `LangChain.Function` when it is executed.\n\nIn a real application, this might be session based user or account information. It's whatever is relevant to our application that changes how a function should operate or the data it should access.\n\n```elixir\ncontext = %{user_id: 2}\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n%{user_id: 2}\n```\n\nAfter trying this with `user_id: 2`, a member who should have a Haiku generated for them, change it to `user_id: 1` to see it be politely denied.","ref":"custom_functions.html#defining-our-application-s-user-context"},{"type":"extras","title":"Making the API Call - Executing Custom Elixir Functions","doc":"We're ready to make the API call!\n\nNotice the `custom_context: context` setting that is passed in when creating the `LLMChain`. That information is the application-specific context we want to be passed to our `Function` when executed.\n\nAlso, note the `verbose: true` setting. That causes a number of `IO.inspect` calls to be printed showing what's happening internally.\n\nAdditionally, the `stream: false` option says we want the result only when it's complete. This example isn't setup for receiving a streaming response. We're keeping it simple!\n\n```elixir\nalias LangChain.Chains.LLMChain\n\n{:ok, updated_chain} =\n  %{llm: chat_model, custom_context: context, verbose: true}\n  |> LLMChain.new!()\n  |> LLMChain.add_messages(messages)\n  |> LLMChain.add_tools([function])\n  # keep running the LLM chain against the LLM if needed to evaluate\n  # function calls and provide a response.\n  |> LLMChain.run(mode: :while_needs_response)\n\nresponse = updated_chain.last_message\nIO.write(response.content)\nresponse.content\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\nLLM: %LangChain.ChatModels.ChatOpenAI{\n  endpoint: \"https://api.openai.com/v1/chat/completions\",\n  model: \"gpt-4o\",\n  api_key: nil,\n  temperature: 1.0,\n  frequency_penalty: 0.0,\n  reasoning_mode: false,\n  reasoning_effort: \"medium\",\n  receive_timeout: 60000,\n  seed: nil,\n  n: 1,\n  json_response: false,\n  json_schema: nil,\n  stream: false,\n  max_tokens: nil,\n  stream_options: nil,\n  tool_choice: nil,\n  callbacks: [],\n  user: nil\n}\nMESSAGES: [\n  %LangChain.Message{\n    content: \"You are a helpful haiku poem generating assistant.\\n    ONLY generate a haiku for users with an `account_type` of \\\"member\\\".\\n    If the user has an `account_type` of \\\"trial\\\", say you can't do it,\\n    but you would love to help them if they upgrade and become a member.\",\n    processed_content: nil,\n    index: nil,\n    status: :complete,\n    role: :system,\n    name: nil,\n    tool_calls: [],\n    tool_results: nil\n  },\n  %LangChain.Message{\n    content: \"The current user is requesting a Haiku poem about their favorite animal.\",\n    processed_content: nil,\n    index: nil,\n    status: :complete,\n    role: :user,\n    name: nil,\n    tool_calls: [],\n    tool_results: nil\n  }\n]\nTOOLS: [\n  %LangChain.Function{\n    name: \"get_user_info\",\n    description: \"Return JSON object of the current users's relevant information.\",\n    display_text: nil,\n    strict: false,\n    function: #Function<41.18682967/2 in :erl_eval.expr/6>,\n    async: true,\n    parameters_schema: nil,\n    parameters: []\n  }\n]\nSINGLE MESSAGE RESPONSE: %LangChain.Message{\n  content: nil,\n  processed_content: nil,\n  index: 0,\n  status: :complete,\n  role: :assistant,\n  name: nil,\n  tool_calls: [\n    %LangChain.Message.ToolCall{\n      status: :complete,\n      type: :function,\n      call_id: \"call_2puDjavxfbFQ7uDvrE9vafnZ\",\n      name: \"get_user_info\",\n      arguments: %{},\n      index: nil\n    }\n  ],\n  tool_results: nil\n}\nMESSAGE PROCESSED: %LangChain.Message{\n  content: nil,\n  processed_content: nil,\n  index: 0,\n  status: :complete,\n  role: :assistant,\n  name: nil,\n  tool_calls: [\n    %LangChain.Message.ToolCall{\n      status: :complete,\n      type: :function,\n      call_id: \"call_2puDjavxfbFQ7uDvrE9vafnZ\",\n      name: \"get_user_info\",\n      arguments: %{},\n      index: nil\n    }\n  ],\n  tool_results: nil\n}\nEXECUTING FUNCTION: \"get_user_info\"\n\n09:10:30.474 [debug] Executing function \"get_user_info\"\nFUNCTION RESULT: \"{\\\"name\\\":\\\"Joan Jett\\\",\\\"user_id\\\":2,\\\"account_type\\\":\\\"member\\\",\\\"favorite_animal\\\":\\\"Aardvark\\\"}\"\nTOOL RESULTS: %LangChain.Message{\n  content: nil,\n  processed_content: nil,\n  index: nil,\n  status: :complete,\n  role: :tool,\n  name: nil,\n  tool_calls: [],\n  tool_results: [\n    %LangChain.Message.ToolResult{\n      type: :function,\n      tool_call_id: \"call_2puDjavxfbFQ7uDvrE9vafnZ\",\n      name: \"get_user_info\",\n      content: \"{\\\"name\\\":\\\"Joan Jett\\\",\\\"user_id\\\":2,\\\"account_type\\\":\\\"member\\\",\\\"favorite_animal\\\":\\\"Aardvark\\\"}\",\n      processed_content: nil,\n      display_text: nil,\n      is_error: false\n    }\n  ]\n}\nSINGLE MESSAGE RESPONSE: %LangChain.Message{\n  content: \"Long snout seeking ants,  \\nNight wanderer of moonlight,  \\nAardvark's gentle grace.\",\n  processed_content: nil,\n  index: 0,\n  status: :complete,\n  role: :assistant,\n  name: nil,\n  tool_calls: [],\n  tool_results: nil\n}\nMESSAGE PROCESSED: %LangChain.Message{\n  content: \"Long snout seeking ants,  \\nNight wanderer of moonlight,  \\nAardvark's gentle grace.\",\n  processed_content: nil,\n  index: 0,\n  status: :complete,\n  role: :assistant,\n  name: nil,\n  tool_calls: [],\n  tool_results: nil\n}\nLong snout seeking ants,  \nNight wanderer of moonlight,  \nAardvark's gentle grace.\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n\"Long snout seeking ants,  \\nNight wanderer of moonlight,  \\nAardvark's gentle grace.\"\n```\n\n**TIP:** Try changing the `context` to `user_id: 1` now and see what happens when a different user context is provided.","ref":"custom_functions.html#making-the-api-call"},{"type":"extras","title":"Discussion - Executing Custom Elixir Functions","doc":"After a successful call, we can see in the verbose logs that:\n\n* the LLM requested to execute the tool which is our function\n* LLMChain executed the Elixir function attached to the `Function` struct\n* the response of our Elixir function passed through the anonymous function on `Function` and was re-submitted back to the LLM\n* the LLM reacted to the result of our function call\n\nThis means it worked! We successfully let an LLM directly interact with our Elixir application!\n\nWith this, we could expose functions that allow the LLM to request additional information specific to the current user, or we could even define functions that allow the LLM to **change** things in the user's account on their behalf!\n\nThe rest is up to us.\n\n<!-- livebook:{\"offset\":13867,\"stamp\":{\"token\":\"XCP.UFYBKiTKTivrWqsgBsbdfATOyTtvmvReV-H7dpctAL0z8OepBUJOuQTNOu2-7E33qPbB-Jeyp-I32rgPfwdwE98Ed2JQRodZ3PL1Pn_lWFzuVrqSKbEGO8yYAUxoqCC8kpGjDeoKqIardsK0OtpE\",\"version\":2}} -->","ref":"custom_functions.html#discussion"},{"type":"extras","title":"Images: Generating context-specific descriptions","doc":"<!-- livebook:{\"file_entries\":[{\"name\":\"image1.jpg\",\"type\":\"attachment\"},{\"name\":\"image2.jpg\",\"type\":\"attachment\"}]} -->\n\n# Images: Generating context-specific descriptions\n\n```elixir\nMix.install([\n  {:langchain, \"~> 0.3.0\"},\n  {:kino, \"~> 0.12.0\"}\n])\n```","ref":"context-specific-image-descriptions.html"},{"type":"extras","title":"Overview - Images: Generating context-specific descriptions","doc":"For the AI, we will leverage both [OpenAI's ChatGPT](https://openai.com/api/) and [Anthropic's Claude](https://www.anthropic.com/api) LLMs (Large Language Models) through the [Elixir LangChain library](https://github.com/brainlid/langchain) to perform AI analysis on images. In particular, we want context-aware caption text and shorter context-aware image ALT text. \"Context-aware\" means that the description should relate to how the image is being used and not simply a description of the image. With these two versions of an image description, we can improve SEO (Search Engine Optimization) and improve the accessibility of our content as well.","ref":"context-specific-image-descriptions.html#overview"},{"type":"extras","title":"It starts with an image - Images: Generating context-specific descriptions","doc":"Before we can interact with an LLM, we need an image to work with. There's an image input to make it easy to upload an image.\n\nYou can take a photo or upload an image to use. This notebook example was based on this image: [unsplash.com/photos/two-woman-walking-under-bridge-DcNlJK7kLkk](https://unsplash.com/photos/two-woman-walking-under-bridge-DcNlJK7kLkk)","ref":"context-specific-image-descriptions.html#it-starts-with-an-image"},{"type":"extras","title":"Scenario - Images: Generating context-specific descriptions","doc":"In this example, Sarah is working on a feature article for an online magazine focused on \"The Hidden Gems of Urban Street Art.\" Her article aims to showcase the vibrant and often overlooked artworks that adorn the nooks and crannies around Toronto Canada.\n\nThe article should also be accessible to readers who use assistive technology like screenreaders. There are a number of images for the article and we want to generate high quality ALT text or caption text for the images and we want it written with awareness of the article and the context for witch the text will be used.\n\nWe're developing a solution for this problem in our notebook, but our solution is able to be scaled up to operated on hundreds of images if needed.\n\nLet's get started by getting the image into the notebook.\n\n```elixir\ninput = Kino.Input.image(\"image\", format: :jpeg)\n```\n\n```elixir\nimage = Kino.Input.read(input)\n```\n\nThe LLMs we're working with expect the image to be supplied as `base64` encoded text. Let's get that ready here:\n\n```elixir\nimage_data =\n  image.file_ref\n  |> Kino.Input.file_path()\n  |> File.read!()\n  |> Base.encode64()\n\n:ok\n```","ref":"context-specific-image-descriptions.html#scenario"},{"type":"extras","title":"Elixir LangChain - Images: Generating context-specific descriptions","doc":"To make the requests for AI help, we'll use the [Elixir LangChain library](https://github.com/brainlid/langchain). It makes it much easier to integrate AI into our Elixir applications.\n\nWith our image ready, we'll setup a request to ChatGPT and ask it to write a context aware description of the image.\n\n**NOTE:** You must provide your own OPENAI_API_KEY in the Livebook secrets to do this.\n\n```elixir\nApplication.put_env(:langchain, :openai_key, System.fetch_env!(\"LB_OPENAI_API_KEY\"))\n```\n\nFirst, let's setup the model for talking to ChatGPT. For simplicity, we've set it not stream the response back. We'll get the final analysis once it's complete.\n\n**NOTE:** For ChatGPT, image support requires using the `\"gpt-4o\"` model at the time this was created.\n\n```elixir\nalias LangChain.ChatModels.ChatOpenAI\n\nopenai_chat_model = ChatOpenAI.new!(%{model: \"gpt-4o\"})\n```\n\nHere we setup our messages. The user message contains multiple `ContentPart`s. One is text for our prompt and the secone is the `base64` encoded image data. We submit all of this data together in our request.\n\nThis is where we add **context** to our image description request. We'll assume that we have programmatic access to the images and that we also have access to some data about the image from an external system.\n\n**TIP:** To get consistency across all the images in our set, we'll have better success by providing an example of the type of output we want.\n\n**NOTE:** Make sure the `:media` option matches both the image and what is supported by the LLM you are connecting with.\n\n````elixir\nalias LangChain.Message\nalias LangChain.Message.ContentPart\nalias LangChain.PromptTemplate\n\nmessages = [\n  Message.new_system!(\"\"\"\n  You are an expert at providing an image description for assistive technology and SEO benefits.\n\n  The image is included in an online article titled \"The Hidden Gems of Urban Street Art.\"\n\n  The article aims to showcase the vibrant and often overlooked artworks that adorn\n  the nooks and crannies around the city of Toronto Canada.\n\n  You generate text for two purposes:\n  - an HTML img alt text\n  - an HTML figure, figcaption text","ref":"context-specific-image-descriptions.html#elixir-langchain"},{"type":"extras","title":"Alt text format - Images: Generating context-specific descriptions","doc":"Briefly describe the contents of the image where the context is focusing on the urban street art.\n  Be concise and limit the description to 125 characters or less.\n\n  Example alt text:\n  > A vibrant phoenix graffiti with blazing orange, red, and gold colors on the side of a brick building in an urban setting.","ref":"context-specific-image-descriptions.html#alt-text-format"},{"type":"extras","title":"figcaption format - Images: Generating context-specific descriptions","doc":"Image caption descriptions should focus on the urban artwork, providing a description of the appearance,\n  style, street address if available, and how it relates to the surroundings. Be concise.\n\n  Example caption text:\n  > A vibrant phoenix graffiti on a brick building at Queen St W and Spadina Ave. With wings outstretched, the mural's blazing oranges, reds, and golds contrast sharply against the red brick backdrop. Passersby pause to observe, integrating the artwork into the urban landscape.\n  \"\"\"),\n  Message.new_user!([\n    PromptTemplate.from_template!(\"\"\"\n    Provide the descriptions for the image. Incorporate relevant information from the following additional details if applicable:\n\n    <%= @extra_image_info %>\n\n    Output in the following format:\n\n    ```json\n    {\n      \"alt\": \"generated alt text\",\n      \"caption\": \"generation caption text\"\n    }\n    ```\n    \"\"\"),\n    ContentPart.image!(image_data, media: :jpg, detail: \"low\")\n  ])\n]\n````\n\nBefore we continue, notice that the System message provides the general context for what we are doing and what we want from the LLM.\n\nThe User message is made up of two parts:\n\n* `PromptTemplate`: supports variable replacement tags using EEx templates. This allows us to easily customize the prompt for each image as we process through a whole batch. This turns into a `ContentPart`.\n* `ContentPart`: Makes it easy for us to provide our image directly to the LLM.\n\nWe are providing the LLM with the context for the task, specific instructions about an image, and an image to analyze with a \"vision\" enabled model so it can finally perform the task for us.\n\n<!-- livebook:{\"break_markdown\":true} -->\n\nNote the use of a `PromptTemplate` and `<%= @image_data %>` in our user Message. As we are processing a whole set of images, data from the system where we get the image is rendered into our prompt, helping to further customize the generated text from the LLM, making it far more specific and relevant.","ref":"context-specific-image-descriptions.html#figcaption-format"},{"type":"extras","title":"JSON Output - Images: Generating context-specific descriptions","doc":"For each image we want two pieces of generated content. To make this easy on ourselves, we instruct the LLM to output the two pieces of information in a JSON object.\n\nSpecifically, we instruct it to output in the following format:\n\n```json\n{\n  \"alt\": \"generated alt text\",\n  \"caption\": \"generation caption text\"\n}\n```\n\nTo make working with that output easier, we'll use a `JsonProcessor` for processing messages from the LLM. It has the added ability to return a JSON formatting error to the LLM in the case when it gets it wrong.\n\nWe'll see this next when we put it all together.","ref":"context-specific-image-descriptions.html#json-output"},{"type":"extras","title":"Making the Request - Images: Generating context-specific descriptions","doc":"Everything is ready to make the request!\n\n* We have the image\n* We setup which LLM we are connecting with\n* We provide context in our prompt and instructions for the type of description we want\n\nNow, we'll submit the request to the server and review the response. For this example, the \"image_data_from_other_system\" is a substitute for a database call or other lookup for the information we have on the image.\n\n````elixir\nalias LangChain.Chains.LLMChain\nalias LangChain.MessageProcessors.JsonProcessor\n\n# This data comes from an external data source per image.\n# When we `apply_prompt_templates` below, the data is rendered into the template.\nimage_data_from_other_system = \"image of urban art mural on underpass at 507 King St E\"\n\n{:ok, updated_chain} =\n  %{llm: openai_chat_model, verbose: true}\n  |> LLMChain.new!()\n  |> LLMChain.apply_prompt_templates(messages, %{extra_image_info: image_data_from_other_system})\n  |> LLMChain.message_processors([JsonProcessor.new!(~r/```json(.*?)```/s)])\n  |> LLMChain.run(mode: :until_success)\n\nupdated_chain.last_message.processed_content\n````\n\nNotice that when running the chain, we use the option `mode: :until_success`. Some LLMs are better are generating valid JSON than other LLMs. When we included the `JsonProcessor`, it parses the assistant's content, converting it into an Elixir map. The converted data is stored on the `message.processed_content`.\n\nIf the LLM fails to give us valid JSON, the `JsonProcessor` generates a `:user` message reporting the issue for the LLM. The `:until_success` mode will repeatedly make the round-trip requests to the LLM, giving it an opportunity to fix it. But don't worry, it won't run forever! The LLMChain's `max_retry_count` gives up after X failures, the default being 3.\n\nHere's a sample of what was generated when I ran it:\n\n<!-- livebook:{\"force_markdown\":true} -->\n\n```elixir\n%{\n  \"alt\" => \"Colorful mural of a face under a bridge at 507 King St E\",\n  \"caption\" => \"A captivating face mural under the 507 King St E underpass, featuring vivid hues and expressive eyes. The art adds a pop of color to the urban landscape, drawing the attention of pedestrians as they pass by.\"\n}\n```\n\nThis demonstrates how adding context to an image description request can really become immediately useful. The same system prompt and message template can be used for the whole set of images and generate high quality descriptive text for images.\n\nWill it be as good as a human? No, honestly it may not be. However, it can be done much faster, automated and performed on more images than a human could do. In many instances, that means we get good, context aware ALT text where we may otherwise have none because we don't have the people nor the time to manually create it.\n\nThe result is our visual content is more accessible to people using assistive technology like a screen reader.","ref":"context-specific-image-descriptions.html#making-the-request"},{"type":"extras","title":"Anthropic (BONUS) - Images: Generating context-specific descriptions","doc":"While we're all setup for it, if you have an Anthropic API key, then we'll submit the same request to Anthropic and see how that compares.\n\n**NOTE:** You must provide your own ANTHROPIC_API_KEY in the Livebook secrets to do this.\n\n```elixir\nApplication.put_env(:langchain, :anthropic_key, System.fetch_env!(\"LB_ANTHROPIC_API_KEY\"))\n```\n\nLet's setup our Anthropic chat model.\n\n**NOTE:** Keep in mind that different versions of Claude will give different results. You can play with that to find a good cost/accuracy for your specific need.\n\n```elixir\nalias LangChain.ChatModels.ChatAnthropic\n\nanthropic_chat_model = ChatAnthropic.new!(%{model: \"claude-3-5-sonnet-latest\"})\n```\n\nNow we run the same messages through an identical LLMChain but passing in the Anthropic chat model.\n\n````elixir\nalias LangChain.Chains.LLMChain\nalias LangChain.MessageProcessors.JsonProcessor\n\n# This data comes from an external data source per image.\n# When we `apply_prompt_templates` below, the data is rendered into the template.\nimage_data_from_other_system = \"image of urban art mural on underpass at 507 King St E\"\n\n{:ok, updated_chain} =\n  %{llm: anthropic_chat_model, verbose: true}\n  |> LLMChain.new!()\n  |> LLMChain.apply_prompt_templates(messages, %{extra_image_info: image_data_from_other_system})\n  |> LLMChain.message_processors([JsonProcessor.new!(~r/```json(.*?)```/s)])\n  |> LLMChain.run(mode: :until_success)\n\nupdated_chain.last_message.processed_content\n````\n\nNice! The Elixir LangChain library abstracted away the differences between the two services. With no code changes, we can make a similar request about the image from Anthropic's Claude LLM as well!\n\nHere's what I got from it:\n\n<!-- livebook:{\"force_markdown\":true} -->\n\n```elixir\n%{\n  \"alt\" => \"Colorful street art mural with a face in vibrant colors on an underpass pillar in an urban setting.\",\n  \"caption\" => \"A striking urban art mural adorns an underpass pillar at 507 King St E. The artwork features a mesmerizing face composed of vivid, rainbow-like hues and intricate patterns. Framed by the industrial yellow beams above, the mural transforms the concrete structure into a captivating focal point for passersby.\"\n}\n```\n\nWe would want to run multiple tests on a small sampling of images and tweak our prompt until we are happy with the result. Then, we can process full batch and save our work as a template for future projects as well.\n\n<!-- livebook:{\"offset\":12833,\"stamp\":{\"token\":\"XCP.ima-5H7hVmSu3ukHhzotNc1v6kxQNfrdVgWq9fQbqecYia8FBsPHDdAnj9Kk-bZbTOj0Dm6xuwH0qQAJHp4aKwWZFcWb0V_5oRVwaD8NBLyAAq5Ih1RC1ksRmBMCJ8jH7B7tRfksKMMvbicRcpv-\",\"version\":2}} -->","ref":"context-specific-image-descriptions.html#anthropic-bonus"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,49]}}